Package: flowGraph
Type: Package
Title: Identifying differential cell populations in flow cytometry data
        accounting for marker frequency
Version: 1.21.0
Description: Identifies maximal differential cell populations in flow cytometry data taking into account dependencies between cell populations; flowGraph calculates and plots SpecEnr abundance scores given cell population cell counts.
Date: 2019-11-30
Authors@R: person("Alice", "Yue", email = "aya43@sfu.ca", role = c("aut", "cre"))
License: Artistic-2.0
VignetteBuilder: knitr
Depends: R (>= 4.1)
Imports: effsize, furrr, future, purrr, ggiraph, ggrepel, ggplot2,
        igraph, Matrix, matrixStats, stats, utils, visNetwork,
        htmlwidgets, grDevices, methods, stringr, stringi, Rdpack,
        data.table (>= 1.9.5), gridExtra,
Suggests: BiocStyle, dplyr, knitr, rmarkdown, testthat (>= 2.1.0)
biocViews: FlowCytometry, StatisticalMethod, ImmunoOncology, Software,
        CellBasedAssays, Visualization
BugReports: https://github.com/aya49/flowGraph/issues
URL: https://github.com/aya49/flowGraph
RoxygenNote: 7.1.1
RdMacros: Rdpack
LazyData: no
git_url: https://git.bioconductor.org/packages/flowGraph
git_branch: devel
git_last_commit: 04c5027
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 22:04:26 UTC; biocbuild
Author: Alice Yue [aut, cre]
Maintainer: Alice Yue <aya43@sfu.ca>
Built: R 4.6.0; ; 2026-05-19 04:51:37 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cell_type_layers)
export(fg_add_feature)
export(fg_add_summary)
export(fg_clear_features)
export(fg_clear_summary)
export(fg_extract_phenotypes)
export(fg_extract_raw)
export(fg_extract_samples)
export(fg_feat_edge_prop)
export(fg_feat_edge_specenr)
export(fg_feat_mean_class)
export(fg_feat_node_prop)
export(fg_feat_node_specenr)
export(fg_get_feature)
export(fg_get_feature_desc)
export(fg_get_feature_means)
export(fg_get_graph)
export(fg_get_markers)
export(fg_get_meta)
export(fg_get_summary)
export(fg_get_summary_desc)
export(fg_get_summary_index)
export(fg_get_summary_tables)
export(fg_gsub_ids)
export(fg_gsub_markers)
export(fg_load)
export(fg_merge)
export(fg_merge_samples)
export(fg_plot)
export(fg_plot_box)
export(fg_plot_pVSdiff)
export(fg_plot_qq)
export(fg_replace_meta)
export(fg_rm_feature)
export(fg_rm_summary)
export(fg_save)
export(fg_save_plots)
export(fg_set_layout)
export(fg_summary)
export(flowGraph)
export(flowGraphSubset)
export(flowGraphSubset_summary_adjust)
export(flowGraphSubset_summary_pars)
export(get_phen_list)
export(get_phen_meta)
export(ggdf)
export(plot_gr)
export(set_layout_graph)
export(summary_table)
exportClasses(flowGraph)
exportMethods(show)
import(igraph)
importFrom(Matrix,Matrix)
importFrom(Rdpack,reprompt)
importFrom(data.table,".I")
importFrom(data.table,as.data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,setattr)
importFrom(effsize,cohen.d)
importFrom(furrr,future_map)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(ggiraph,geom_boxplot_interactive)
importFrom(ggiraph,geom_point_interactive)
importFrom(ggiraph,geom_segment_interactive)
importFrom(ggiraph,girafe)
importFrom(ggplot2,aes_)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_dotplot)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,boxplot.stats)
importFrom(gridExtra,arrangeGrob)
importFrom(htmlwidgets,saveWidget)
importFrom(igraph,V)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout.reingold.tilford)
importFrom(matrixStats,rowMaxs)
importFrom(matrixStats,rowMins)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(purrr,compact)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_dfr)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(stats,chisq.test)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,t.test)
importFrom(stats,var.test)
importFrom(stats,wilcox.test)
importFrom(stringi,stri_rand_strings)
importFrom(stringr,str_count)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_pad)
importFrom(stringr,str_split)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(visNetwork,visEdges)
importFrom(visNetwork,visEvents)
importFrom(visNetwork,visHierarchicalLayout)
importFrom(visNetwork,visInteraction)
importFrom(visNetwork,visNetwork)
importFrom(visNetwork,visNodes)
importFrom(visNetwork,visOptions)
importFrom(visNetwork,visSave)

# flowGraph

[![DOI](https://zenodo.org/badge/DOI/10.1101/837765.svg)](https://doi.org/10.1101/837765)

[Summary blog post](https://aya49.github.io/2020/09/30/flowGraph/)

flowGraph is an R package used to identify candidate biomarkers for disease diagnosis in flow cytometry data. It does so by identifying driver cell populations whose abundance changes significantly and independently given a disease.

flowGraph takes cell counts as input and outputs SpecEnr values for each cell population within a flow cytometry sample, based on their expected proportion. SpecEnr accounts for dependencies between cell populations such that we can use it to flag only cell populations whose abundance change is incurred wholly or in part because of its association with a sample class (e.g. healthy vs sick).

## Citation

The theory, proof, and algorithm behind the SpecEnr statistic used in the flowGraph package can be found in the following [paper](https://doi.org/10.1002/cyto.a.24503). Please consider citing if you found it helpful.

bibtex:
```
@article{yue2022automated,
  title={Automated identification of maximal differential cell populations in flow cytometry data},
  author={Yue, Alice and Chauve, Cedric and Libbrecht, Maxwell W and Brinkman, Ryan R},
  journal={Cytometry Part A},
  volume={101},
  number={2},
  pages={177--184},
  year={2022},
  publisher={Wiley Online Library}
}
```

The scripts and data from the paper can be downloaded on [Zenodo](https://zenodo.org/record/3991166).


## Installation

**flowGraph** can be installed via Bioconductor.

You can also intall the development version directly from Github using BiocManager:

```{r}
if (!require("BiocManager")) install.packages('BiocManager') 
BiocManager::install("aya49/flowGraph")
```

This package has been tested on R version 3.5+. If you have R version 3.5 - 4.0, download this package as a zip file, go to the `DESCRIPTION` file, change the R version requirement, and install the package locally.

## Usage

See our [vignette](vignettes/flowGraph.Rmd) for different use cases of the package: generating features, calculating summary statistics, visualizing results, and data specific use cases.

flowGraph takes as input a (sample x cell population) raw cell count matrix.

flowGraph gives as output a flowGraph object that contains the SpecEnr abundance statistic and optionally its q-values. The flowGraph object can then be used to calculate more statistics, plot and visualize q-values on a cell hierarchy, etc.

```{r}
# load the package
library("flowGraph)
```

PLEASE DO NOT MAKE CHANGES TO THIS FILE.
Last saved: 2021-02-17 12:17:06

This folder contains files in a flowGraph object created by the flowGraph package.

For a given flow cytometry sample and its cell populations, flowGraph generates SpecEnr values for each cell population or immunophenotype of based their expected proportion.
By doing so, flowGraph accounts for the relation between cell populations and flags truly enriched cell populations rather than those with induced changes.

The Summary_statistics folder contains p-values obtained from features associated with each immunophenotype or/ relation between immunophenotypes. The plots folder contains plots to help interpret those p-values.

The Features folder contains features for:
- 20 flow cytometry samples.
- 4 markers: A, B, C, D
- 81 cell population nodes (nodes for short) and 216 edges on 4 layers.

Cell hierarchy plots: By default, for SpecEnr features, we generate two cell_hierarchy plots. The original one is where the colours represent difference between mean SpecEnr values across sample classes. We also add one where the colours represent difference betwee mean original values across sample classes. Original here is usually proportion: the feature used to create SpecEnr. Note that if a node is coloured lightly on the second plot, then the difference is very small, meaning the SpecEnr value may become sporadic. Therefore, when analyzing the plots, for most cases we recommend looking at most important cell populations as the ones with large difference in both plots.

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----eval=FALSE, label=library1, warning=FALSE, message=FALSE, echo=TRUE, fig=FALSE, results='hide'----
# if (!require("BiocManager")) install.packages('BiocManager')
# BiocManager::install("aya49/flowGraph")

## ----label=library2, warning=FALSE, message=FALSE, echo=TRUE, fig=FALSE, message=FALSE----
library(flowGraph)

## ----message=FALSE------------------------------------------------------------
no_cores <- 1
data(fg_data_pos2)

meta_cell <- get_phen_meta(colnames(fg_data_pos2$count))
suppressWarnings({ pccell <- flowGraph:::get_phen_list(meta_cell, no_cores) })
gr <- set_layout_graph(list(e=pccell$edf, v=meta_cell)) # layout cell hierarchy

gr <- ggdf(gr)
gr$v$colour <- ifelse(!grepl("[-]",gr$v$phenotype), 1,0)
                     # "nodes with only marker conditions", "other nodes")
gr$v$label <- gr$v$phenotype
gr$v$v_ind <- gr$v$label_ind <- TRUE
gr$e$e_ind <- !grepl("[-]",gr$e$from) & !grepl("[-]",gr$e$to)

## ----fig.wide=TRUE------------------------------------------------------------
knitr::opts_template$set(figure1=list(fig.height=9, fig.width=9))
plot_gr(gr, main="Example cell hierarchy with markers A, B, C, D")

## ----eval=FALSE---------------------------------------------------------------
# no_cores <- 1 # number of cores to parallelize on.
# data(fg_data_pos2)
# fg <- flowGraph(
#     fg_data_pos2$count, # sample x cell population matrix
#     meta=fg_data_pos2$meta, # a data frame with each sample's meta data
#     path="flowGraph_example", # a directory for flowGraph to output results to
#     no_cores=no_cores) # number of cores to use, typically 1 for small data

## ----eval=FALSE---------------------------------------------------------------
# fg <- fg_load("flowGraph_example")

## ----label=Load_Data, fig=FALSE, message=FALSE--------------------------------
# data(fg_data_fca)
data(fg_data_pos2)
# data(fg_data_pos30)

# ?fg_data_fca
# ?fg_data_pos2
# ?fg_data_pos30

## ----message=FALSE------------------------------------------------------------
# no_cores <- 1 # number of cores to parallelize on.
data(fg_data_pos2)
fg <- flowGraph(fg_data_pos2$count, meta=fg_data_pos2$meta, no_cores=no_cores)


## -----------------------------------------------------------------------------
meta <- fg_get_meta(fg)
head(meta)

mcount <- fg_get_feature(fg, "node", "count")
head(rownames(mcount))

## -----------------------------------------------------------------------------
# get feature descriptions
fg_get_summary_desc(fg)
# get a summary statistic
fg_sum <- fg_get_summary(fg, type="node", summary_meta=list(
    node_feature="SpecEnr", 
    test_name="t_diminish", 
    class="class", 
    labels=c("exp","control"))
)
# fg_sum <- fg_get_summary(fg, type="node", index=1) # same as above

# list most significant cell populations
p <- fg_sum$values # p values
head(sort(p),30)

## ----eval=FALSE---------------------------------------------------------------
# fg_save(fg, "path/to/user/specified/folder/directory") # save flowGraph object
# fg <- fg_load("path/to/user/specified/folder/directory") # load flowGraph object

## -----------------------------------------------------------------------------
show(fg)

## -----------------------------------------------------------------------------
# get sample meta data
head(fg_get_meta(fg))

# modify sample meta data
meta_new <- fg_get_meta(fg)
meta_new$id[1] <- "new_sample_id1"
fg <- fg_replace_meta(fg, meta_new)

## -----------------------------------------------------------------------------
# get cell population meta data
gr <- fg_get_graph(fg)
head(gr$v)
head(gr$e)

## -----------------------------------------------------------------------------
# get feature descriptions
fg_get_feature_desc(fg)
# get count node feature
mc <- fg_get_feature(fg, type="node", feature="count")
dim(mc)

## -----------------------------------------------------------------------------
# add a new feature; 
# input matrix must contain the same row and column names as existing features;
# we recommend users stick with default feature generation methods 
# that start with fg_feat_
fg <- fg_add_feature(fg, type="node", feature="count_copy", m=mc)
fg_get_feature_desc(fg)

## -----------------------------------------------------------------------------
# remove a feature; note, the count node feature cannot be removed
fg <- fg_rm_feature(fg, type="node", feature="count_copy")
fg_get_feature_desc(fg)

## -----------------------------------------------------------------------------
fg_get_summary_desc(fg)

# calculate summary statistic
fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                 node_features="count", edge_features="NONE",
                 overwrite=FALSE, test_name="t", diminish=FALSE)
fg_get_summary_desc(fg)

## -----------------------------------------------------------------------------
# get a summary statistic
fg_sum1 <- fg_get_summary(fg, type="node",  summary_meta=list(
    feature="count", test_name="t", 
    class="class", label1="control", label2="exp"))
names(fg_sum1)

## -----------------------------------------------------------------------------
# remove a summary statistic
fg <- fg_rm_summary(fg, type="node", summary_meta=list(
    feature="count", test_name="t", 
    class="class", label1="control", label2="exp"))
fg_get_summary_desc(fg)

## -----------------------------------------------------------------------------
# add a new feature summary; 
# input list must contain a 'values', 'id1', and 'id2' containing summary 
# statistic values and the sample id's compared;
# we recommend users stick with default feature generation method fg_summary
fg <- fg_add_summary(fg, type="node",  summary_meta=list(
    feature="SpecEnr", test_name="t_copy",
    class="class", label1="control", label2="exp"), p=fg_sum1)
fg_get_summary_desc(fg)

## -----------------------------------------------------------------------------
# plotting functions default to plotting node feature SpecEnr 
# labelled with mean expected/proportion (maximum 30 labels are shown for clarity)
# and only significant nodes based on the wilcox_byLayer_diminish summary statistic
# are shown.
# gr <- fg_plot(fg, p_thres=.01, show_bgedges=TRUE, # show background edges
#               node_feature="SpecEnr", edge_feature="prop",
#               test_name="t_diminish", label_max=30)
gr <- fg_plot(fg, index=1, p_thres=.01, show_bgedges=TRUE)
# plot_gr(gr)

## -----------------------------------------------------------------------------
# interactive version in beta
plot_gr(gr, interactive=TRUE)

## ----message=FALSE------------------------------------------------------------
data(fg_data_pos2)
fg1 <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class, 
                 no_cores=no_cores)

## -----------------------------------------------------------------------------
fg_get_summary_desc(fg)

fg_plot_qq(fg, type="node", index=1)
fg_plot_qq(fg, type="node", index=1, logged=TRUE)

# interactive version
fg_plot_qq(fg, type="node", index=1, interactive=TRUE)

## -----------------------------------------------------------------------------
fg_plot_box(fg, type="node", summary_meta=NULL, index=1, node_edge="A+")

## -----------------------------------------------------------------------------
fg_plot_pVSdiff(fg, type="node", summary_meta=NULL, index=1)

# interactive version
fg_plot_pVSdiff(fg, type="node", summary_meta=NULL, index=1, interactive=TRUE)

## -----------------------------------------------------------------------------
gr <- fg_get_graph(fg)
gr <- ggdf(gr)

gr$v$colour <- ifelse(!grepl("[-]",gr$v$phenotype), 1, 0)
                     # "nodes with only marker conditions", "other nodes")
gr$v$label <- gr$v$phenotype
gr$v$v_ind <- gr$v$label_ind <- TRUE
gr$e$e_ind <- !grepl("[-]",gr$e$from) & !grepl("[-]",gr$e$to)

plot_gr(gr, main="Example cell hierarchy with markers A, B, C, D")

## ----evaluate=FALSE-----------------------------------------------------------
data(fg_data_pos2)
fg <- flowGraphSubset(fg_data_pos2$count, meta=fg_data_pos2$meta, no_cores=no_cores,
                 summary_pars=flowGraphSubset_summary_pars(),
                 summary_adjust=flowGraphSubset_summary_adjust())

## -----------------------------------------------------------------------------
sessionInfo()

#' @title Generates the proportion node feature.
#' @description Generates the proportion node feature and returns it
#'  inside the returned flowGraph object.
#' @param fg flowGraph object.
#' @param overwrite A logical variable indicating whether to
#'  overwrite the existing proportion node feature if it exists.
#' @return flowGraph object containing the proportion node feature.
#' @details Given a flowGraph object, \code{fg_feat_node_prop} returns the same
#'  flowGraph object, inside of which is an additional proportions \code{prop}
#'  \code{node} feature
#'  and its meta data. The proportions feature is made using the node count
#'  feature and is the cell count of each cell population over the total
#'  cell count.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_feat_node_prop(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_feat_node_specenr}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#' @rdname fg_feat_node_prop
#' @export
fg_feat_node_prop <- function(fg, overwrite=FALSE) {
    start1 <- Sys.time()
    message("preparing feature(s): node proportion ")

    fg <- fg_add_feature(fg, type="node", feature="prop", overwrite=overwrite,
                         m=NULL, feat_fun=fg_feat_node_prop_)

    time_output(start1)
    return(fg)
}

fg_feat_node_prop_ <- function(fg) {
    mc <- fg_get_feature(fg, "node", "count")
    mp <- mc/mc[,colnames(mc)==""]
    dimnames(mp) <- dimnames(mc)
    return(mp)
}


#' @title Generates the proportion edge feature.
#' @description Generates the proportion edge feature and returns it
#'  inside the flowGraph object.
#' @param fg flowGraph object.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @param overwrite A logical variable indicating whether to
#'  overwrite the existing proportion edge feature if it exists.
#' @return flowGraph object containing the proportion edge feature.
#' @details Given a flowGraph object, \code{fg_feat_edge_prop} returns the same
#'  flowGraph object with an additional proportions \code{prop} \code{edge}
#'  feature and its meta data. The proportions feature is
#'  made using the node count
#'  feature and is the cell count of each cell population (e.g. A+B+)
#'  over the cell count of its parent (e.g. A+);
#'  each edge then corresponds with such a relationship.
#'  The edge feature matrix has column names <from>_<to> e.g. A+_A+B+.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_feat_edge_prop(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_feat_node_prop}}
#'  \code{\link[flowGraph]{fg_feat_node_specenr}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#' @rdname fg_feat_prop_edge
#' @export
fg_feat_edge_prop <- function(fg, no_cores=1, overwrite=FALSE) {
    start1 <- Sys.time()
    message("preparing feature(s): proportion on edges ")

    fg <- fg_add_feature(fg, type="edge", feature="prop", overwrite=overwrite,
                         m=NULL, feat_fun=fg_feat_edge_prop_, no_cores=no_cores)

    time_output(start1)
    return(fg)
}

#' @importFrom future plan multisession sequential
#' @importFrom furrr future_map
#' @importFrom purrr map
fg_feat_edge_prop_ <- function(fg, no_cores=1) {
    if (no_cores>1) future::plan(future::multisession)

    edf <- fg_get_graph(fg)$e
    mc <- fg_get_feature(fg, "node", "count")

    rooti <- which(colnames(mc)=="")

    childprop_ <- do.call(cbind, fpurrr_map(seq_len(nrow(edf)), function(i) {
        pname <- ifelse(edf$from[i]=="", rooti, edf$from[i])
        mc[,edf$to[i]]/mc[,pname]
    }, no_cores=no_cores, prll=nrow(edf)>1000))
    colnames(childprop_) <- paste0(edf$from,"__",edf$to)
    childprop_[is.nan(as.matrix(childprop_))] <- 0

    future::plan(future::sequential)
    return(childprop_)
}


#' @title Generates the SpecEnr edge feature.
#' @description Generates the SpecEnr edge feature and returns it
#'  inside the flowGraph object.
#' @param fg flowGraph object.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @param overwrite A logical variable indicating whether to
#'  overwrite the existing proportion edge feature if it exists.
#' @return flowGraph object containing the proportion edge feature.
#' @details Given a flowGraph object, \code{fg_feat_edge_SpecEnr} returns the same
#'  flowGraph object with an additional SpecEnr and expected proportions
#'  \code{expect_prop} \code{edge}
#'  feature and its meta data. The expected proportions edge feature is
#'  calculated by taking the ratio of the child nodes' (e.g. A+B+)
#'  expected proportion value over
#'  its parent nodes' (e.g. A+) actual proportion value.
#'  The SpecEnr feature is the actual over expected proportion ratio, logged.
#'  The edge feature matrix has column names <from>_<to> e.g. A+_A+B+.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_feat_edge_specenr(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_feat_node_prop}}
#'  \code{\link[flowGraph]{fg_feat_node_specenr}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#' @rdname fg_feat_edge_specenr
#' @export
fg_feat_edge_specenr <- function(fg, no_cores=1, overwrite=FALSE) {
    start1 <- Sys.time()
    message("preparing feature(s): expected proportion on edges ")

    # mp: sample x cell population, cell count matrix
    if (is.null(fg_get_feature_all(fg)$node$prop) | overwrite)
        fg <- fg_feat_node_prop(fg)

    # ep: sample x edge, proportion matrix i.e. if column name is A+B+C+_A+B+,
    # then the value is the count of A+B+C+ over A+B+.
    if (is.null(fg_get_feature_all(fg)$edge$prop) | overwrite)
        fg <- fg_feat_edge_prop(fg, no_cores=no_cores)

    # create expected features
    if (is.null(fg_get_feature_all(fg)$node$expect_prop) | overwrite)
        fg <- fg_add_feature(fg, type="node", feature="expect_prop",
                             overwrite=overwrite,
                             m=NULL, feat_fun=fg_feat_node_exprop_,
                             no_cores=no_cores)

    if (is.null(fg_get_feature_all(fg)$edge$expect_prop) | overwrite)
    fg <- fg_add_feature(fg, type="edge", feature="expect_prop",
                         overwrite=overwrite,
                         m=NULL, feat_fun=fg_feat_edge_exprop_, no_cores=no_cores)

    mp <- fg_get_feature(fg, "edge", "prop")
    mep <- fg_get_feature(fg, "edge", "expect_prop")

    a <- mp/mep
    aa <- as.matrix(a)
    a[is.infinite(aa)] <- max(a[is.finite(aa)])
    a[is.nan(aa)] <- 0
    e0 <- as.matrix(mep)==0
    suppressWarnings({ specenr1 <- log(a) })
    specenr1[is.nan(as.matrix(specenr1))] <- 0
    specenr1[e0] <- log(as.matrix(mp)[e0])
    specenr1[mp==0] <- 0

    fg <- fg_add_feature(
        fg, type="edge", feature="SpecEnr",
        m=specenr1, no_cores=no_cores, overwrite=overwrite)

    time_output(start1)
    return(fg)
}

#' @importFrom future plan multisession sequential
#' @importFrom furrr future_map
#' @importFrom purrr map
fg_feat_edge_exprop_ <- function(fg, no_cores=1) {
    if (no_cores>1) future:: plan(future::multisession)

    edf <- fg_get_graph(fg)$e
    mp <- fg_get_feature(fg, "node", "prop")
    mep <- fg_get_feature(fg, "node", "expect_prop")

    rooti <- which(colnames(mp)=="")

    childprop_ <- do.call(cbind, fpurrr_map(seq_len(nrow(edf)), function(i) {
        pname <- ifelse(edf$from[i]=="", rooti, edf$from[i])
        mep[,edf$to[i]]/mp[,pname]
    }, no_cores=no_cores, prll=nrow(edf)>1000))
    colnames(childprop_) <- paste0(edf$from,"__",edf$to)
    childprop_[is.nan(as.matrix(childprop_))] <- 0

    future::plan(future::sequential)
    return(childprop_)
}


#' @title Generates the SpecEnr node feature.
#' @description Generates the SpecEnr node feature and returns it
#'  inside the returned flowGraph object.
#' @param fg flowGraph object
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @param feature A string indicating feature name; this is the feature
#'  SpecEnr will be calculated on.
#' @param overwrite A logical variable indicating whether to
#'  overwrite the existing SpecEnr node feature if it exists.
#' @return flowGraph object containing the SpecEnr node feature.
#' @details Given a flowGraph object, \code{fg_feat_node_specenr}
#'  returns the same
#'  flowGraph object with an additional \code{SpecEnr} and \code{expect_prop}
#'  \code{node} feature and its meta data.
#'  The expected proportions feature is made using the \code{prop} \code{node}
#'  and \code{edge} features; therefore, the returned flowGraph will also
#'  contain these two features. For details on how these feature is calculated.
#'
#' @references
#'  \insertRef{yue2019identifying}{flowGraph}
#'
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  # SpecEnr is by default calculated based on proportions
#'  fg <- fg_feat_node_specenr(fg, no_cores=no_cores)
#'
#'  # SpecEnr can be calculated for other feature values too
#'  fg <- fg_feat_node_specenr(fg, feature="count")
#'
#'  show(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_feat_node_prop}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#' @rdname fg_feat_node_specenr
#' @export
#' @importFrom Rdpack reprompt
fg_feat_node_specenr <- function(fg,no_cores=1,feature="prop",overwrite=FALSE) {

    start1 <- Sys.time()
    message("preparing feature(s): SpecEnr ")

    fg_feat <- fg_get_feature_all(fg)

    # mp: sample x cell population, cell count matrix
    if (is.null(fg_feat$node$prop) | overwrite)
        fg <- fg_feat_node_prop(fg)

    # ep: sample x edge, proportion matrix i.e. if column name is A+B+C+_A+B+,
    # then the value is the count of A+B+C+ over A+B+.
    if (is.null(fg_feat$edge$prop) | overwrite)
        fg <- fg_feat_edge_prop(fg, no_cores=no_cores)

    # create expeced features
    if (is.null(fg_feat$node$expect_prop) | overwrite)
        fg <- fg_add_feature(fg, type="node", feature="expect_prop",
                             overwrite=overwrite,
                             m=NULL, feat_fun=fg_feat_node_exprop_,
                             no_cores=no_cores)

    if (is.null(fg_feat$node[[paste0("expect_",feature)]]) | overwrite)
        fg <- fg_add_feature(
            fg, type="node", feature=paste0("expect_",feature),
            m=fg_get_feature(fg, type="node", feature=feature)[,1] *
                fg_get_feature(fg, type="node", feature="expect_prop"))

    mp <- fg_get_feature(fg, "node", feature)
    exp1 <- fg_get_feature(fg, "node", paste0("expect_",feature))

    a <- mp/exp1 ### SPECENR ---------------------------
    aa <- as.matrix(a)
    a[is.infinite(aa)] <- max(a[is.finite(aa)])
    a[is.nan(aa)] <- 0
    e0 <- as.matrix(exp1)==0
    suppressWarnings({ specenr1 <- log(a) })
    specenr1[is.nan(as.matrix(specenr1))] <- 0
    specenr1[e0] <- log(as.matrix(mp)[e0])
    specenr1[mp==0] <- 0

    fg <- fg_add_feature(
        fg, type="node", feature=paste0( "SpecEnr", ifelse(
            feature=="prop", "", paste0("_",feature)) ),
        m=specenr1, no_cores=no_cores, overwrite=overwrite)

    time_output(start1)
    return(fg)
}

# expected proportion: this is the version currently used.
# this version is the same as the new version, but the new
# version is easier to experiment on and calculates everything at once.
#' @importFrom future plan multisession sequential
#' @importFrom stringr str_extract_all str_extract str_count
#' @importFrom furrr future_map
#' @importFrom purrr map
#' @importFrom stats median
fg_feat_node_exprop_ <- function(fg, no_cores=1) {
    # prepare parallel backend
    if (no_cores>1) future::plan(future::multisession)

    fg_graph <- fg_get_graph(fg)

    # meta data for samples
    meta_cell <- fg_graph$v

    # pparen: edge list i.e. the name of elements in the list are cell pops;
    # the vector in each element lists corresponding parent cell populations.
    # pchild: edge list i.e. the name of elements in the list are cell pops;
    # the vector in each element lists corresponding child cell populations.
    pparen <- fg_get_el(fg)$parent
    # child: pchild <- fg_get_el(fg)$child

    # mp: sample x cell population, cell count matrix
    mp <- fg_get_feature(fg, "node", "prop")

    # ep: sample x edge, proportion matrix i.e. if column name is A+B+C+_A+B+,
    # then the value is the count of A+B+C+ over A+B+.
    ep <- fg_get_feature(fg, "edge", "prop")
    phens <- fg_graph$v$phenotype[
        fg_graph$v$phenolayer != max(fg_graph$v$phenolayer)]
    phens <- phens[phens!=""]
    # max edges

    ep_ <- do.call(cbind, fpurrr_map(phens, function(phen)
        matrixStats::rowMins(ep[,paste0(pparen[[phen]], "__", phen),drop=FALSE])
        , no_cores=no_cores, prll=length(phens)>1000))
    colnames(ep_) <- phens

    rooti <- which(colnames(mp)=="")

    ## start calculating expected proportion
    ## first we prepare the list of cell populations;
    ## note we only calc expected proportion for cell populations in layers 2+.
    # vector of cell populations in the zeroth and first layer.
    cells1 <- append("",meta_cell$phenotype[meta_cell$phenolayer==1])
    expe1 <- matrix(.5,nrow=nrow(mp), ncol=length(cells1),
                          dimnames=list(rownames(mp),cells1))
    expe1[,1] <- 1

    # vector of cell populations in layers 2+.
    cells <- meta_cell$phenotype[meta_cell$phenolayer>1]
    # vector of the layer in which the cell populations in layers 2+ reside on.
    cellsn <- meta_cell$phenolayer[meta_cell$phenolayer>1]
    # logical: if there is any e.g. A++, in this data set
    # (as opposed to just A+, A-)
    multiplus <- any(nchar(unlist(
        stringr::str_extract_all(cells,"[+]+")))>1)
    cells1_p <- stringr::str_extract(cells1,"[+]+")
    maxp <- max(nchar(cells1_p[!is.na(cells1_p)]))

    ## calc expected props for cell pops w/ positive marker conds only ---------
    # this reduces computation time e.g. A+B+C+, not A+B+C-
    # cpind <- seq_len(length(cells))
    cpind <- which(!grepl("[-]",cells))
    # all: loop_ind <- loop_ind_f(1:length(cells),no_cores)
    expecp <- do.call(cbind, fpurrr_map(cpind, function(ic) {
        # cell population name e.g. A+B+C+
        cpop <- cells[ic]

        # node proportion values extracted from mp for cpop's parents
        # e.g. A+B+, A+C+, B+C+; and grandparents e.g. A+, B+, C+.
        pnames <- pparen[[cpop]]
        parent <- mp[,pnames,drop=FALSE] # proportion values of parents.

        # edge proportion values extracted from ep for edges between
        # cpop's parents and grandparents.
        pedges <- ep_[,pnames,drop=FALSE]

        p <- pnames[apply(parent,1,which.max)]
        p_ <- pnames[apply(parent,1,which.min)]
        pmin <- apply(parent,1,which.max)
        for (pname in pnames) {
            eind <- Reduce("|", lapply(pnames, function(pname)
                grepl(paste0("_",gsub("[+]","[+]",pname),"$"),colnames(ep)) ))
            # enames <- colnames(ep)[eind]
            # e <- enames[apply(ep[p==pname,enames,drop=FALSE],1,which.min)]

            # print(pname)
            # print(table(e))

        }
        for (pname in unique(pnames)) {

            p__ = p_[p==pname]

            # print(pname)
            # print(table(p__))

        }

        # working great: p=HLA, q=CD34 or CD117
        # HLA: CD34 HLA, CD117 HLA
        # CD45: HLA CD45, CD34 CD45
        # CD34: HLA CD34, CD117 CD34
        #
        # NOT JUST MAX AND IN PARENTS
        # HLA: CD117
        # CD45: HLA, CD117
        # CD34: HLA, CD117

        # using the above, get expected proportion; see formula in
        # https://www.biorxiv.org/content/10.1101/837765v2
        expect1 <- apply(pedges,1,min) * apply(parent,1,max)
        return(expect1)
    }, no_cores=no_cores, prll=length(cbind)>1000))
    expecp[is.nan(expecp)] <- 0
    colnames(expecp) <- cells[cpind]


    ## infer the expected prop of all other cell pops --------------------------
    # this isn't necessary, we could've just done it all above,
    # but this saves time; see same paper as above link
    expec0 <- expec <- as.matrix(cbind(expe1,expecp))
    cpopneg <- setdiff(cells,colnames(expec))
    cpopnegl <- cell_type_layers(cpopneg)

    # placeholder;
    # "count" is a variable given by gsubfn to any function it is given
    # count <- 0
    csp <- fg_get_etc(fg)$cumsumpos | !multiplus

    for (lev in sort(unique(cpopnegl))) {
        sibsl <- cells[cellsn==lev]
        cpopl <- cpopneg[cpopnegl==lev]
        # number of negative marker conditions
        cpopnegno <- stringr::str_count(cpopl,"[-]")
        cpopnegnos <- purrr::map(sort(unique(cpopnegno)),
                                 function(x) cpopl[cpopnegno==x])
        for (cpops in cpopnegnos) {
            if (csp) {
                expecn <- do.call(cbind, fpurrr_map(cpops, function(cpop) {
                    sib <- gsub('(.*?)-(.*)', '\\1+\\2', cpop)
                    if (!sib%in%colnames(expec)) {
                        pari <- which(purrr::map_lgl(
                            pparen[[cpop]],
                            ~grepl(.x,sib)))
                        if (length(pari)==0) return(rep(0,nrow(mp)))
                        return(mp[,pparen[[cpop]][pari[1]]])
                    }
                    pname <- intersect(pparen[[cpop]],
                                       pparen[[sib]])
                    return(mp[,pname] - expec[,sib])
                }, no_cores=no_cores, prll=length(cpops)>1000))
            } else {
                expecn <- do.call(cbind, fpurrr_map(cpops, function(cpop) {
                    cpopgsub <- gsub("[+]","[+]", cpop)
                    cpopgsub <- gsub('(.*?)-(.*)', '\\1[+]+\\2',
                                     cpopgsub)
                    cpopgsub <- gsub("[-]","[-]", cpopgsub)
                    sibs <- sibsl[grepl(cpopgsub,sibsl)]
                    sibs_ <- sibs%in%colnames(expec)
                    if (sum(sibs_)==0) {
                        sibs__ <- paste0(sibs, collapse="")
                        pari <- which(purrr::map_lgl(
                            pparen[[cpop]],
                            ~grepl(.x,sibs__)))
                        if (length(pari)==0)
                            return(rep(0,nrow(mp)))
                        return(mp[,pparen[[cpop]][pari[1]]])
                    }
                    sibs <- sibs[sibs_]
                    pname <- intersect(pparen[[cpop]],
                                       pparen[[sibs[1]]])
                    return(mp[,pname] - rowSums(expec[,sibs,drop=FALSE]))
                }, no_cores=no_cores, prll=length(cpops)>1000))
            }
            colnames(expecn) <- cpops
            expec <- cbind(expec, expecn)
        }
    }

    exp1 <- cbind(expe1,expec[,match(cells,colnames(expec)),
                                    drop=FALSE])
    # some: exp1 <- cbind(expe1,expecp[,match(cells,colnames(expecp)),drop=FALSE])

    exp1[as.matrix(exp1)<0] <- 0
    # some: exp1 <- cbind(expe1,expecp[,match(cells,colnames(expecp)),drop=FALSE])
    return(exp1) ### EXPECTED PROPORTION
    # mp/exp1 ### SPECENR
}


fg_feat_node_exprop_new <- function(fg, no_cores=1) {
    # prepare parallel backend
    if (no_cores>1) future::plan(future::multisession)

    fg_graph <- fg_get_graph(fg)

    # meta data for samples
    meta_cell <- fg_graph$v

    # pparen: edge list i.e. the name of elements in the list are cell pops;
    # the vector in each element lists corresponding parent cell populations.
    # pchild: edge list i.e. the name of elements in the list are cell pops;
    # the vector in each element lists corresponding child cell populations.
    pparen <- fg_get_el(fg)$parent
    # child: pchild <- fg@edge_list$child

    # mp: sample x cell population, cell count matrix
    mp <- fg_get_feature(fg, "node", "prop")

    # ep: sample x edge, proportion matrix i.e. if column name is A+B+C+_A+B+,
    # then the value is the count of A+B+C+ over A+B+.
    ep <- fg_get_feature(fg, "edge", "prop")
    phens <- fg_graph$v$phenotype[
        fg_graph$v$phenolayer != max(fg_graph$v$phenolayer)]
    phens <- phens[phens!=""]

    rooti <- which(colnames(mp)=="")

    ep_ <- do.call(cbind, fpurrr_map(phens, function(phen)
        matrixStats::rowMins(ep[,paste0(pparen[[phen]], "__", phen),drop=FALSE])
        , no_cores=no_cores, prll=length(phens)>1000))
    colnames(ep_) <- phens


    ## start calculating expected proportion
    ## first we prepare the list of cell populations;
    ## note we only calc expected proportion for cell populations in layers 2+.
    # vector of cell populations in the zeroth and first layer.
    cells1 <- append("",meta_cell$phenotype[meta_cell$phenolayer==1])
    expe1 <- matrix(.5,nrow=nrow(mp), ncol=length(cells1),
                          dimnames=list(rownames(mp),cells1))
    expe1[,1] <- 1

    # vector of cell populations in layers 2+.
    cells <- meta_cell$phenotype[meta_cell$phenolayer>1]
    # vector of the layer in which the cell populations in layers 2+ reside on.
    cellsn <- meta_cell$phenolayer[meta_cell$phenolayer>1]
    # logical: if there is any e.g. A++, in this data set
    # (as opposed to just A+, A-)
    multiplus <- any(nchar(unlist(
        stringr::str_extract_all(cells,"[+]+")))>1)
    cells1_p <- stringr::str_extract(cells1,"[+]+")
    maxp <- max(nchar(cells1_p[!is.na(cells1_p)]))

    ## calc expected props for cell pops w/ positive marker conds only ---------
    # this reduces computation time e.g. A+B+C+, not A+B+C-
    cpind <- seq_len(length(cells))
    # all: cpind <- which(!grepl("[-]",cells))

    # get all pairs of marker condition indices
    markers <- fg_get_markers(fg)
    ml <- max(fg_graph$v$phenolayer)
    mlcomb <- purrr::map(2:ml, function(mly)
        Reduce(cbind,purrr::map(seq_len(mly-1), function(x) {
        xl <- (x+1):mly
        rbind(xl,rep(x,mly-x))
    })))


    cells_ <- stringr::str_extract_all(cells, "[^_^+^-]+[+-]+")
    # some: loop_ind <- loop_ind_f(1:length(cells),no_cores)
    expecp <- do.call(cbind, fpurrr_map(cpind, function(ic) {
        # cell population name e.g. A+B+C+
        cpop <- cells[ic]
        cmarkers <- cells_[[ic]]

        # node proportion values extracted from mp for cpop's parents
        # e.g. A+B+, A+C+, B+C+; and grandparents e.g. A+, B+, C+.
        pnames <- pparen[[cpop]]
        parent <- mp[,pnames,drop=FALSE] # proportion values of parents.
        if (length(cmarkers)==2) return(apply(parent,1,prod))


        # try all combinations, because...
        mpi <- mp[,cpop]
        mlcombi <- mlcomb[[length(cmarkers)-1]]
        allcombexp <- purrr::map(seq_len(ncol(mlcombi)),function(yi) {
            y <- mlcombi[,yi]
            par1 <- gsub(cmarkers[y[1]],"",cpop,fixed=TRUE)
            mpe1p <- mp[,gsub(cmarkers[y[1]],"",cpop,fixed=TRUE)]
            mpe1e <- ep[,paste0(gsub(cmarkers[y[2]],"",par1,fixed=TRUE),
                                "_",gsub(cmarkers[y[2]],"",cpop,fixed=TRUE))]
            mpe1 <- mpe1p*mpe1e

            par1 <- gsub(cmarkers[y[2]],"",cpop,fixed=TRUE)
            mpe2p <- mp[,gsub(cmarkers[y[2]],"",cpop,fixed=TRUE)]
            mpe2e <- ep[,paste0(gsub(cmarkers[y[1]],"",par1,fixed=TRUE),
                                "_",gsub(cmarkers[y[1]],"",cpop,fixed=TRUE))]
            mpe2 <- mpe2p*mpe2e # mpe1 and mpe are the same
            return(cbind(mpe1p,mpe1e,mpe2p,mpe2e,mpe1))
        })
        acediff <- sapply(allcombexp, function(x)
            stats::median(abs(x[,5]-mp[,cpop])))
        return(allcombexp[[which.min(acediff)]][,5])

        allcombexp_par <- sapply(allcombexp, function(x)
            apply(x[,c(1,3)],1,min) )
        allcombexp_edg <- sapply(allcombexp,function(x)
            apply(x[,c(2,4)],1,max) )
        allcombexp_which <- apply(abs(allcombexp_par-allcombexp_edg),1,which.max)
        expect1 <- purrr::map_dbl(
            seq_len(length(allcombexp_which)), function(x) {
                allcombexp[[allcombexp_which[x]]][x,5]
            })

        # # edge proportion values extracted from ep for edges between
        # # cpop's parents and grandparents.
        # some: pes <- ep[,paste0(pparen[[cpop]], "_", cpop),drop=FALSE]
        pedges <- ep_[,pnames,drop=FALSE]

        # # using the above, get expected proportion; see formula in
        # # https://www.biorxiv.org/content/10.1101/837765v2
        # some: expect1 <- apply(pedges,1,min) * apply(parent,1,max)
        return(expect1)
    }, no_cores=no_cores, prll=length(cpind)>1000))
    expecp[is.nan(expecp)] <- 0
    colnames(expecp) <- cells[cpind]


    ## infer the expected prop of all other cell pops --------------------------
    # this isn't necessary, we could've just done it all above,
    # but this saves time; see same paper as above link
    expec0 <- as.matrix(cbind(expe1,expecp))

    exp1 <- expec0
    exp1[as.matrix(exp1)<0] <- 0
    # exp1 <- cbind(expe1,expecp[,match(cells,colnames(expecp)),drop=FALSE])
    future::plan(future::sequential)
    return(exp1) ### EXPECTED PROPORTION
}



#' @title Converts cell counts into cumulated cell counts.
#' @description Converts the cell counts in a flowGraph object into
#'  cumulated cell counts; this is optional and can be done only for there is
#'  more than one threshold for one or more markers.
#'  This should also only be ran when initializing a flowGraph object as
#'  converting back and forth is computationally expensive.
#'  If the user is interested in seeing non- and cumulated counts, we recommend
#'  keeping two flowGraph objects, one for each version.
#'  This function simply converts e.g. the count of A+ or A++ into
#'  the sum of count of A+, A++, and A+++ or A++, and A+++.
#' @param fg flowGraph object.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @return flowGraph object with cumulated counts.
#' @details \code{fg_feat_cumsum} returns the given flowGraph object with an
#'  adjusted count feature. As in our example,
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- flowGraph:::fg_feat_cumsum(fg, no_cores=no_cores)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[Matrix]{Matrix}}
#' @rdname fg_feat_cumsum
#' @importFrom future plan multisession sequential
#' @importFrom stringr str_split
#' @importFrom Matrix Matrix
#' @importFrom furrr future_map
#' @importFrom purrr map
fg_feat_cumsum <- function(fg, no_cores) {
    if (no_cores>1) future::plan(future::multisession)

    # check if already cumsum
    if (fg_get_etc(fg)$cumsumpos) return(fg)

    fg_graph <- fg_get_graph(fg)

    # check if do-able (there exists multple ++)
    if (!any(grepl("3",fg_graph$v$phenocode))) return(fg)

    mc <- as.matrix(fg_get_feature(fg, "node", "count"))
    meta_cell <- fg_graph$v
    meta_cell_grid <-
        do.call(rbind, stringr::str_split(meta_cell$phenocode,""))
    meta_cell_grid <- apply(meta_cell_grid, 2, as.numeric)
    meta_cell_grid_TF1 <- apply(meta_cell_grid,2, function(x) {
        xmax <- max(x)
        if (xmax==2) return(rep(FALSE,length(x)))
        x>1 & x<xmax
    })
    cany1 <- which(apply(meta_cell_grid_TF1, 2, any))
    for (marker in cany1) {
        coldo <- which(meta_cell_grid_TF1[,marker])
        coldo <- coldo[order(meta_cell_grid[coldo,marker],
                             decreasing=TRUE)]
        mc[,coldo] <- do.call(
            cbind, fpurrr_map(coldo, function(j) {
                mcgi <- mcgis <- mcgip <- meta_cell_grid[j,]
                mcgis[marker] <- mcgis[marker]+1
                jsib <- meta_cell$phenocode==paste0(mcgis,collapse="")
                mc[,j] + mc[,jsib]
            }, no_cores=no_cores, prll=length(coldo)>1000))

    }
    fg@feat$node$count_original <- fg_get_feature(fg, "node", "count")
    fg@feat$node$count <- mc
    fg@etc$cumsumpos <- TRUE
    if (length(fg_get_feature_all(fg)$node)>2 |
        length(fg_get_feature_all(fg)$edge)>0)
        warning("IMPORTANT: see function fg_rm_features to remove
                node features (other than count), and edge features.")
    future::plan(future::sequential)
    return(fg)
}


#' @title Normalizes all features for class.
#' @description For each class label in column \code{class} of \code{meta},
#'  \code{fg_feat_mean_class}
#'  takes the column mean of the rows in the given feature matrices
#'  (as specified in \code{node_features} and \code{edge_features})
#'  associated with that class;
#'  it then takes the difference point by point between these means and
#'  the original rows for that class.
#' @return A numeric matrix whose dimensions equate to that of the input
#'  and whose values are normalized per class.
#' @title Normalizes matrix values in a flowGraph object by class.
#' @description FUNCTION_DESCRIPTION
#' @param fg PARAM_DESCRIPTION
#' @param class a column name in \code{fg_get_meta(fg)} indicating the
#'  meta data that should be used as the class label of each sample
#'  while conudcting normalization.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @param node_features A string vector indicating the node features to
#'  perform normalization on; set as \code{NULL} to normalize all.
#' @param edge_features A string vector indicating the edge features to
#'  perform normalization on; set as \code{NULL} to normalize all.
#' @return flowGraph object with normalized features.
#' @details For all features in the given \code{flowGraph} object and for
#'  each class label in column \code{class} of \code{meta},
#'  \code{fg_feat_mean_class}. It takes the column mean of the rows in the given
#'  feature matrices
#'  (as specified in \code{node_features} and \code{edge_features})
#'  associated with that class;
#'  it then takes the difference point by point between these means and
#'  the original rows for that class.
#'  \code{fg_feat_mean_class}
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_feat_mean_class(fg, class="class", node_features="count",
#'                         no_cores=no_cores)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#' @rdname fg_feat_mean_class
#' @export
#' @importFrom future plan multisession sequential
#' @importFrom furrr future_map
fg_feat_mean_class <- function(
    fg, class, no_cores=1, node_features=NULL, edge_features=NULL
) {
    if (no_cores>1) future::plan(future::multisession)

    fg_meta <- fg_get_meta(fg)

    if (!class%in%colnames(fg_meta))
        stop("invalid class name, choose one from fg_get_meta(fg)\n")

    if (length(unique(fg_meta[,class]))<2) return(fg)
    label <- paste0("_MEAN_",class)

    fg_feat <- fg_get_feature_all(fg)

    if (is.null(node_features)) node_features <- names(fg_feat$node)
    node_features <- node_features[node_features%in%names(fg_feat$node)]
    node_features <- node_features[!grepl("MEAN", node_features)]
    if (length(node_features)>0) {
        fg_nodefs0 <- fg_feat$node[node_features]
        fg_nodefs <- fpurrr_map(fg_nodefs0, function(m0)
            mean_diff(as.matrix(m0), fg_meta[,class]),
            no_cores=no_cores, prll=length(fg_nodefs0)>1000)
        names(fg_nodefs) <- paste0(names(fg_nodefs),label)
        fg@feat$node <- append(fg_feat$node, fg_nodefs)
    }

    if (is.null(edge_features)) edge_features <- names(fg_feat$edge)
    edge_features <- edge_features[edge_features%in%names(fg_feat$edge)]
    edge_features <- edge_features[!grepl("MEAN", edge_features)]
    if (length(edge_features)>0) {
        fg_edgefs0 <- fg_feat$edge[edge_features]
        fg_edgefs <- fpurrr_map(fg_edgefs0, function(m0)
            mean_diff(as.matrix(m0), fg_meta[,class]),
            no_cores=no_cores, prll=length(fg_nodefs0)>1000)
        names(fg_nodefs) <- paste0(names(fg_edgefs),label)
        fg@feat$edge <- append(fg_feat$edge, fg_edgefs)
    }

    fg@feat_desc <- fg_get_feature_desc(fg, re_calc=TRUE)
    future::plan(future::sequential)
    return(fg)
}

#' @title Saves flowGraph object to a specified path.
#' @description Saves flowGraph object to a specified path.
#' @param fg flowGraph object to save.
#' @param folder_path A string indicating the folder path to where the flowGraph
#'  object should save its elements; if this is the first time the object is
#'  being saved, this folder should be empty or if it is
#'  not yet created, the function will create it. If the object has previously
#'  been saved before and this parameter is set to \code{NULL}, the function
#'  will save the object into the save folder it was previously saved in.
#' @param save_plots A logical indicating whether or not to save plots.
#' @param paired A logical indicating whether the summary is paired; used in
#'  function \code{fg_plot_box}.
#' @param ... Other parameters for the \code{fg_save_plots} function.
#' @details See generated README.md file.
#' @return TRUE if flowGraph object successfully saved.
#' @examples
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_save(fg, "tmp")
#'
#' @seealso
#'  \code{length},\code{c("nrow", "nrow")},\code{NULL}
#'  \code{\link[purrr]{map}}
#' @rdname fg_save
#' @export
#' @importFrom purrr map
#' @importFrom data.table fwrite
#' @importFrom utils write.csv
fg_save <- function(fg, folder_path=NULL, save_plots=TRUE, paired=FALSE, ...) {
    options(encoding="UTF-8")

    # check folder path
    if (is.null(folder_path)) {
        folder_path <- fg_get_etc(fg)$save$path
        if (is.null(folder_path))
            stop("please provide valid folder path")
    }
    if (!dir.exists(folder_path))
        dir.create(folder_path, recursive=TRUE, showWarnings=FALSE)
    if (length(dir(folder_path, all.files=TRUE))==0) {
        stop_save <- TRUE

        etc_dir <- paste0(folder_path,"/etc/etc.rds")
        if (file.exists(etc_dir))
            if (readRDS(etc_dir)$save$id == fg_get_etc(fg)$save$id)
                stop_save <- FALSE

        if (stop_save)
            stop("the folder provided is non-empty")
    }

    # create readme file
    fg_graph <- fg_get_graph(fg)
    fc <- file(paste0(folder_path,"/README.md"))
    writeLines(c(
        "PLEASE DO NOT MAKE CHANGES TO THIS FILE.",
        paste0("Last saved: ",  Sys.time()),
        "",
        "This folder contains files in a flowGraph object created by the flowGraph package.",
        "",
        "For a given flow cytometry sample and its cell populations, flowGraph generates SpecEnr values for each cell population or immunophenotype of based their expected proportion.",
        "By doing so, flowGraph accounts for the relation between cell populations and flags truly enriched cell populations rather than those with induced changes.",
        "",
        "The Summary_statistics folder contains p-values obtained from features associated with each immunophenotype or/ relation between immunophenotypes. The plots folder contains plots to help interpret those p-values.",
        "",
        "The Features folder contains features for:",
        paste0("- ", nrow(fg_get_meta(fg)), " flow cytometry samples."),
        paste0("- ", length(fg_get_markers(fg)), " markers: ",
               paste0(fg_get_markers(fg), collapse=", ")),
        paste0("- ", nrow(fg_graph$v),
               " cell population nodes (nodes for short) and ",
               nrow(fg_graph$e), " edges on ",
               max(fg_get_graph(fg)$v$phenolayer), " layers."),
        "",
        "Cell hierarchy plots: By default, for SpecEnr features, we generate two cell_hierarchy plots. The original one is where the colours represent difference between mean SpecEnr values across sample classes. We also add one where the colours represent difference betwee mean original values across sample classes. Original here is usually proportion: the feature used to create SpecEnr. Note that if a node is coloured lightly on the second plot, then the difference is very small, meaning the SpecEnr value may become sporadic. Therefore, when analyzing the plots, for most cases we recommend looking at most important cell populations as the ones with large difference in both plots.",
        ""), fc)

    close(fc)


    # save sample meta data and markers
    meta <- fg_get_meta(fg)
    utils::write.csv(meta, file=paste0(folder_path, "/sample_meta.csv"),
                     row.names=FALSE)


    # save features
    fg_feat <- fg_get_feature_all(fg)
    fg_feat_desc <- fg_get_feature_desc(fg)
    fn_dir <- paste0(folder_path, "/features/nodes")
    dir.create(fn_dir, recursive=TRUE, showWarnings=FALSE)
    a <- purrr::map(names(fg_feat$node), function(fn) {
        m <- as.matrix(fg_feat$node[[fn]])
        utils::write.csv(m, file=paste0(fn_dir,"/", fn, ".csv"))
    })
    sm <- fg_feat_desc$node
    utils::write.csv(as.matrix(sm), file=paste0(fn_dir,".csv"), row.names=FALSE)

    if (!is.null(fg_feat$edge)) {
        fe_dir <- paste0(folder_path, "/features/edges")
        dir.create(fe_dir, recursive=TRUE, showWarnings=FALSE)
        a <- purrr::map(names(fg_feat$edge), function(fe) {
            m <- as.matrix(fg_feat$edge[[fe]])
            # if (fe=="prop") fn <- "Proportion"
            utils::write.csv(m, file=paste0(fe_dir,"/", fe, ".csv"))
        })
        sm <- fg_feat_desc$edge
        utils::write.csv(sm, file=paste0(fe_dir,".csv"), row.names=FALSE)
    }


    # save summaries
    fg_summary_desc <- fg_get_summary_desc(fg)
    fg_summary <- fg_get_summary_all(fg)
    if (!is.null(fg_summary$node)) {
        sn_dir <- paste0(folder_path, "/summary_statistics/nodes")
        dir.create(sn_dir, recursive=TRUE, showWarnings=FALSE)
        tb <- fg_get_summary_tables(fg)
        utils::write.csv(tb, file=paste0(sn_dir, "/pvalues.csv"))
        sm <- fg_summary_desc$node
        data.table::fwrite(sm, file=paste0(sn_dir,".csv"), sep=",",
                           row.names=FALSE, col.names=TRUE)
    }

    if (!is.null(fg_summary$edge)) {
        se_dir <- paste0(folder_path, "/summary_statistics/edges")
        dir.create(se_dir, recursive=TRUE, showWarnings=FALSE)
        tb <- fg_get_summary_tables(fg, type="edge")
        utils::write.csv(tb, file=paste0(se_dir, "/pvalues.csv"))
        sm <- as.matrix(fg_summary_desc$edge)
        data.table::fwrite(sm, file=paste0(sn_dir,".csv"), sep=",",
                           row.names=FALSE, col.names=TRUE)
    }

    # save edge lists, graph layout and others for reproducibility
    # add folder and save id to object
    elist <- fg_get_el(fg)
    markers <- fg_get_markers(fg)
    pl <- fg_get_plot_layout(fg)
    fg@etc$save$id <<- list(id=stringi::stri_rand_strings(1,5))
    fg@etc$save$path <<- folder_path
    etc <- fg_get_etc(fg)
    save(fg_graph, elist, fg_summary, markers, pl, etc,
         file=paste0(folder_path, "/other.Rdata"))

    if (save_plots)
        fg_save_plots(fg, plot_path=paste0(folder_path,"/plots"), paired=paired)

    return(TRUE)
}

#' @title Load a flowGraph object from a specified folder path.
#' @description Load a flowGraph object from a specified folder path.
#' @param folder_path A string indicating the folder path to where a flowGraph
#'  object was saved using the \code{fg_save} function.
#' @return flowGraph object
#' @details see function \code{fg_save}
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_save(fg, "tmp")
#'  fg <- fg_load("tmp")
#' @seealso
#'  \code{\link[flowGraph]{fg_save}}
#' @rdname fg_load
#' @export
#' @importFrom data.table fread
#' @importFrom purrr map
#' @importFrom methods new
fg_load <- function(folder_path) {
    options(encoding="UTF-8")
    options(stringsAsFactors=FALSE)

    # load sample meta data and markers
    meta <- data.table::fread(paste0(folder_path, "/sample_meta.csv"), data.table=FALSE)

    # load features
    fn_dir <- paste0(folder_path, "/features/nodes")
    fn_files <- gsub(".csv","",list.files(fn_dir))
    feats <- list(node=purrr::map(fn_files, function(fn) {
        a <- data.table::fread(file=paste0(fn_dir,"/", fn, ".csv"))
        rownames_a <- a[,1]
        a <- as.matrix(a[,-1,drop=FALSE])
        rownames(a) <- unlist(rownames_a)
        colnames(a)[1] <- ""
        a
    }))
    names(feats$node) <- fn_files
    feats_desc <- list(node=data.table::fread(paste0(fn_dir,".csv"), data.table=FALSE))

    fe_dir <- paste0(folder_path, "/features/edges")
    if (dir.exists(fe_dir)) {
        fe_files <- gsub(".csv","",list.files(fe_dir))
        feats$edge <- purrr::map(fe_files, function(fe) {
            a <- data.table::fread(file=paste0(fe_dir,"/", fe, ".csv"))
            rownames_a <- a[,1]
            a <- as.matrix(a[,-1,drop=FALSE])
            rownames(a) <- unlist(rownames_a)
            a
        })
        feats_desc$edge <- data.table::fread(paste0(fe_dir,".csv"), data.table=FALSE)
        names(feats$edge) <- fe_files
    }


    # load summary descriptions
    summary_desc <- list()
    sn_dir <- paste0(folder_path, "/summary_statistics/nodes")
    if (dir.exists(sn_dir))
        summary_desc$node <- data.table::fread(paste0(sn_dir,".csv"), data.table=FALSE)

    se_dir <- paste0(folder_path, "/summary_statistics/edges")
    if (dir.exists(se_dir)) {
        summary_desc$edge <- data.table::fread(paste0(se_dir,".csv"), data.table=FALSE)
    }

    # load edge lists, graph layout and others for reproducibility
    # originally: load(paste0(folder_path, "/other.Rdata"))
    # fg_graph, elist, fg_summary, markers, pl, etc
    other_path <- paste0(folder_path, "/other.Rdata")
    attach(other_path, warn.conflicts=FALSE)

    other_path_ <- paste0("file:", other_path)

    # organize into flowGraph object
    methods::new(
        "flowGraph",
        feat=feats,
        feat_desc=feats_desc,
        summary=get("fg_summary", pos=other_path_),
        summary_desc=summary_desc,
        markers=get("markers", pos=other_path_),
        graph=get("fg_graph", pos=other_path_),
        edge_list=get("elist", pos=other_path_),
        meta=meta,
        plot_layout=get("pl", pos=other_path_),
        etc=get("etc", pos=other_path_)
    )
}


#' @title Retrieves a feature matrix.
#' @description Retrieves a feature matrix from a given flowGraph object,
#'  the feature type, and feature name.
#' @param fg flowGraph object.
#' @param type A string indicating feature type 'node' or 'edge'.
#' @param feature A string indicating feature name;
#' @return A numeric matrix of the specified feature values.
#' @details Returns \code{NULL} if the requested feature does not exist.
#' @examples
#'
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=1)
#'
#'  feature_matrix <- fg_get_feature(fg, type='node', feature='count')
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#' @rdname fg_get_feature
#' @export
fg_get_feature <- function(fg, type="node", feature="count") {
    type <- match.arg(type, c("node", "edge"))
    as.matrix(fg_get_feature_all(fg)[[type]][[feature]])
}

fg_get_feature_all <- function(fg) fg@feat


#' @title Retrieves the index of the requested summary.
#' @description Retrieves the index of the requested summary
#'  from a given flowGraph object.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{type} (feature type: \code{node} or \code{edge}),
#'  \code{feature} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{lable1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @return An integer analagous to \code{index}.
#'  If both \code{index} and \code{summary_meta} are \code{NULL}, returns 1.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  # set features to NULL to apply summary statistic to all features.
#'  fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
#'                   overwrite=FALSE, test_name="t", diminish=FALSE,
#'                   node_features=NULL, edge_features=NULL)
#'  show(fg)
#'
#'  index <- flowGraph:::fg_get_summary_index(
#'   fg, type="node", summary_meta=list(
#'     feature="SpecEnr", test_name="t", class="class",
#'     label1="control", label2="exp"))
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_plot}}
#' @rdname fg_get_summary_index
#' @export
fg_get_summary_index <- function(fg, type="node", index=NULL, summary_meta=NULL) {
    type <- match.arg(type, c("node", "edge"))
    if (is.null(fg_get_summary_desc(fg))) stop("summary not found")
    if (is.null(fg_get_summary_desc(fg)[[type]])) stop()
    if (is.null(index) & is.null(summary_meta)) {
        warning("no summary statistic index/summary_meta provided, returning index=1")
        return(1)
    }
    if (is.null(index)) {
        index <- which(apply(fg_get_summary_desc(fg)[[type]],1, function(x)
            identical(as.character(unlist(x)),
                      as.character(unlist(summary_meta)))))
        if (length(index) == 0) stop("summary not found")
    }
    return(index)
}


#' @title Retrieves a summary statistic.
#' @description Retrieves a summary statistic from a given flowGraph object;
#'  while \code{fg} is required, the user can choose to input parameters
#'  \code{summary_meta}, \code{index}, or all of \code{type},
#'  \code{feat}, \code{test_name}, \code{class}, \code{label1}, and \code{label2}.
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{feat} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param SpecEnr_filt A logicle indicating whether or not to filter p-values
#'  for SpecEnr.
#' @param summary_fun A function that takes in a matrix and outputs a
#'  vector the same length as the number of columns this matrix has.
#'  Set to \code{NULL} to not calculate this summary
#'  (i.e. returned list will not contain \code{m1} and \code{m2}).
#'  See \code{\link[flowGraph]{fg_get_feature_means}}.
#' @param adjust0_lim A vector of two numeric values indicating a range around
#'  0, default set to -0.1 and 0.1.
#' @param filter_adjust0 A numeric variable indicating what percentage of
#'  SpecEnr values compared (minimum) should be not close to 0.
#'  Set to 1 to not conduct filtering. Original p-values stored in
#'  \code{values_original}.
#' @param filter_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant.
#' @param filter_btwn_tpthres A numeric variable between 0 and 1 indicating the
#'  unadjusted T-test p-value threshold used to test whether the actual
#'  and expected feature values used to calculate the specified SpecEnr
#'  feature are significantly different for each sample class. Note this only
#'  needs to be specified for SpecEnr features. Combined
#'  with \code{filter_btwn_es}, we conduct three tests to understand if
#'  there is an actual large difference between actual and expected features:
#'  (1,2) T-test of significance between the actual and expected raw feature value
#'  (e.g. proportion) for samples in each of the compared classes, (3) and the
#'  T-test of significance between the differences of actual and
#'  expected feature values of the two classes. If any two of the three tests
#'  come out as insignificant, we set the p-value for the associated node/edge
#'  to 1.
#' @param filter_btwn_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant -- see \code{filter_btwn_tpthres}.
#' @param default_p_thres A numeric variable indicating the p-value threshold
#'  user is using. Currently, all nodes/edges not passing the \code{filter}
#'  criterion will be defaulted to 1; if this parameter is set, then all
#'  of these nodes/edges will be set to a minimum of \code{default_p_thres}.
#' @return A list containing elements on feature summary retrieved by the user
#'  as in the \code{summary} slot of
#'  \code{\link[flowGraph]{flowGraph-class}}.
#'  If \code{summary_fun} is not \code{NULL}, this list also includes:
#'  \itemize{
#'   \item{\code{m1}: a numeric vector the same length as \code{values};
#'   this is a summary of the samples compared e.g. mean.}
#'   \item{\code{m2}: a numeric vector the same length as \code{values};
#'   this is a summary of the samples compared e.g. mean.}
#'   \item{\code{cohensd}: a numberic vector indicating cohen's d values
#'    considering effect size.}
#'   \item{\code{cohensd_size}: a factor vector interpreting cohen's d values.}
#'   \item{\code{adjust0}: a numeric vector indicating the percentage of
#'    samples that have a SpecEnr value in the range of \code{adjust0_lim}
#'    around 0; if there are two classes of samples being compared, we output
#'    the smaller percentage between the two classes.}
#'   \item{\code{btwn}: a data frame containing columns:
#'    \itemize{
#'     \item{\code{tpv1}: unadjusted p-value calculated
#'      between the actual and expected raw feature values of class 1.}
#'     \item{\code{tpv2}: unadjusted p-value calculated
#'      between the actual and expected raw feature values of class 2.}
#'     \item{\code{cd1}: Cohen's D between the actual and expected raw
#'      feature values of class 1.}
#'     \item{\code{cd2}: Cohen's D between the actual and expected raw
#'      feature values of class 2.}
#'     \item{\code{btp}: unadjusted p-value calculated between the
#'      difference between actual and expected raw feature of the two classes.}
#'     \item{\code{bcd}: Cohen's D calculated between the
#'      difference between actual and expected raw feature of the two classes.}
#'     \item{\code{btp_}: unadjusted p-value calculated between the
#'      log ratio between actual and expected raw feature of the two classes.}
#'     \item{\code{bcd_}: Cohen's D calculated between the
#'      log ratio between actual and expected raw feature of the two classes.}
#'    }
#'   }
#'  }
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  # set features to NULL to apply summary statistic to all features.
#'  fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
#'                   overwrite=FALSE, test_name="t", diminish=FALSE,
#'                   node_features=NULL, edge_features=NULL)
#'  show(fg)
#'
#'  feat_summ <- fg_get_summary(fg, type="node", summary_meta=list(
#'      feature="SpecEnr", test_name="t", class="class",
#'      label1="control", label2="exp"))
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_means}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#' @rdname fg_get_summary
#' @export
#' @importFrom purrr map_lgl
#' @importFrom effsize cohen.d
fg_get_summary <- function(
    fg, type="node", index=NULL, summary_meta=NULL, adjust_custom="byLayer",
    SpecEnr_filt=TRUE,
    summary_fun=colMeans,
    adjust0_lim=c(-.1,.1), filter_adjust0=1, filter_es=0,
    filter_btwn_tpthres=.05, filter_btwn_es=.5,
    default_p_thres=1
) {
    type <- match.arg(type, c("node", "edge"))
    index <- fg_get_summary_index(fg,type=type, index, summary_meta)
    summary_meta <- unlist(fg_get_summary_desc(fg)[[type]][index,])
    if (!grepl("SpecEnr",unlist(summary_meta["feat"])))
        filter_adjust0 <- 1

    sl <- fg_get_summary_all(fg)[[type]][[index]]
    if (is.list(sl$values)) sl$values <- unlist(sl$values)
    sl <- append(sl, list(
        id1=fg_get_meta(fg)[,summary_meta["class"]]==summary_meta["label1"],
        id2=fg_get_meta(fg)[,summary_meta["class"]]==summary_meta["label2"]))

    # adjust function
    if (!is.function(adjust_custom))
        if (adjust_custom=="none") {
            adjust_custom <- function(x) x
        } else if (adjust_custom!="byLayer") {
            ac <- adjust_custom
            adjust_custom <- function(x) stats::p.adjust(x,method=ac)
        }

    pna <- is.na(sl$values)
    pnaw <- which(!pna)
    if (!is.function(adjust_custom)) {
        fg_graph <- fg_get_graph(fg)
        lyrs <- fg_graph$v$phenolayer[!pna]
        if (type=="edge")
            lyrs <- fg_graph$v$phenolayer[match(
                fg_graph$e$to[!pna], fg@graph$v$phenotype)]
        lyrsu <- unique(lyrs)
        for (lyr in lyrsu) {
            lyri <- lyrs==lyr
            sl$values[pnaw[lyri]] <- sl$values[pnaw[lyri]] * sum(lyri)
        }
        sl$values <- sl$values * length(lyrsu)
    } else {
        sl$values[pna] <- adjust_custom(sl$values[pna])
    }
    sl$values[pna] <- 1
    belowthres <- sl$values<default_p_thres

    # colmeans
    if (!is.null(summary_fun)) {
        feat <- unlist(fg_get_summary_desc(fg)[[type]]$feat[index])
        m1 <- fg_get_feature_means(
            fg, type, feat, id=sl$id1, summary_fun=summary_fun)
        m2 <- fg_get_feature_means(
            fg, type, feat, id=sl$id2, summary_fun=summary_fun)
        sl <- append(sl, list(m1=m1, m2=m2))
    }

    # cohen's d
    ename <- paste(summary_meta[c("feat", "class","label1","label2")], collapse="_")
    fg_etc <- fg_get_etc(fg)
    if (is.null(fg_etc$effect_size[[type]]))
        fg_etc$effect_size[[type]] <- list()
    if (is.null(fg_etc$effect_size[[type]][[ename]]))
        fg_etc$effect_size[[type]][[ename]] <-
            fg_cohensd_(fg@feat[[type]][[feat]], sl$id1, sl$id2)

    sl <- append(sl, fg_etc$effect_size[[type]][[ename]])

    if (grepl("SpecEnr", summary_meta["feat"]) & SpecEnr_filt) {
        aname <- paste(summary_meta[c("feat","class","label1","label2")], collapse="_")
        if (is.null(fg_etc$adjust0[[type]]))
            fg_etc$adjust0[[type]] <- list()
        if (is.null(fg_etc$adjust0[[type]][[aname]]))
            fg_etc$adjust0[[type]][[aname]] <-
                fg_adjust0_(fg_get_feature_all(fg)[[type]][[feat]],
                            sl$id1,sl$id2, adjust0_lim)

        sl$adjust0 <- fg_etc$adjust0[[type]][[aname]]

        sm_name <- paste0(summary_meta[-2], collapse="_")
        if (is.null(fg_etc$actualVSexpect[[type]]))
            fg_etc$actualVSexpect[[type]] <- list()
        if (is.null(fg_etc$actualVSexpect[[type]][[sm_name]])) {
            feat_ <- se_feats(feat)
            mp <- fg_get_feature(fg, type=type, feature=feat_[2])
            mep <- fg_get_feature(fg, type=type, feature=feat_[3])
            fg_etc$actualVSexpect[[type]][[sm_name]] <- fg_btwn_(
                sl$id1, sl$id2, mp, mep,
                btwn_test_custom=function(x,y)
                    tryCatch(stats::t.test(x,y)$p.value, error=function(e) 1))
        }
        sl$btwn <- df_ps <- fg_etc$actualVSexpect[[type]][[sm_name]]

        sl0 <- sl$values
        if (filter_btwn_es>0 | filter_btwn_tpthres<1) {

            sig1 <- df_ps$tpv1<filter_btwn_tpthres
            sig2 <- df_ps$tpv2<filter_btwn_tpthres
            sig3 <- df_ps$btp<filter_btwn_tpthres
            sigcands <- (sig1 | sig2) & sig3

            sl$values_original <- sl0
            sl$values[!sigcands & belowthres] <- default_p_thres
        }

        # 0's
        if (filter_adjust0<1 & grepl("SpecEnr", unlist(summary_meta["feat"]))) {
            sl$values_original <- sl0
            sl$values[sl$adjust0>filter_adjust0 & belowthres] <- default_p_thres
        }
        # cohen's d
        if (filter_es>0) {
            sl$values_original <- sl0
            sl$values[abs(sl$cohensd)<filter_es & belowthres] <- default_p_thres
        }
    }
    # choice to update etc # fg@etc <<- fg_etc
    sl$values[is.na(sl$values) | sl$values>1] <- 1
    return(sl)
}

fg_get_summary_all <- function(fg) fg@summary


#' @title Retrieves a table containing all node or edge summary statistics.
#' @description Retrieves a table containing all node or edge summary statistics
#'  given a flowGraph object.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summaries the user
#'  wants to retrieve were created for, 'node' or 'edge'.
#' @return A list; this output is the same as that of function
#'  \code{fg_get_graph} with additional columns.
#'  These columns contain summary statistics from the \code{summary}
#'  slot of the flowGraph object. These columns are named:
#'  <feature type: node/edge>.<feature>.<summary name>.<class>.<class labels>.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
#'                   overwrite=FALSE, test_name="t", diminish=FALSE)
#'  show(fg)
#'
#'  feat_summ_table_node <- fg_get_summary_tables(fg, type="node")
#'  head(feat_summ_table_node)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_means}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#' @rdname fg_get_summary_tables
#' @export
fg_get_summary_tables <- function(fg, type="node") {
    type <- match.arg(type, c("node", "edge"))
    if (type=="node") {
        gr <- fg_get_graph(fg)$v
        gr <- gr[,!colnames(gr)%in%c("x", "y")]
    } else {
        gr <- fg_get_graph(fg)$e
        gr <- gr[,!colnames(gr)%in%c("from.x", "from.y", "to.x", "to.y")]
    }
    if (!type%in%names(fg_get_summary_all(fg))) stop("no summaries found")

    desc <- fg_get_summary_desc(fg)[[type]]
    descn <- apply(desc, 1, function(x) paste0(x, collapse="."))
    descn <- paste0(type, ".", descn)
    for (ft1 in seq_len(length(descn)))
        gr[[descn[ft1]]] <- fg_get_summary(fg, type, index=ft1)$values
    return(gr)
}


#' @title Retrieves and/or recalculates a feature description table.
#' @description Retrieves and/or recalculates a feature
#'  description table for a given flowGraph object.
#' @param fg flowGraph object.
#' @param re_calc A logical variable specifying whether or not a feature summary
#'  should be re-calculated or directly retrieved from \code{fg}.
#' @return A data frame where each row contains information on a feature
#' from the given flowGraph object; its columns is as in the \code{feat_desc}
#' slot of \code{\link[flowGraph]{flowGraph-class}}.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_get_feature_desc(fg, re_calc=TRUE)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#' @rdname fg_get_feature_desc
#' @export
fg_get_feature_desc <- function(fg, re_calc=FALSE) {
    if (re_calc) {
        fg_feat <- fg_get_feature_all(fg)
        result1 <- as.data.frame(sapply(names(fg_feat$node), function(x)
            summary_table(fg_feat$node[[x]], x)))

        result2 <- NULL
        if (length(fg_feat$edge) >
            0) {
            result2 <- as.data.frame(sapply(
                names(fg_feat$edge), function(x)
                    summary_table(fg_feat$edge[[x]], x)))
        }
        return(list(node=result1, edge=result2))
    }
    fg@feat_desc
}


#' @title Retrieves a feature summary description table.
#' @description Retrieves a feature summary description table for
#'  a given flowGraph object.
#' @param fg flowGraph object.
#' @return A data frame where each row contains information
#'  on a feature summary from \code{fg}:
#' \itemize{
#'   \item{\code{type}: feature type (i.e. 'node' or 'edge').}
#'   \item{\code{feat}: feature name.}
#'   \item{\code{test_name}: summary name.}
#'   \item{\code{class}: class or the column name of \code{fg_get_meta(fg)}
#'    whose values represent the class label of each sample on which
#'    the summary was created for.}
#'   \item{\code{label1}: A string from the \code{class} column of the
#'    \code{meta} slot indicating the label of samples compared.}
#'   \item{\code{label2}: A string from the \code{class} column of the
#'    \code{meta} slot indicating the label of samples compared.}
#' }
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_get_summary_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#' @rdname fg_get_summary_desc
#' @export
fg_get_summary_desc <- function(fg) fg@summary_desc



## get/replace fg_get_meta meta data

#' @title Retrieves sample meta.
#' @description Retrieves sample meta from a given flowGraph object.
#' @param fg flowGraph object.
#' @return A data frame containing sample meta data.
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_replace_meta}}
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  head(fg_get_meta(fg))
#'
#' @export
#' @rdname fg_get_meta
fg_get_meta <- function(fg) fg@meta



## get graph

#' @title Retrieves a graph list from a given flowGraph object.
#' @description Retrieves a graph list from a given flowGraph object.
#' @param fg flowGraph object.
#' @return A list containing two data frames (\code{v} and ]code{e})
#'  from the \code{graph} slot of the given \code{flowGraph} object containing
#'  information on the cell populations phenotype nodes and edges representing
#'  relation between cell populations.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  gr <- fg_get_graph(fg)
#'  head(gr$v)
#'  head(gr$e)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_plot}}
#'  \code{\link[flowGraph]{ggdf}}
#'  \code{\link[flowGraph]{plot_gr}}
#' @rdname fg_get_graph
#' @export
fg_get_graph <- function(fg) fg@graph

fg_get_el <- function(fg) fg@edge_list


## get markers
#' @title Retrieves the markers from a given flowGraph object.
#' @description Retrieves the markers from a given flowGraph object.
#' @param fg flowGraph object.
#' @return A character vector containing the markers used in a flowGraph object.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  fg_get_markers(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#' @rdname fg_get_markers
#' @export
fg_get_markers <- function(fg) fg@markers

#' @title Retrieves feature summaries.
#' @description Retrieves a feature summary (e.g. \code{colMeans}) for samples
#'  specified by sample id's \code{id} OR class label
#'  \code{label} for class \code{class} given a feature specified
#'  by \code{type} and \code{feat}.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param feature A string indicating feature name the summary was created for;
#' @param class A string corresponding to a column name of the \code{meta} slot of
#'  \code{fg} whose values represent the class label of each sample
#'  on which the summary was created to compare or analyze;
#' @param label A string indicating a class label.
#' @param id A string vector containing the sample id's corresponding to the
#'  \code{id} column of the \code{meta} slot of \code{fg}.
#' @param summary_fun A function that takes in a matrix and outputs a
#'  vector the same length as the number of columns this matrix has.
#' @return A list containing two numeric vectors calculated using the
#'  \code{summary_fun} function on the subset of samples specified by
#'  sample id's \code{id} OR class label
#'  \code{label} for class \code{class} from a feature matrix
#'  specified by \code{type} and \code{feat}.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'  fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
#'                   overwrite=FALSE, test_name="t", diminish=FALSE)
#'  show(fg)
#'  feat_mean <- fg_get_feature_means(fg, type="node", feature="count",
#'                                    class="class", label="control")
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#' @rdname fg_get_feature_means
#' @export
fg_get_feature_means <- function(
    fg, type=c("node","edge"), feature="count",
    class=NULL, label=NULL, id=NULL,
    summary_fun=colMeans
) {
    m <- fg_get_feature(fg, type, feature)
    if (!is.null(id)) {
        m1 <- m[id,,drop=FALSE]
    } else {
        m1 <- m[fg_get_meta(fg)[[class]]==label,,drop=FALSE]
    }
    summary_fun(as.matrix(m1))
}

fg_get_etc <- function(fg) fg@etc

fg_get_plot_layout <- function(fg) fg@plot_layout


# given number of markers, output number of nodes, edges
get_graph_info <- function(m) {
    npl <- epl <- rep(0,m)
    for (i in seq_len(m)) {
        npl[i] <- choose(m,i)*(2^i)
        epl[i] <- npl[i]*i
    }
    n <- 3^m
    e <- n*2*m/3
    return (list(n=n, npl=npl, e=e, epl=epl))
}


# given a "SpecEnr" feature, get the raw and expected features
se_feats <- function(feature) {
    if (feature=="SpecEnr")
        return(c(feature, "prop", "expect_prop"))
    if (grepl("SpecEnr", feature)) {
        fl <- gsub("SpecEnr_","",feature)
        return(c(feature, fl, paste0("expect_",fl)))
    }
    return(feature)
}

test_c <- function(test_custom) {
    if (!is.function(test_custom))
        if (test_custom=="t") {
            test_custom <- function(x,y)
                tryCatch(stats::t.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="wilcox") {
            test_custom <- function(x,y)
                tryCatch(stats::wilcox.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="ks") {
            test_custom <- function(x,y)
                tryCatch(stats::ks.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="var") {
            test_custom <- function(x,y)
                tryCatch(stats::var.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="chisq") {
            test_custom <- function(x,y)
                tryCatch(stats::chisq.test(x,y)$p.value,error=function(e)1)
        }
    test_custom
}

#' @title Calculates feature summary statistics.
#' @description Calculates feature summary statistics for flowGraph features;
#'  users can choose from a list of statistical significance tests/adjustments
#'  or define custom summary functions.
#'  For special cases, see example in function
#'  \code{\link[flowGraph]{fg_add_summary}} on
#'  how to manually calculate summary statistics without using this function.
#' @param fg flowGraph object.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @param class A string corresponding to the column name or index
#'  of \code{fg_get_meta(fg)} whose values represent
#'  the class label of each sample.
#' @param label1 A string from the \code{class} column of the
#'     \code{meta} slot indicating one of the labels compared to create
#'     the summary statistic. If you would like to compare all other class
#'     labels against one label, set \code{label2} to \code{NULL}.
#'     If you would like to compare all labels against
#'     all labels, set \code{label1} and \code{label2} to \code{NULL}.
#' @param label2 A string from the \code{class} column of the
#'     \code{meta} slot indicating one of the labels compared to create
#'     the summary statistic.
#' @param class_labels A list of vectors, each containing two strings
#'  represeting labels to compare; this parameter is an alternative to
#'  parameters \code{label1} and \code{label2} that supports multiple label
#'  pairings.
#' @param node_features A string vector indicating which node feature(s)
#'  to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
#'  and the function will perform summary statistics on all or no node features.
#' @param edge_features A string vector indicating which edge feature(s)
#'  to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
#'  and the function will perform summary statistics on all or no edge features.
#' @param test_name A string with the name of the test you are performing.
#' @param diminish A logical variable indicating whether to use
#'  diminishing summary statistics;
#'  if \code{TRUE}, a summary statistic for a node or edge will only be done if
#'  at least one of its parent node or edge is significant. Otherwise, the test
#'  will be performed on all nodes or edges.
#' @param p_thres A double indicating the summary statistic threshold;
#'  if the result of a statistical test is greater than \code{p_thres}, then it
#'  is insignificant.
#' @param p_rate A double; if \code{diminish=TRUE}, then \code{p_rate}
#'  needs to be specified.
#'  to determine whether or not a node or edge's parent is significant,
#'  we use \code{p_thres}. However, the higher the layer on which a node
#'  resides or to which an edge points to,
#'  the less stringent this \code{p_thres} should be.
#'  Therefore, we set \code{p_thres}
#'  as the threshold for the parent node or edge of the last
#'  layer and multiply \code{p_thres}
#'  by \code{p_rate} for each increasing layer e.g. given
#'  default values and 4 layers,
#'  the thresholds for layers 1 through 4 would be .4, .2, .1, and .05.
#' @param test_custom A function or a string indicating the statistical test
#'  to use. If a string is provided, it should be one of
#'  \code{c("t","wilcox","ks","var","chisq")};
#'  these correspond to statistical tests \code{stats::t.test},
#'  \code{stats::wilcox.test}, and so on.
#'  If a function is provided, it should take as input two numeric vectors and
#'  output a numeric variable.
#' @param effect_size A logical variable indicating whether or not to calculate
#'  effect size statistic (cohen's d) for this set of class labels;
#'  later used for plotting.
#' @param adjust0 A logical variable indicating whether or not to calculate the
#'  minimum percentage of values from samples of each class label that falls
#'  within the range of \code{adjust0_lim}. This is only done for SpecEnr values
#'  as p-values become unstable when comparing near 0 values.
#' @param adjust0_lim A vector of two numeric values indicating a range around
#'  0, default set to -0.1 and 0.1.
#' @param btwn A logical variable indicating whether or not to calculate the
#'  \code{btwn} data frame given in the \code{fg_get_summary} function.
#' @param btwn_test_custom Same as \code{test_custom} but for \code{btwn}.
#' @param save_functions A logical variable indicating whether to save test and adjust functions.
#' @param overwrite A logical variable indicating whether to
#'  overwrite the existing summary statistics if it exists.
#' @return flowGraph object containing claculated summary statistics.
#' @details \code{fg_summary} calculates a summary statistic as specified
#'  by the user in parameters \code{test_name}, \code{diminish}
#'  (\code{p_thres}, \code{p_rate}), and \code{test_custom}.
#'  The test is done for a node or edge feature of interest within a given
#'  flowGraph object as specified by parameters \code{node_features},
#'  \code{edge_features}. It then returns information on the summary statistic
#'  inside the same flowGraph object and returns it to the user.
#'  See \code{\link[flowGraph]{flowGraph-class}} slot \code{summary}
#'  for details on the contents.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  fg_get_summary_desc(fg)
#'
#'  fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
#'                   overwrite=FALSE, test_name="t", diminish=FALSE,
#'                   node_features="count", edge_features="NONE")
#'  fg_get_summary_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_clear_summary}}
#' @rdname fg_summary
#' @export
#' @importFrom purrr map_lgl map map_dbl
#' @importFrom stats t.test wilcox.test ks.test var.test chisq.test p.adjust
fg_summary <- function(
    fg, no_cores=1, class="class", label1=NULL, label2=NULL, class_labels=NULL,
    node_features="SpecEnr", edge_features="NONE",
    # name your statistical test / adjustment method;
    # if same name, overwrite=T will overwrite, overwrite=F will return nothing
    test_name="t_diminish",
    # don't test if all parents are insignificant, stricter the lower the layer
    diminish=TRUE,
    p_thres=.05, p_rate=2, # only used if diminish=TRUE
    test_custom="t",

    effect_size=TRUE, adjust0=TRUE, adjust0_lim=c(-.1, .1),
    btwn=TRUE, btwn_test_custom="t", save_functions=FALSE,
    overwrite=FALSE
) {
    # if label1 is NULL, compare all
    # if node/edge_features is NULL, do all

    fg_meta <- fg_get_meta(fg)

    if (!class%in%colnames(fg_meta)) {
        print(class)
        stop(paste0("invalid class, choose one from ",
                    paste0(colnames(fg_meta), collapse=", "),
                    ": did not find ", class))
    }

    test_name <- gsub("[.]","_",test_name)
    classes_ <- fg_meta[,class]
    classes <- unique(classes_)
    if (length(classes)<2)
        stop("please provide a valid class from columns of fg_get_meta(fg).")

    # class list lists what classes to compare
    if (!is.null(label1))
        if (!label1%in%fg_meta[,class] & !is.null(class_labels)) {
            label1 <- NULL
            stop("we will proceed to compare all class labels with each other.")
        }
    if (!is.null(class_labels)) {
        if (!all(purrr::map_lgl(class_labels, function(x) x%in%classes_)))
            stop("please provide valid class labels class_labels.")
        class_list <- list(class_labels)
    } else if (!is.null(label1) & is.null(label2)) {
        class_list <- purrr::map(classes[classes!=label1],
                               function(x) c(label1, x))
    } else if (!is.null(label1) & !is.null(label2)) {
        class_list <- list(a=c(label1, label2))
    } else {
        class_list <- purrr::map(2:length(classes), function(i)
            purrr::map(seq_len(i-1), function(j) c(classes[i], classes[j])))
        class_list <- unlist(class_list, recursive=FALSE)
    }

    # test functions
    test_custom <- test_c(test_custom)

    fg_feat <- fg_get_feature_all(fg)
    for (type in c("node","edge")) {
        if (type=="node")
            features <- node_features
        if (type=="edge")
            features <- edge_features

        if (is.null(features)) features <- names(fg_feat[[type]])
        if (length(features)==0) next
        if (features[1]=="NONE") next
        features <- features[features%in%names(fg_feat[[type]])]
        if (length(features)==0) next

        for (feature in features) {
            if (!feature%in%names(fg_feat[[type]])) next
            for (classl in class_list) {
                start1 <- Sys.time()
                message("- claculating summary statistics (for class ",
                        class, ": ", paste0(classl, collapse=" & "), ") ",
                        test_name," for ", type, " feature ", feature)

                id1 <- classes_==classl[1]
                id2 <- classes_==classl[2]
                m <- fg_get_feature(fg, type=type, feature=feature)

                # calculate summary
                fg <- fg_add_summary(fg, type=type, summary_meta=list(
                    feature=feature, test_name=test_name,
                    class=class, label1=classl[1], label2=classl[2]),
                    overwrite=overwrite, p=NULL, summ_fun=fg_summary_,
                    m1=m[id1,,drop=FALSE], m2=m[id2,,drop=FALSE],
                    diminish=diminish, test_custom=test_custom,
                    p_thres=p_thres, p_rate=p_rate,
                    save_functions=save_functions, no_cores=no_cores)


                sm_name <- paste0(c(feature, class, classl), collapse="_")

                if (effect_size) {
                    if (is.null(fg_get_etc(fg)$effect_size))
                        fg@etc$effect_size <- list()
                    if (is.null(fg_get_etc(fg)$effect_size[[type]]))
                        fg@etc$effect_size[[type]] <- list()
                    if (is.null(fg_get_etc(fg)$effect_size[[type]][[sm_name]])) {
                        fg@etc$effect_size[[type]][[sm_name]] <-
                            fg_cohensd_(fg_get_feature(fg, type, feature), id1, id2)
                    }
                }

                if (adjust0 & grepl("SpecEnr", feature)) {
                    if (is.null(fg@etc$adjust0))
                        fg@etc$adjust0 <- list()
                    if (is.null(fg@etc$adjust0[[type]]))
                        fg@etc$adjust0[[type]] <- list()
                    if (is.null(fg@etc$adjust0[[type]][[sm_name]])) {
                        fg@etc$adjust0[[type]][[sm_name]] <-
                            fg_adjust0_(fg@feat[[type]][[feature]], id1, id2,
                                        adjust0_lim=adjust0_lim)
                    }
                }

                if (btwn & grepl("SpecEnr", feature)) {
                    # test functions
                    btwn_test_custom <- test_c(btwn_test_custom)

                    cf <- se_feats(feature)
                    if (is.null(fg@etc$actualVSexpect))
                        fg@etc$actualVSexpect <- list()
                    if (is.null(fg@etc$actualVSexpect[[type]]))
                        fg@etc$actualVSexpect[[type]] <- list()
                    if (is.null(fg@etc$actualVSexpect[[type]][[sm_name]])) {
                        mp <- fg_get_feature(fg, type=type, feature=cf[2])
                        mep <- fg_get_feature(fg, type=type, feature=cf[3])
                        fg@etc$actualVSexpect[[type]][[sm_name]] <-
                            fg_btwn_(id1, id2, mp, mep,
                                     btwn_test_custom=btwn_test_custom)
                    }
                }
                time_output(start1)
            }
        }
    }
    return(fg)
}


# calculates relation between expected and actual raw feature values
fg_btwn_ <- function(id1, id2, mp, mep, btwn_test_custom) {
    id1_ <- id1
    id2_ <- id2
    if (sum(id1_)>sum(id2_)) {
        id1 <- sample(which(id1_),sum(id2_))
    } else if (sum(id2_)>sum(id1_)) {
        id2 <- sample(which(id2_),sum(id1_))
    }

    # whether specenr should be significant based on how different actual
    # and expected raw values are; sigcands are those that are significantly different

    pp3v1 <- sapply(seq_len(ncol(mp)), function(ci)
        btwn_test_custom(mp[id1,ci], mep[id1,ci]) )
    pp3v2 <- sapply(seq_len(ncol(mp)), function(ci)
        btwn_test_custom(mp[id2,ci], mep[id2,ci]) )

    pp3v1[is.na(pp3v1)] <- pp3v2[is.na(pp3v2)] <- 1

    pp3c1 <- sapply(seq_len(ncol(mp)), function(ci)
        effsize::cohen.d(mp[id1,ci], mep[id1,ci])$estimate )
    pp3c2 <- sapply(seq_len(ncol(mp)), function(ci)
        effsize::cohen.d(mp[id2,ci], mep[id2,ci])$estimate )
    pp3c1[is.na(pp3c1)] <- pp3c2[is.na(pp3c2)] <- 0
    # |d|<0.2 "negligible", |d|<0.5 "small", |d|<0.8 "medium", otherwise "large"

    pp3b_raw <- lapply(seq_len(ncol(mp)), function(ci) {
        a <- mp[id1_,ci] - mp[id2_,ci]
        b <- mep[id1_,ci] - mep[id2_,ci]
        return(list(a=a,b=b))
    })
    pp3bt <- sapply(seq_len(ncol(mp)), function(ci)
        btwn_test_custom(
            pp3b_raw[[ci]]$a, pp3b_raw[[ci]]$b) )
    pp3bt[is.na(pp3bt)] <- 1
    pp3bc <- sapply(seq_len(ncol(mp)), function(ci)
        effsize::cohen.d( pp3b_raw[[ci]]$a, pp3b_raw[[ci]]$b)$estimate )
    pp3bc[is.na(pp3bc)] <- 0

    pp3b_raw_ <- lapply(seq_len(ncol(mp)), function(ci) {
        a <- log(mp[id1_,ci] / mp[id2_,ci])
        b <- log(mep[id1_,ci] / mep[id2_,ci])
        return(list(a=a,b=b))
    })
    pp3bt_ <- sapply(seq_len(ncol(mp)), function(ci)
        btwn_test_custom(
            pp3b_raw_[[ci]]$a, pp3b_raw_[[ci]]$b) )
    pp3bt_[is.na(pp3bt_)] <- 1
    pp3bc_ <- sapply(seq_len(ncol(mp)), function(ci)
        effsize::cohen.d( pp3b_raw_[[ci]]$a, pp3b_raw_[[ci]]$b)$estimate )
    pp3bc_[is.na(pp3bc_)] <- 0

    df_ps <- data.frame(
        phenotype=colnames(mp),
        tpv1=pp3v1, tpv2=pp3v2,
        cd1=pp3c1, cd2=pp3c2,
        btp=pp3bt, bcd=pp3bc,
        btp_=pp3bt_, bcd_=pp3bc_
    )
    return(df_ps)
}

# calculates min percentage of 0's between two groups of samples for each SpecEnr value
fg_adjust0_ <- function(m, id1, id2, adjust0_lim=c(-.1, .1)) {
    m <- as.matrix(m)
    xl <- min(sum(id1), length(id1))
    yl <- min(sum(id2), length(id2))
    purrr::map_dbl(seq_len(ncol(m)), function(ci)
        min(sum(m[id1,ci]<adjust0_lim[2] & m[id1,ci]>adjust0_lim[1])/xl,
            sum(m[id2,ci]<adjust0_lim[2] & m[id2,ci]>adjust0_lim[1])/yl))
}

fg_med0_ <- function(m, id1, id2) {
    m <- as.matrix(m)
    purrr::map_dbl(seq_len(ncol(m)), function(ci)
        max(abs(c(median(m[id1,ci]), stats::median(m[id2,ci])))))
}

# calculates effect size statistics cohens d
#' @importFrom purrr map map_dbl map_int
#' @importFrom effsize cohen.d
fg_cohensd_ <- function(m, id1, id2) {
    cd <- purrr::map(seq_len(ncol(m)), function(ci)
        effsize::cohen.d(m[id1,ci], m[id2,ci]))
    cde <- purrr::map_dbl(cd, function(x) x$estimate)
    cde[is.na(cde)] <- 0
    cdd <- purrr::map_int(cd, function(x) x$magnitude)
    cdd[is.na(cdd)] <- 1
    cdd <- factor(cdd)
    levels(cdd) <- levels(cd[[which(!is.na(cdd))[1]]]$magnitude)
    return(list(cohensd=cde, cohensd_size=cdd))
}

# calculates summary statistics
#' @importFrom future plan multisession sequential
#' @importFrom furrr future_map
#' @importFrom purrr map
fg_summary_ <- function(
    fg, m1, m2, type, diminish,
    test_custom, p_thres, p_rate, save_functions,
    no_cores=1
) {
    if (no_cores>1) future::plan(future::multisession)

    # split feature matrix into sample classes
    mnames <- colnames(m1)
    ml <- length(mnames)

    fg_graph <- fg_get_graph(fg)

    # prepare layers
    if (type=="node") layers_ <- fg_graph$v$phenolayer
    if (type=="edge") {
        phen_to <- fg_graph$e$to
        phen_from <- fg_graph$e$from
        layers_ <- fg_graph$v$phenolayer[
            match(phen_to, fg_graph$v$phenotype)]
    }
    layers <- sort(unique(layers_))

    if (!diminish) {
        # calculate p values
        p <- fpurrr_map(seq_len(ml), function(i)
            test_custom(m1[,i], m2[,i]), no_cores=no_cores, prll=ml>1000)

    } else {
        pchild <- fg_get_el(fg)$child

        # p value thresholds determining whether to continue down hierarchy
        p_thress <- rep(p_rate, length(layers))
        if (any(layers==0)) p_thress <- p_thress[-length(p_thress)]
        p_thress[1] <- p_thres
        p_thress <- cumprod(p_thress)
        p_thress <- sort(p_thress, decreasing=TRUE)

        # initialize p values vector
        p <- rep(NA, ml)
        root_ <- mnames==""
        if (sum(root_)>0) {
            p[root_] <- test_custom(m1[,root_], m2[,root_])
            if (is.nan(p[root_])) p[root_] <- 1
        }

        # calculate p values for layer 1
        testis <- testi <- layers_==min(layers_[!root_])
        p_thress_i <- 1
        while (sum(testi)>0) {
            wtesti <- which(testi)
            pl <- unlist(fpurrr_map(wtesti, function(i)
                pt <- test_custom(m1[,i], m2[,i])
                , no_cores=no_cores, prll=length(wtesti)>1000))
            p[testi] <- pl

            pl_cpops <- pl < p_thress[p_thress_i]
            pl_cpops_ <- mnames[which(testi)[pl_cpops]]
            p_thress_i <- p_thress_i + 1
            if (type=="node") {
                testi <- mnames %in% unlist(pchild[pl_cpops_])
            } else {
                good_edges <- mnames %in% pl_cpops_
                testi <- phen_from %in% phen_to[good_edges]
            }
            testis <- testis | testi
        }
    }

    names(p) <- mnames
    p <- unlist(p)
    p_ <- list(values=p)
    if (save_functions)
        p_$test_fun <- test_custom

    future::plan(future::sequential)
    return(p_)
}

library(flowGraph)
# library(ftf***)
library(testthat)

# prepare parallel backend
no_cores <- 1#parallel::detectCores()-1
# future::plan(future::multisession)

## create Phenotypes data ----------------------

data(fg_data_pos30)

# # define marker and cell count
# celln <- 10000
# markern <- 3
# markers <- LETTERS[1:markern]
#
# # define marker thresholds
# cvd <- rnorm(celln,2,1)
# p50 <- quantile(cvd, .5)
# thres <- furrr::future_map(markers, function(x) p50)
# names(thres) <- markers
#
# # generate Phenotypes list
samplen <- 10
# ftl <- furrr::future_map(1:samplen, function(i) {
#     # make flow frame
#     f <- new("flowFrame")
#     f@exprs <- matrix(rnorm(celln*markern,2,1), nrow=celln)
#     colnames(f@exprs) <- markers
#
#     # marker indices in flow frame
#     ci <- c(1:ncol(f@exprs))
#     names(ci) <- colnames(f@exprs)
#
#     # modify experiment samples such that ABC increases by 50%
#     if (i>(samplen/2)) {
#         ap <- f@exprs[,1]>thres[[1]]
#         bp <- f@exprs[,2]>thres[[2]]
#         cp <- f@exprs[,3]>thres[[3]]
#         tm <- sum(ap & bp & cp)/2
#         f@exprs <- rbind(f@exprs, f@exprs[sample(which(ap & bp & cp),tm),])
#     }
#
#     # make Phenotypes
#     ftf***(Frame=f, PropMarkers=ci, MarkerNames=colnames(f@exprs),
#              MaxMarkersPerPop=markern, PartitionsPerMarker=2, Thresholds=thres,
#              Methods='Thresholds', verbose=FALSE, MemLimit=60)#@CellFreqs
# })

meta_file <- data.frame(
    id=1:samplen,
    class=append(rep("control", samplen/2), rep("exp", samplen/2)),
    stringsAsFactors=FALSE
)



## 00 helpers ----------------------------------

# context("00_helpers")

# test_that("ncores", {
#     expect_equal(ncores(1), 1)
#     expect_equal(ncores(Inf), parallel::detectCores())
# })

# test_that("loop_ind_f", {
#     expect_equal(length(loop_ind_f(1,10)), 1)
#     expect_equal(length(loop_ind_f(rep(1,2),10)), 2)
#     expect_equal(length(loop_ind_f(rep(1,10),10)), 10)
#     expect_equal(length(loop_ind_f(rep(1,15),10)), 10)
# })


## 00 flowgraph constructor --------------------

context("00_flowGraph_constructor")

test_that("errors", {
    expect_error(flowGraph(NA))#, "make sure input type is correct")
    expect_error(flowGraph(fg_data_pos30$count[1:2,],
                           meta=fg_data_pos30$meta[1:5,]))
    expect_error(flowGraph(fg_data_pos30$count[1:2,],
                           meta=fg_data_pos30$meta[1:2,-1]))
})

contexts <- c("vector", "matrix") #"phenotype", "phenotype list",
for (context_no in contexts) {
    if (context_no == "phenotype")
        fg <- flowGraph(ftl[[1]], no_cores=no_cores)
    if (context_no == "phenotype list")
        fg <- flowGraph(ftl[1:2], no_cores=no_cores)
    if (context_no == "vector")
        fg <- flowGraph(fg_data_pos30$count[1,], no_cores=no_cores)
    if (context_no == "matrix")
        fg <- flowGraph(fg_data_pos30$count[1:2,], no_cores=no_cores)

    test_that(context_no, {
        expect_equal(nrow(fg@meta), nrow(fg@feat$node$count))
        expect_equal(nrow(fg@graph$v), ncol(fg@feat$node$count))
        expect_equal(nrow(fg@graph$e), ncol(fg@feat$edge$prop))

        expect_equal(nrow(fg@feat_desc$node), length(fg@feat$node))
        expect_equal(nrow(fg@feat_desc$edge), length(fg@feat$edge))

        a = lapply(fg@feat$node, function(x)
            expect_equal(nrow(x), nrow(fg@feat$node$count)) )
        a = lapply(fg@feat$edge, function(x)
            expect_equal(nrow(x), nrow(fg@feat$node$count)) )
    })
}

test_that("graph", {
    expect_setequal(fg@edge_list$child[["A++B++"]],
                 c("A++B++C-", "A++B++C+", "A++B++D-", "A++B++D+"))
    expect_setequal(fg@edge_list$parent[["A++B++"]],
                 c("A++", "B++"))
    expect_setequal(fg@graph$e$from[fg@graph$e$to=="A++B++"],
                 c("A++", "B++"))
})


context("02_flowGraph_features")

test_that("prop", {
    expect_equal(all(fg@feat$node$prop[,1]==1), TRUE)
})

# fg <- flowGraph(fg_data_pos30$count[1:2,], no_cores=no_cores)
fg_ <- flowGraph(fg_data_pos30$count[1:2,], no_cores=no_cores, cumsumpos=TRUE)

test_that("cumsumpos", {
    expect_equal(fg_@feat$node$count[1,"A+"],
                 sum(fg@feat$node$count[1,c("A+","A++")]))
    expect_equal(fg_@feat$node$count[1,"B++"],
                 sum(fg@feat$node$count[1,c("B++","B+++")]))
    expect_equal(fg_@feat$node$count[1,"B++C+"],
                 sum(fg@feat$node$count[1,c("B++C+","B+++C+")]))
    expect_equal(fg_@feat$node$count[1,"A++B++"],
                 sum(fg@feat$node$count[1,c("A++B++","A++B+++")]))
})


context("03_flowGraph_summary")

# fg <- flowGraph(ftl, meta=meta_file, no_cores=no_cores)

fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class, no_cores=no_cores)

fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                overwrite=FALSE, test_name="wilcox", diminish=FALSE)

fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                overwrite=FALSE, test_name="wilcox_diminish", diminish=TRUE)

test_that("summary", {
    expect_equal(length(fg@summary$node), nrow(fg@summary_desc$node))
})

fg <- fg_clear_summary(fg)

test_that("summary clear", {
    expect_equal(fg@summary, list())
    expect_equal(fg@summary_desc, list())
})

#' 'flowGraph': A class for storing cell count feature values
#'  for the Phenotype class.
#' @name flowGraph-class
#' @aliases flowGraph-class
#' @docType class
#' @slot feat A list containing elements \code{node} and \code{edge},
#' each containing a list with feature values;
#'   each element in this list is named by the feature name and contains a
#'   numeric matrix with the sample id's as row names and cell populations
#'   phenotype labels or edge labels as column names. Column names
#'   for edge features are labelled as <from>_<to> e.g. A+_A+B+.
#' @slot feat_desc A list containing elements \code{node} and \code{edge},
#' each containing a data frame describing the features in the \code{feat} slot
#' with columns:
#' \itemize{
#'   \item{\code{feat}: feature name.}
#'   \item{\code{nrow}: number of samples.}
#'   \item{\code{ncol}: number of nodes or edges.}
#'   \item{\code{inf}: number of infinite values in the matrix.}
#'   \item{\code{neginf}: number of negative infinite values in the matrix.}
#'   \item{\code{na}: number of NA values in the matrix.}
#'   \item{\code{nan}: number of NaN values in the matrix.}
#'   \item{\code{neg}: number of negative values in the matrix.}
#'   \item{\code{pos}: number of positive values in the matrix.}
#'   \item{\code{zero}: number of 0's in the matrix.}
#'   \item{\code{max}: The maximum value in the matrix.}
#'   \item{\code{min}: The minimum value in the matrix.}
#' }
#' @slot summary A list containing elements \code{node} and \code{edge},
#'  each containing a list with a feature summary list;
#'  each feature summary in this list contains elements:
#' \itemize{
#'   \item{\code{values}: a numeric vector the same length
#'    as the number of nodes or edges.}
#'   \item{test_custom: a function or a string name
#'    of the summary test method used.}
#' }
#' @slot summary_desc
#'  A list containing elements \code{node} and \code{edge},
#'  each containing a data frame describing the features in \code{feat} with
#'  columns:
#' \itemize{
#'   \item{\code{feat}: A string indicating feature name the
#'    summary was created for.}
#'   \item{\code{test_name}: A string containing the name of the summary.}
#'   \item{\code{class}: A string corresponding to the column name
#'    of the \code{meta} slot whose values represent the class label of
#'    each sample on which the summary was created to compare or analyze.}
#'    \item{\code{label1}: A string from the \code{class} column of the
#'     \code{meta} slot indicating one of the labels compared to create
#'     the summary statistic.}
#'    \item{\code{label2}: A string from the \code{class} column of the
#'     \code{meta} slot indicating one of the labels compared to create
#'     the summary statistic.}
#' }
#' @slot meta A data frame containing the column(s) \code{id} (sample id's
#'  corresponding to row names of features in the \code{feat} slot) and any
#'  other meta data pertaining to samples being analyzed.
#' @slot markers A character vector containing markers used.
#' @slot edge_list A list containing elements \code{child} and \code{parent}.
#'  These elements contain an edge list from child to parent and vice versa.
#' @slot graph A list containing data frames \code{v} and \code{e} with
#'  information on cell population nodes and edges. \code{v} contains columns:
#'  \itemize{
#'    \item{\code{phenotype}: The cell population node label names e.g. A+B+C+.}
#'    \item{\code{phenocode}: A string of "0", "1", "2", ... indicating the
#'     whether each marker is expressed on a cell population.}
#'    \item{\code{phenolayer}: The layer on which a cell population resides i.e.
#'     the numer of markers in its phenotype label.}
#'    \item{\code{phenogroup}: The markers used the make up the phenotype.}
#'  }
#' @slot plot_layout A string indicating the name of the \code{igraph} layout
#'  function used to layout the cell population nodes for plotting.
#' @slot etc A list containing other information
#'  (see \code{\link[flowGraph]{fg_get_summary}} for other things stored
#'  in this slot):
#'  \itemize{
#'    \item{\code{cumsumpos}: A logical indicating whether cell counts
#'     in \code{flowGraph} object contains cumulated cell counts;
#'     this is optional and can be done only for there is
#'     more than one threshold for one or more markers.
#'     This should also only be ran when initializing a \code{flowGraph} object
#'     as converting back and forth is computationally expensive.
#'     If the user is interested in seeing non- and cumulated counts,
#'     we recommend keeping two \code{flowGraph} objects, one for each version.
#'     This function simply converts e.g. the count of A+ or A++ into
#'     the sum of count of A+, A++, and A+++ or A++, and A+++.}
#'    \item{\code{class_mean_normalized}: A logical indicating whether the
#'     features in the flowGraph object has been normalized according to
#'     some sample meta e.g. subject.}
#'     \item{\code{save}: A list containing a string indicating the save ID
#'      of the object
#'  and a string indicating path where the object is saved -- used in function
#'  \code{save_fg} to identify whether or not to save to the same folder.}
#'  }
#' @section Creating Objects: Objects can be created using
#' \code{new("flowFrame")} or the constructor \code{flowGraph},
#' with mandatory argument \code{input_}.
#' Creating objects using \code{new} is discouraged.
#'
#' @section Methods: 'object' represents a \code{flowGraph} object.
#' \itemize{
#'   \item{\code{show(fg)}: Shows a description of the \code{flowGraph} object.}
#'   \item{\code{fg_get_meta}: Retrieves the sample meta data
#'    from a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_get_meta}}.}
#'   \item{\code{fg_get_graph}: Retrieves the cell population (v) and edge (e)
#'    meta data from a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_get_graph}}.}
#'   \item{\code{fg_get_feature}: Retrieves the numeric feature matrix requested
#'    by the user from a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_get_feature}}.}
#'   \item{\code{fg_get_summary}: Retrieves the feature summary list requested
#'    by the user from a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_get_summary}}.}
#'   \item{\code{fg_get_feature_desc}: Retrieves the data frame from the
#'    \code{feat_desc} slot of a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_get_feature_desc}}.}
#'   \item{\code{fg_get_summary_desc}: Retrieves the data frame
#'    from the \code{summary_desc}
#'    slot of a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_get_summary_desc}}.}
#'   \item{\code{fg_add_feature}: Adds a feature to a given
#'    \code{flowGraph} object;
#'    we do not recommend users directly use this method, instead please use
#'    wrapper functions e.g. \code{\link[flowGraph]{fg_feat_node_prop}},
#'    \code{\link[flowGraph]{fg_feat_node_specenr}},
#'    See \code{\link[flowGraph]{fg_add_feature}}.}
#'   \item{\code{fg_rm_feature}: Removes a user specified feature
#'    from a given flowGraph object.
#'    See \code{\link[flowGraph]{fg_rm_feature}}.}
#'   \item{\code{fg_add_summary}: Adds a feature to a given \code{flowGraph}
#'    object;
#'    we do not recommend users directly use this method, instead please use
#'    wrapper function \code{\link[flowGraph]{fg_summary}}.}
#'   \item{\code{fg_clear_summary}: Removes all feature summaries
#'    from a given flowGraph object.
#'    See \code{\link[flowGraph]{fg_clear_summary}}.}
#'   \item{\code{fg_rm_summary}: Removes a user specified feature summaries
#'    from a given flowGraph object.
#'    See \code{\link[flowGraph]{fg_rm_summary}}.}
#'   \item{\code{fg_gsub_markers}:
#'    Substitutes marker names in a given \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_gsub_markers}}.}
#'   \item{\code{fg_gsub_ids}: substitutes sample id's in a \code{flowGraph}
#'    object
#'    See \code{\link[flowGraph]{fg_gsub_ids}}.}
#'   \item{\code{fg_merge_samples}: Merges the samples of two \code{flowGraph}
#'    objects;
#'    we recomment users use the wrapper function
#'     \code{\link[flowGraph]{fg_merge}} instead.
#'    See \code{\link[flowGraph]{fg_merge_samples}}.}
#'   \item{\code{fg_extract_samples}: Extract data for specific samples
#'    from a flowGraph object.
#'    See \code{\link[flowGraph]{fg_extract_samples}}.}
#'   \item{\code{fg_extract_phenotypes}: Extract data for specific
#'    cell population nodes from a \code{flowGraph} object.
#'    See \code{\link[flowGraph]{fg_extract_phenotypes}}.}
#'   \item{\code{fg_merge}: Merges two given \code{flowGraph} objects.
#'    See \code{\link[flowGraph]{fg_merge}}.}
#'   \item{\code{fg_set_layout}: Sets layout for cell population nodes
#'    for the purpose of plotting.
#'    See \code{\link[flowGraph]{fg_set_layout}}.}
#'    \item{\code{fg_plot}: Plots cell hierarchies in the flowGraph object.
#'    See \code{\link[flowGraph]{fg_plot}}.}
#' }
#' @return a flowGraph object.
#' @examples
#'
#'  showClass("flowGraph")
#' @importFrom methods setClass
#' @export
methods::setClass(
    "flowGraph", slots=list(
        feat="list",
        feat_desc="list",#list(node="list", edge="list"),
        summary="list",
        summary_desc="list",#list(node="list", edge="list",
        meta="data.frame",
        markers="character", edge_list="list",
        graph="list", plot_layout="character",
        etc="list")
)



#' @describeIn flowGraph show method
#' @usage \S4method{show}{flowGraph}(object)
#' @param object A flowGraph object.
#' @exportMethod show
#' @export
#' @importFrom methods setMethod show
methods::setMethod(
    "show", "flowGraph", function(object) {
        cat("flowGraph object with ",
            length(fg_get_feature_all(object)$node), "/",
            length(fg_get_summary_all(object)$node)," node and ",
            length(fg_get_feature_all(object)$edge), "/",
            length(fg_get_summary_all(object)$edge),
            " edge feature(s)/summaries.",
            "\n- markers: ", paste0(fg_get_markers(object), collapse=", "),
            "\n- contains: ", nrow(fg_get_graph(object)$v),
            " cell populations and ",
            nrow(fg_get_graph(object)$e), " edges\n", sep="")
        return(list(features=fg_get_feature_desc(object),
                    summaries=fg_get_summary_desc(object)))
    }
)

#' fg_data_fca
#'
#' @format A list containing the following elements
#'  derived from the flowCAP-II AML data set for cell populations up to layer 3.
#' \itemize{
#'   \item{\code{count}: A numeric sample x cell population node
#'   matrix with cell count values.}
#'   \item{\code{meta}: A data frame containing meta information on
#'    samples in \code{count}; it contains columns:}
#'   \itemize{
#'       \item{\code{class}: a string indicating whether a sample is
#'        from a "control" or "aml" subject.}
#'       \item{\code{id}: a string containing sample id's.}
#'       \item{\code{train}: a logical variable indicating whether
#'        a sample is from the train or test set.}
#'       \item{\code{subject}: a numeric variable containing the
#'        id of the subject from whom the sample came from.}
#'       \item{\code{tube}: the tube or panel number;
#'        all samples in this data set is
#'        analyzed under the 6th panel.}
#'   }
#' }
#' @source {
#'   \insertRef{aghaeepour2013critical}{flowGraph}
#' }
"fg_data_fca"

#' fg_data_pos2
#'
#' @format A list containing the following elements
#'  for a positive control data set with markers A, B, C, D. This is a positive
#'  control data set where node A+B+C+ increased by 50%.
#' \itemize{
#'   \item{\code{count}: A numeric sample x cell population node
#'   matrix with cell count values}
#'   \item{\code{meta}: A data frame containing meta information on
#'    samples in \code{count}; it contains columns:}
#'   \itemize{
#'       \item{\code{id}: a string containing sample id's.}
#'       \item{\code{class}: a string indicating whether a sample is
#'        from a "control" or "exp" (experiment) subject.}
#'   }
#' }
"fg_data_pos2"


#' fg_data_pos30
#'
#' @format A list containing the following elements
#'  for a positive control data set with
#'  markers A, B, C, D; note it was made with two and three thresholds
#'  for markers A and B to test functions with multiple thresholds
#'  (this is a positive
#'  control data set where nodes A+..B+..C+ increased by 50%).
#' \itemize{
#'   \item{\code{count}: A numeric sample x cell population node
#'   matrix with cell count values}
#'   \item{\code{meta}: A data frame containing meta information on
#'   samples in \code{count}; it contains columns:}
#'   \itemize{
#'       \item{\code{id}: a string containing sample id's.}
#'       \item{\code{class}: a string indicating whether a sample is
#'        from a "control" or "exp" (experiment) subject.}
#'   }
#' }
"fg_data_pos30"

#' @title Creates a cell hierarchy plot.
#' @description Creates a cell hierarchy plot given a flowGraph object. If a path is not provided for \code{fg_plot} to save the plot, please use \code{plot_gr} to view plot given the output of \code{fg_plot}.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{feature} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param show_nodes_edges A logical vector indicating which nodes/edges (type)
#'  to show in the plot; if this is not specified, only nodes/edges with
#'  significant summary statistics will be shown.
#' @param label_max An integer specifying the maximum number of nodes to label.
#' @param p_thres A double indicating a summary statistic threshold
#'  e.g. if we are plotting a T test summary statistic, we can set the threshold
#'  to .05; nodes with a p-value greater than .05 will not be plotted.
#' @param filter_adjust0 A numeric variable indicating what percentage of
#'  SpecEnr values compared (minimum) should be not close to 0.
#'  Set to 1 to not conduct filtering.
#' @param filter_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant.
#' @param filter_btwn_tpthres A numeric variable between 0 and 1 indicating the
#'  unadjusted T-test p-value threshold used to test whether the actual
#'  and expected feature values used to calculate the specified SpecEnr
#'  feature are significantly different for each sample class. Note this only
#'  needs to be specified for SpecEnr features. Combined
#'  with \code{filter_btwn_es}, we conduct three tests to understand if
#'  there is an actual large difference between actual and expected features:
#'  (1,2) T-test of significance between the actual and expected raw feature value
#'  (e.g. proportion) for samples in each of the compared classes, (3) and the
#'  T-test of significance between the differences of actual and
#'  expected feature values of the two classes. If any two of the three tests
#'  come out as insignificant, we set the p-value for the associated node/edge
#'  to 1.
#' @param filter_btwn_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant -- see \code{filter_btwn_tpthres}.
#' @param node_labels A string vector indicating which node feature(s)
#'  should be used to label a node.
#'  We recommend keeping the length of this vector to below 2.
#'  Set to "NONE" if no p-value labels are needed.
#' @param summary_fun A function that takes in a matrix and outputs a
#'  vector the same length as the number of columns this matrix has;
#'  see \code{\link[flowGraph]{fg_summary}}.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param layout_fun A string representing a function from the
#'  \code{igraph} package that
#'  indicates what layout should be used if a cell hierarchy is to be ploted;
#'  all such functions have prefix \code{layout_}. Only specify if different
#'  from the default one already calculated in the \code{fg} flowGraph object
#'  given.
#' @param show_bgedges A logical variable indicating whether or not
#'  edges not specified for plotting should be plotted as
#'  light grey in the background.
# #' @param mod_layout A logical variable indicating whether or not to re-space
# #'  nodes if the layout is a cell hierarchy and \code{show_bgedges=FALSE}.
#' @param main A string or the title of the plot; if left as \code{NULL},
#'  a default title will be applied.
#' @param interactive A logical variable indicating whether the plot should be
#'  an interactive plot; see package \code{ggiraph}.
#' @param visNet_plot A logical variable indicating if an interactive plot is
#'  chosen, if function should output a visNetwork plot; if set to \code{FALSE},
#'  ggplot's girafe will be used instead.
#' @param path A string indicating the path to where the function should save
#'  the plot; leave as \code{NULL} to not save the plot. Static plots are saved
#'  as PNG, interactive plots are saved as HTML.
#' @param width A numeric variable specifying, in inches,
#'  what the plot width should be.
#' @param height A numeric variable specifying, in inches,
#'  what the plot height should be.
#' @return A list of nodes and edges for plotting with the \code{plot_gr}
#'  function. Other elements in this list include \code{show_bgedges},
#'  which has the same value as parameter \code{show_bgedges}, and \code{main},
#'  the title of the plot.
#' @details \code{fg_plot} takes a flowGraph object as input and returns the
#'  \code{graph} slot of the given object with additional columns to serve as
#'  input into \code{\link[flowGraph]{plot_gr}} for plotting using functions
#'  in the \code{ggplot2} package. Users can choose to save a PNG version of
#'  the plot by filling out the \code{path} parameter with a full path to the
#'  PNG plot. In addition to specifying columns added from
#'  \code{\link[flowGraph]{ggdf}}, \code{fg_plot} also adds label column(s)
#'  whose values serve as labels in the interactive version of the plot.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  gr <- fg_plot(fg, type="node", index=1, label_max=30,
#'    show_nodes_edges=NULL, p_thres=.01, node_labels=c("prop", "expect_prop"),
#'    path=NULL) # set path to a full path to save plot as a PNG
#'  # plot_gr(gr)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{get_phen_meta}}
#'  \code{\link[flowGraph]{ggdf}}
#'  \code{\link[flowGraph]{plot_gr}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#' @rdname fg_plot
#' @export
#' @importFrom htmlwidgets saveWidget
#' @importFrom ggplot2 ggsave
#' @importFrom visNetwork visSave
fg_plot <- function(
    fg, type="node", index=1, summary_meta=NULL, adjust_custom="byLayer",
    show_nodes_edges=NULL,
    label_max=30, p_thres=.05, filter_adjust0=1, filter_es=0,
    filter_btwn_tpthres=1, filter_btwn_es=0,
    # show_nodes_edges: a vector of true false, length equals to number of nodes
    node_labels=c("prop", "expect_prop"),
    summary_fun=colMeans, layout_fun=NULL,
    show_bgedges=TRUE,
    main=NULL, interactive=FALSE, visNet_plot=TRUE,
    path=NULL, width=9, height=9
) {
    # width in inches feat must be list with names node and edge
    # cellhierarchy plots p values

    type <- match.arg(type, c("node", "edge"))

    gr <- ggdf(fg_get_graph(fg))

    # change layout if needed
    if (!is.null(layout_fun)) {
        if (layout_fun!=fg_get_plot_layout(fg))
            gr <- set_layout_graph(gr, layout_fun) # layout cell hierarchy
    }

    # get summary statistics
    summary_meta <- unlist(fg_get_summary_desc(fg)[[type]][
        fg_get_summary_index(fg, type, index, summary_meta),])
    if (!grepl("SpecEnr",unlist(summary_meta["feat"])))
        filter_adjust0 <- 1
    pms <- fg_get_summary(
        fg, type, index, summary_meta, adjust_custom=adjust_custom,
        summary_fun=summary_fun, default_p_thres=p_thres,
        filter_adjust0=filter_adjust0, filter_es=filter_es,
        filter_btwn_tpthres=filter_btwn_tpthres, filter_btwn_es=filter_btwn_es
    )
    id1 <- pms$id1
    id2 <- pms$id2
    m1 <- pms$m1
    m2 <- pms$m2
    p <- pms$values

    # show_nodes_edges: which nodes/edges to show
    sn_ <- p < p_thres
    if (!is.null(show_nodes_edges))
        if (length(show_nodes_edges)==length(m1) &
            all(show_nodes_edges%in%c(TRUE,FALSE)) |
            length(show_nodes_edges)>1)
            sn_ <- show_nodes_edges
    if (sum(sn_)==0) {
        warning("no significant nodes found, maybe increase p_thres?")
        return(NULL)
    }

    if (type=="node") {
        gr$v$v_ind <- gr$v$label_ind <- sn_
        gr$e$e_ind <- gr$e$from%in%gr$v$phenotype[sn_] &
            gr$e$to%in%gr$v$phenotype[sn_]
        if (sum(sn_)>label_max) {
            gr$v$label_ind <- FALSE
            gr$v$label_ind[utils::head(order(p), label_max)] <- TRUE
        }
    } else {
        gr$e$e_ind <- sn_
        gr$v$v_ind <- gr$v$label_ind <-
            gr$v$phenotype%in%unlist(gr$e[sn_,c("to","from")])
        if (sum(gr$v$v_ind)>label_max) {
            gr$v$label_ind <- FALSE
            po <- order(p); po <- po[po %in% which(sn_)]
            while (sum(gr$v$label_ind) < label_max |
                   length(po)==0) {
                po1 <- gr$v$phenotype%in%gr$e[po[1],c("to","from")]
                gr$v$label_ind[po1] <- TRUE
                po <- po[-1]
            }
        }
    }

    # plot title
    if (is.null(main))
        main <- paste0(
            "cell hierarchy plot.\n",
            "- ", type, " feature: ", summary_meta["feat"], ".\n",
            "- p-value: ", summary_meta["test_name"],
            "\n- comparing ",
            summary_meta["class"], ", labels: ", summary_meta["label1"], " & ", summary_meta["label2"], ".")

    # create node labels
    if (type=="node") {
        gr$v$label <- gr$v$label_long <- gr$v$phenotype
        if (node_labels[1]!="NONE")
            gr$v$label <- gr$v$label_long <-
                paste0(gr$v$label, " (",signif(p,3),")")
        if (!"NONE"%in%node_labels) {
            node_labels <-
                node_labels[node_labels%in%names(fg_get_feature_all(fg)[[type]])]
            if (length(node_labels)==0) node_labels <- NULL
            if (type=="node") node_labels <-
                    append(summary_meta["feat"], node_labels)
            if (!is.null(node_labels))
                node_labels <- node_labels[!is.null(node_labels)]
            if (!is.null(node_labels))
                node_labels <- node_labels[!duplicated(node_labels)]
            if (!is.null(node_labels))
                for (label in node_labels) {
                    vls <- list(
                        m1=fg_get_feature_means(fg, "node", label, id=id1),
                        m2=fg_get_feature_means(fg, "node", label, id=id2))
                    vl <- paste0(round(vls$m1,3), "/",
                                 round(vls$m2,3))
                    gr$v$label_long <-
                        paste0(gr$v$label_long, "\n", label, ": ", vl)
                    gr$v$label <- paste0(gr$v$label, " ", vl)
                }
        }
    } else {
        gr$v$label <- gr$v$label_long <- paste0(gr$v$phenotype)
    }

    # node colour, size
    if (type=="node") {
        gr$v$colour <- m2-m1
        gr$v$size <- -log(p)
        gr$v$size[!is.finite(gr$v$size)] <-
            max(gr$v$size[is.finite(gr$v$size)])
        gr$v$size[is.nan(gr$v$size)] <- 0
        ew1 <- fg_get_feature_means(fg, type="edge", feature="prop", id=pms$id1)
        ew2 <- fg_get_feature_means(fg, type="edge", feature="prop", id=pms$id2)
        gr$e$colour <- ew1 - ew2
        gr$e$colour1 <- ew1
        gr$e$colour2 <- ew2
    } else {
        gr$e$colour <- m2-m1
        gr$e$size <- -log(p)
        gr$e$size[!is.finite(gr$e$size)] <-
            max(gr$e$size[is.finite(gr$e$size)])
        gr$e$size[is.nan(gr$e$size)] <- 0
    }

    # optionally space out nodes again if no background edges
    # using: gr$v[,c("x","y")] <- space_hierarchy(gr$v[,c("y","x")])[,c(2,1)]

    # plot and save
    gp <- plot_gr(gr, main=main, show_bgedges=show_bgedges,
                  interactive=interactive,
                  visNet_plot=visNet_plot)
    suppressMessages({
        if (!is.null(path) & !interactive)
            ggplot2::ggsave(ifelse(
                grepl("[.]png$",path, ignore.case=TRUE),
                path, paste0(path, ".png")),
                plot=gp, scale=1, width=width, height=height,
                units="in", dpi=600, limitsize=TRUE)
        if (!is.null(path) & interactive & !visNet_plot)
            htmlwidgets::saveWidget(
                gp, ifelse(grepl("[.]html$",path, ignore.case=TRUE),
                           path, paste0(path, ".html")))
        if (!is.null(path) & interactive & visNet_plot)
            visNetwork::visSave(
                gp, file=ifelse(grepl("[.]html$",path, ignore.case=TRUE),
                                path, paste0(path, ".html")),
                background="white")
    })
    if (is.null(path)) message("use function plot_gr to plot fg_plot output")
    gr$main <- main
    gr$show_bgedges <- show_bgedges
    gr$visNet_plot <- visNet_plot
    gp
    return(gr)
}




## graph plot functions

#' @title Prepares a given node and edge graph list for plotting.
#' @description Prepares a given node and edge graph list
#'  for plotting by function plot_gr;
#'  do not use this function on its own.
#' @param gr0 A list containing data frames \code{e} and \code{v}.
#' @return A list containing data frames \code{e} and \code{v}, each
#'  with additional meta data column.
#' @details code{ggdf} adds to the data frames \code{v} and \code{e} in slot
#'  \code{graph} from a \code{flowGraph} object specifying plotting options as
#'  required by \code{\link[flowGraph]{plot_gr}}:
#'  \itemize{
#'    \item{\code{v}}
#'    \itemize{
#'      \item{\code{size}: a numeric indicating node size.}
#'      \item{\code{colour}: a numeric or string indicating node colour.}
#'      \item{\code{label}: a string indicating the label of a node.}
#'      \item{\code{label_long}: a string indicating teh long label of a node;
#'       used in interactive plots in \code{\link[flowGraph]{plot_gr}}}.
#'      \item{\code{label_ind}: a vector of logical variables indicating which
#'       nodes to add a label to in a static plot.}
#'      \item{\code{v_ind}: a vector of logical variables indicating which
#'       nodes to plot.}
#'    }
#'    \item{\code{e}}
#'    \itemize{
#'      \item{\code{colour}: a numeric or string indicating edge colour.}
# #'      \item{\code{size}: a numeric or string indicating edge size.}
#'      \item{\code{e_ind}: a vector of logical variables indicating which
#'       edges to plot.}
#'    }
#'  }
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  gr_ <- ggdf(fg_get_graph(fg))
#'  head(gr_$v)
#'  head(gr_$e)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{get_phen_meta}}
#'  \code{\link[flowGraph]{plot_gr}}
#' @rdname ggdf
#' @export
ggdf <- function(gr0) {
    list(e=data.frame(gr0$e, colour=0, size=1, e_ind=FALSE),
         v=data.frame(gr0$v,
                      size=1, colour=0,
                      #sizeb=1, colourb="", fill="",
                      label=gr0$v$phenotype,
                      label_ind=FALSE, v_ind=FALSE))
}

#' @title Plots a cell hierarchy.
#' @description Plots a cell hierarchy given the output from \code{fg_plot}, a list of nodes and edges.
#' @param gr A list containing data frames \code{e} and \code{v}.
#' @param main A string containing the plot title. If this is set to NULL, the
#'  function will look for a plot title in the \code{main} slot of \code{gr};
#'  otherwise, this defaults to "".
#' @param show_bgedges A logical variable indicating whether or not
#'  edges not specified for plotting should be plotted as light grey
#'  in the background. If this is \code{NULL}, the function will look for a
#'  \code{show_bgedges} in the \code{show_bgedges} slot of \code{gr};
#'  otherwise, this defaults to \code{TRUE}.
#' @param colour_palette A colour palette e.g. the default palette if the user
#'  sets this to \code{NULL} is \code{c('blue','cyan','yellow','red')}.
#' @param label_coloured A logical indicating whether to colour the node
#'  labels using the same colours as the nodes in the non-interactive plot.
#' @param shiny_plot A logical indicating whether this plot is made for shiny;
#'  users don't need to change this.
#' @param interactive A logical variable indicating whether the plot should be
#'  an interactive plot; see package \code{ggiraph}.
#' @param visNet_plot A logical variable indicating if an interactive plot is
#'  chosen, if function should output a visNetwork plot; if set to \code{FALSE},
#'  ggplot's girafe will be used instead.
#' @param colour_edges A logical variable indicating whether to colour edges if
#'  plotting a node feature summary.
#' @param ... Other parameters for \code{ggplot} if \code{interactive}
#'  is set to \code{FALSE}; other parameters for \code{plot_ly}
#'  if \code{interactive} is set to \code{TRUE}.
#' @return  A \code{ggplot} object if \code{interactive} is set to \code{FALSE};
#'  a \code{ggiraph} object if \code{interactive} is set to \code{TRUE}.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  # fg <- fg_summary(fg, no_cores=no_cores, class="class", control="control",
#'  #                  overwrite=FALSE, test_name="t_byLayer", diminish=FALSE)
#'
#'  gr_summary <- fg_plot(
#'    fg, type="node", p_thres=.05, show_bgedges=TRUE,
#'    path=NULL) # set path to a full path to save plot as a PNG
#'
#'  plot_gr(gr_summary, main=gr_summary$main, show_bgedges=TRUE)
#'
#'  plot_gr(gr_summary, main=gr_summary$main, show_bgedges=TRUE, interactive=TRUE)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_plot}}
#'  \code{\link[flowGraph]{get_phen_meta}}
#'  \code{\link[flowGraph]{ggdf}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
# #'  \code{\link[plotly]{plot_ly}}
#' @rdname plot_gr
#' @export
#' @importFrom ggplot2 ggplot aes_ scale_x_continuous scale_y_continuous
#'  theme element_blank ggtitle scale_fill_brewer geom_segment geom_point
#'  scale_colour_gradientn labs
#' @importFrom ggiraph girafe geom_point_interactive geom_segment_interactive
#' @importFrom ggrepel geom_label_repel
#' @importFrom visNetwork visNetwork visNodes visEdges visOptions visInteraction visHierarchicalLayout visEvents
#' @importFrom purrr map
plot_gr <- function(
    gr, main=NULL, show_bgedges=TRUE, colour_palette=NULL, label_coloured=TRUE,
    shiny_plot=FALSE, interactive=FALSE, visNet_plot=TRUE, colour_edges=FALSE,
    ...
) {
    # gr_v: name x y label size colour sizeb colourb
    # gr_e: from to from.x from.y to.x to.y colour

    # place holder; all variables specified in plotting functions
    # refers to columns names in the given data frame
    size <- 0

    gr_v <- gr$v
    gr_e <- gr$e

    if (is.null(main))
        if (is.null(gr$main)) {
            main <- ""
        } else {
            main <- paste0("(",sum(gr_v$v_ind),"/",
                           nrow(gr_v),") ", gr$main)
        }
    if (!is.null(gr$show_bgedges)) show_bgedges <- gr$show_bgedges
    if (!is.null(gr$interactive)) interactive <- gr$interactive
    if (!is.null(gr$visNet_plot)) show_bgedges <- gr$show_bgedges

    if (shiny_plot) {
        interactive <- TRUE
        if (!visNet_plot) {
            max_x <- max(gr$v$x)
            gr_v$x <- gr$v$y
            gr_v$y <- max_x-gr$v$x
            gr_e$from.x <- gr$e$from.y
            gr_e$from.y <- max_x-gr$e$from.x
            gr_e$to.x <- gr$e$to.y
            gr_e$to.y <- max_x-gr$e$to.x
        }
    }

    if(is.null(colour_palette))
        colour_palette <- c('blue','cyan','yellow','red')

    # prepare base plot
    gp <- ggplot2::ggplot(gr_v, ggplot2::aes_(x=~x, y=~y, ...)) +
        ggplot2::scale_x_continuous(expand=c(0,1)) + # expand x limits
        ggplot2::scale_y_continuous(expand=c(0,1)) + # expand y limits
        # theme_bw()+  # use the ggplot black and white theme
        ggplot2::theme(
            axis.text.x=ggplot2::element_blank(),  # rm x-axis text
            axis.text.y=ggplot2::element_blank(), # rm y-axis text
            axis.ticks=ggplot2::element_blank(),  # rm axis ticks
            axis.title.x=ggplot2::element_blank(), # rm x-axis labels
            axis.title.y=ggplot2::element_blank(), # rm y-axis labels
            panel.background=ggplot2::element_blank(),
            panel.border=ggplot2::element_blank(),
            panel.grid.major=ggplot2::element_blank(), #rm grid labels
            panel.grid.minor=ggplot2::element_blank(), #rm grid labels
            plot.background=ggplot2::element_blank()) +
        ggplot2::ggtitle(main) +
        ggplot2::scale_colour_gradientn(colours=colour_palette) +
        ggplot2::labs(size="-ln(p-value)",
                      col="mean feat diff")

    if (show_bgedges)  # keep greyed out edges on
        gp <- gp +
        ggplot2::geom_segment(
            data=gr_e[!gr_e$e_ind,], color="grey75",
            ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y))

    if (!interactive) {
        if (!all(gr_e$colour==gr_e$colour[1]) &
            !all(gr_e$size==gr_e$size[1])) {
            gp <- gp +
                ggplot2::geom_segment(
                    data=gr_e[gr_e$e_ind,],
                    ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y,
                                  color=~colour, size=~size)) +
                ggplot2::geom_point(
                    data=gr_v[gr_v$v_ind,], colour="grey",
                    ggplot2::aes_(x=~x,y=~y))
        } else {
            if (colour_edges) {
                gp <- gp +
                    ggplot2::geom_segment(
                        data=gr_e[gr_e$e_ind,],
                        ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y,
                                      colour=~colour))
            } else {
                gp <- gp +
                    ggplot2::geom_segment(
                        data=gr_e[gr_e$e_ind,], colour="grey",
                        ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y))
            }
            gp <- gp +
                ggplot2::geom_point(
                    data=gr_v[gr_v$v_ind,],
                    ggplot2::aes_(x=~x, y=~y, color=~colour, size=~size))
            if (label_coloured) {
                gp <- gp +
                    ggrepel::geom_label_repel(
                        data=gr_v[gr_v$label_ind,],
                        ggplot2::aes_(x=~x, y=~y,label=~label, color=~colour),
                        nudge_x=-.1, direction="y", hjust=1,
                        segment.size=0.2,force=1.5)

            } else {
                gp <- gp +
                    ggrepel::geom_label_repel(
                        data=gr_v[gr_v$label_ind,],
                        ggplot2::aes_(x=~x, y=~y,label=~label),
                        nudge_x=-.1, direction="y", hjust=1,
                        segment.size=0.2,force=1.5)
            }
        }
    }
    else if (!visNet_plot) {
        if (!all(gr_e$colour==gr_e$colour[1]) &
            !all(gr_e$size==gr_e$size[1])) {
            gp <- gp +
                ggplot2::geom_segment(
                    data=gr_e[gr_e$e_ind,],
                    ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y,
                                  color=~colour, size=~size,
                                  tooltip=paste0(~from, " > ", ~to))) +
                ggplot2::geom_point(
                    data=gr_v[gr_v$v_ind,], colour="grey",
                    ggplot2::aes_(x=~x,y=~y))
        } else {
            if (colour_edges) {
                gp <- gp +
                    ggplot2::geom_segment(
                        data=gr_e[gr_e$e_ind,],
                        ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y,
                                      colour=~colour))
            } else {
                gp <- gp +
                    ggplot2::geom_segment(
                        data=gr_e[gr_e$e_ind,], colour="grey",
                        ggplot2::aes_(x=~from.x, xend=~to.x, y=~from.y, yend=~to.y))
            }

            if (shiny_plot) {
                gp <- gp + ggiraph::geom_point_interactive(
                    data=gr_v[gr_v$v_ind,],
                    ggplot2::aes_(x=~x, y=~y, color=~colour, size=~size,
                                  tooltip=~label_long, data_id=~phenotype))
            } else {
                gp <- gp + ggiraph::geom_point_interactive(
                    data=gr_v[gr_v$v_ind,],
                    ggplot2::aes_(x=~x, y=~y, color=~colour, size=~size,
                                  tooltip=~label_long, data_id=~phenogroup))
            }
        }

        if (!shiny_plot & interactive) gp <- ggiraph::girafe(ggobj=gp)
    }
    else { # visNet
        v_ind <- gr$v$v_ind
        v_label <- purrr::map_chr(
            stringr::str_split(gr$v$label[v_ind]," "),
            function(x) paste0(x[c(1,2)], collapse=" "))
        nodes <- data.frame(
            id=gr$v$phenotype[v_ind],
            # get only phenotype and p
            label=v_label,
            title=gr$v$label_long[v_ind],
            color=noTOcol(gr$v$colour[v_ind],
                          colourp=colour_palette),
            size=gr$v$size[v_ind],
            group=gr$v$phenogroup[v_ind],
            level=gr$v$phenolayer[v_ind],
            stringsAsFactors=FALSE)

        e_ind <- gr$e$from%in%gr$v$phenotype[v_ind] &
            gr$e$to%in%gr$v$phenotype[v_ind]
        if (sum(e_ind)==0) {
            edges <- data.frame(from=gr$e$from[1], to=gr$e$to[1],
                                hidden=TRUE)
        } else {
            edges <- data.frame(
                from=gr$e$from[e_ind],
                to=gr$e$to[e_ind],
                width=gr$e$size[e_ind],
                label=gr$e$marker[e_ind],
                stringsAsFactors=FALSE)
            if (!is.null(gr$e$size1)) {
                edges$title=paste0(
                    signif(gr$e$size1,3), " vs ",
                    signif(gr$e$size2,3))[e_ind] # tooltip
            }
        }

        gp <- visNetwork::visNetwork(nodes, edges) %>%
            visNetwork::visNodes(shape="dot") %>%
            visNetwork::visEdges(arrows="to") %>%
            visNetwork::visOptions(highlightNearest=TRUE, nodesIdSelection=TRUE) %>%
            visNetwork::visInteraction(hover=TRUE, multiselect=!shiny_plot) %>%
            visNetwork::visHierarchicalLayout(
                parentCentralization=TRUE,
                treeSpacing=50,
                nodeSpacing=300,
                levelSeparation=200
            ) %>%
            visNetwork::visEvents(select="function(nodes) {
                                  Shiny.onInputChange('current_node_selection', nodes.id);
                                  ;}")

}
    # plot(gp)
    return(gp)
    }

# colour palette; cols = a colour for each node
noTOcol <- function(
    values, colourp=c('blue','cyan','yellow','red'), col_names=NULL,
    colour_n=1000) {
    colorFunc <- grDevices::colorRampPalette(colourp)
    z <- pretty(c(min(values), max(values))) # colour breakpoints
    colinds <- unlist(
        lapply(seq_len(length(values)), function(i)
            max(1, ceiling(
                (values-min(z))*1000/(max(z)-min(z)))[i]) ))
    cols=colorFunc(colour_n)[colinds]
    if (is.null(col_names) & !is.null(names(values)))
        col_names <- names(values)
    names(cols) <- col_names
    return(cols)
}


#' @title Creates a QQ plot of a summary statistic.
#' @description Creates a QQ plot of a summary statistic.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{feature} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param logged A logical indicating whether or not to log the  summary
#'  statistic p value.
#' @param p_thres A double indicating a summary statistic threshold
#'  e.g. if we are plotting a T-test summary statistic, we can set the threshold
#'  to .05; nodes with a p-value greater than .05 will not be plotted.
#' @param filter_adjust0 A numeric variable indicating what percentage of
#'  SpecEnr values compared (minimum) should be not close to 0.
#'  Set to 1 to not conduct filtering.
#' @param filter_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant.
#' @param filter_btwn_tpthres A numeric variable between 0 and 1 indicating the
#'  unadjusted T-test p-value threshold used to test whether the actual
#'  and expected feature values used to calculate the specified SpecEnr
#'  feature are significantly different for each sample class. Note this only
#'  needs to be specified for SpecEnr features. Combined
#'  with \code{filter_btwn_es}, we conduct three tests to understand if
#'  there is an actual large difference between actual and expected features:
#'  (1,2) T-test of significance between the actual and expected raw feature value
#'  (e.g. proportion) for samples in each of the compared classes, (3) and the
#'  T-test of significance between the differences of actual and
#'  expected feature values of the two classes. If any two of the three tests
#'  come out as insignificant, we set the p-value for the associated node/edge
#'  to 1.
#' @param filter_btwn_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant -- see \code{filter_btwn_tpthres}.
#' @param shiny_plot A logical indicating whether this plot is made for shiny;
#'  users don't need to change this.
#' @param main A string or the title of the plot; if left as \code{NULL},
#'  a default title will be applied.
#' @param interactive A logical indicating whether or not plot should be an
#'  interactive ggiraph plot as opposed to a static plot.
#' @param path A string indicating the path to where the function should save
#'  the plot; leave as \code{NULL} to not save the plot. Static plots are saved
#'  as PNG, interactive plots are saved as HTML.
#' @return A static or interactive qq plot.
#' @details The interactive plot is made using the \code{ggiraph} package.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_plot_qq(fg, type="node", summary_meta=NULL, adjust_custom="byLayer", index=1,
#'          interactive=TRUE, logged=FALSE)
#'
#'  fg_plot_qq(fg, type="node", summary_meta=NULL, adjust_custom="byLayer", index=1,
#'          interactive=FALSE, logged=FALSE)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_plot}}
#'  \code{\link[flowGraph]{plot_gr}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#' @rdname fg_plot_qq
#' @export
#' @importFrom htmlwidgets saveWidget
#' @importFrom ggiraph girafe geom_point_interactive
#' @importFrom ggplot2 ggplot ggtitle geom_point geom_abline labs ggsave aes_
fg_plot_qq <- function(
    fg, type="node", index=1, summary_meta=NULL, adjust_custom="byLayer",
    logged=TRUE, p_thres=.05,
    filter_adjust0=1, filter_es=0,
    filter_btwn_tpthres=1, filter_btwn_es=0,
    shiny_plot=FALSE,
    main=NULL, interactive=FALSE, path=NULL
) {
    type <- match.arg(type, c("node", "edge"))

    if (shiny_plot) interactive <- TRUE

    index <- fg_get_summary_index(
        fg,type=type, index, summary_meta)
    summary_meta <- unlist(fg_get_summary_desc(fg)[[type]][index,])
    if (!grepl("SpecEnr",unlist(summary_meta["feat"])))
        filter_adjust0 <- 1
    qvals_ <- fg_get_summary(
        fg, type, index, summary_meta, default_p_thres=p_thres,
        filter_adjust0=filter_adjust0, filter_es=filter_es,
        filter_btwn_tpthres=filter_btwn_tpthres, filter_btwn_es=filter_btwn_es,
        adjust_custom=adjust_custom
    )
    qvals <- qvals_$values

    if (is.null(main))
        main <- paste0(
            "(", sum(qvals<p_thres), "/", length(qvals), ") ",
            ifelse(logged,"logged ",""), "qq plot.\n",
            "- ", type, " feature: ", summary_meta["feat"], ".\n",
            "- p-value: ", summary_meta["test_name"],
            "\n- comparing ", summary_meta["class"], ", labels: ", summary_meta["label1"], " & ", summary_meta["label2"], ".")

    qo <- order(qvals)

    uni <- seq_len(length(qvals))/(length(qvals)+1)

    p_thres_ <- p_thres
    if (logged) {
        qvals <- log(qvals)
        if (any(qvals==-Inf)) {
            warning("converting log(0) p values to minimum value for display")
            qvals[qvals==-Inf] <- min( qvals[is.finite(qvals)])
        }
        uni <- log(uni)
        p_thres <- log(p_thres)
    }



    df <- data.frame(y=qvals, cohensd_size=qvals_$cohensd_size,
                     phenotype=names(qvals),
                     phenogroup=fg_get_graph(fg)$v$phenogroup,
                     d_size=colMeans(fg_get_feature(fg, "node", "count")))
    df <- df[qo,]
    df$x <- uni
    qp <- ggplot2::ggplot(df, ggplot2::aes_(
        y=~y, x=~x, colour=~cohensd_size, alpha=.3, stroke=1)) +
        ggplot2::geom_abline(intercept=0, slope=1) +
        ggplot2::ggtitle(main) +
        ggplot2::labs(x="uniform distribution", y="p-value",
                      col="cohen's d", size="count mean") +
        ggplot2::geom_hline(yintercept=p_thres)

    if (interactive) {
        if (shiny_plot) {
            qp <- qp + ggiraph::geom_point_interactive(alpha=.4,
                                                       ggplot2::aes_(tooltip=~phenotype, data_id=~phenotype, size=~d_size))
        } else {
            qp <- qp + ggiraph::geom_point_interactive(shape=1,
                                                       ggplot2::aes_(tooltip=~phenotype, data_id=~phenogroup, size=~d_size))
        }
        if (!shiny_plot) {
            qp <- ggiraph::girafe(ggobj=qp)
            if (!is.null(path))
                htmlwidgets::saveWidget(
                    qp, ifelse(grepl("[.]html$",path, ignore.case=TRUE),
                               path, paste0(path, ".html")))
        }
    } else {
        qp <- qp + ggplot2::geom_point(shape=1, ggplot2::aes_(size=~d_size))
        if (!is.null(path))
            suppressMessages({
                ggplot2::ggsave(
                    ifelse(grepl("[.]png$",path, ignore.case=TRUE),
                           path, paste0(path, ".png")),
                    plot=qp, scale=1, width=5, height=5,
                    units="in", dpi=500, limitsize=TRUE)
            })
    }
    return(qp)
}


#' @title Creates a boxplot of the values of one node/edge
#' @description Creates a boxplot comparing the
#'  features of samples belonging to different classes corresponding
#'  to an existing summary statistic using ggplot2.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{feature} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param node_edge An integer/index of or a string of the cell population (node) /
#'  edge name (edge) the user wants to plot.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param p_thres A numeric variable indicating a p-value threshold
#' @param filter_adjust0 A numeric variable indicating what percentage of
#'  SpecEnr values compared (minimum) should be not close to 0.
#'  Set to 1 to not conduct filtering.
#' @param filter_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant.
#' @param filter_btwn_tpthres A numeric variable between 0 and 1 indicating the
#'  unadjusted T-test p-value threshold used to test whether the actual
#'  and expected feature values used to calculate the specified SpecEnr
#'  feature are significantly different for each sample class. Note this only
#'  needs to be specified for SpecEnr features. Combined
#'  with \code{filter_btwn_es}, we conduct three tests to understand if
#'  there is an actual large difference between actual and expected features:
#'  (1,2) T-test of significance between the actual and expected raw feature value
#'  (e.g. proportion) for samples in each of the compared classes, (3) and the
#'  T-test of significance between the differences of actual and
#'  expected feature values of the two classes. If any two of the three tests
#'  come out as insignificant, we set the p-value for the associated node/edge
#'  to 1.
#' @param filter_btwn_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant -- see \code{filter_btwn_tpthres}.
#' @param paired A logical indicating whether the summary is paired.
#' @param dotplot A logical indicating whether or not to plot sample points.
#' @param outlier A logical indicating whether or not outliers should be plotted.
#' @param all_labels A logical indicating whether or not to plot samples of all
#'  classes outside of just those used in the summary statistic test.
#' @param show_mean A logical indicating whether or not to label the mean.
#' @param main A string or the title of the plot; if left as \code{NULL},
#'  a default title will be applied.
#' @param path A string indicating the path to where the function should save
#'  the plot; leave as \code{NULL} to not save the plot. Static plots are saved
#'  as PNG.
#' @return A static boxplot.
#' @details The plot is made using the \code{ggplot2} package. The interactive
#'  version is the same as the static version, it is only here to support the
#'  shiny app.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_plot_box(fg, type="node", summary_meta=NULL, adjust_custom="byLayer", index=1, node_edge=10)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_plot}}
#'  \code{\link[flowGraph]{plot_gr}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_plot_qq}}
#' @rdname fg_plot_box
#' @export
#' @importFrom htmlwidgets saveWidget
#' @importFrom ggplot2 ggsave ggplot ggtitle geom_point geom_dotplot stat_summary geom_line aes_ position_dodge labs
#' @importFrom ggiraph girafe geom_boxplot_interactive
#' @importFrom stringr str_split
#' @importFrom grDevices boxplot.stats
#' @importFrom utils head tail
fg_plot_box <- function(
    fg, type="node", index=1, summary_meta=NULL, node_edge=1,
    adjust_custom="byLayer", p_thres=.05,
    filter_adjust0=.5, filter_es=.5,
    filter_btwn_tpthres=.05, filter_btwn_es=.5,
    paired=FALSE, dotplot=TRUE, outlier=TRUE, all_labels=FALSE, show_mean=TRUE,
    main=NULL, path=NULL
) {
    type <- match.arg(type, c("node", "edge"))

    index <- fg_get_summary_index(fg,type="node",index,summary_meta)
    summary_meta <- unlist(fg_get_summary_desc(fg)[[type]][index,])
    pp  <- fg_get_summary(
        fg, type, index, summary_meta, default_p_thres=p_thres,
        filter_adjust0=filter_adjust0, filter_es=filter_es,
        filter_btwn_tpthres=filter_btwn_tpthres, filter_btwn_es=filter_btwn_es,
        adjust_custom=adjust_custom
    )
    p <- pp$values
    node_edge <-
        ifelse(is.character(node_edge),
               which(names(p)==node_edge), node_edge)

    feature <- summary_meta["feat"]
    features <- se_feats(feature)
    fg_meta <- fg_get_meta(fg)
    if (all_labels) {
        id_ord <- order(fg_meta[,summary_meta["class"]])
        class_ <- fg_meta[id_ord,summary_meta["class"]]
        dta <- data.frame(
            val=fg_get_feature(fg, type, feature)[id_ord,node_edge],
            class=class_,
            ID=unlist(purrr::map(unique(class_), function(x)
                seq_len(sum(class_==x))))) # paired assum ordered!

    } else {
        a = fg_get_feature(fg, type, feature)[pp$id1,node_edge]
        b = fg_get_feature(fg, type, feature)[pp$id2,node_edge]

        classes <- c(summary_meta["label1"], summary_meta["label2"])
        class_ <- append(rep(classes[1], length(a)),
                         rep(classes[2], length(b)))
        dta <- data.frame(
            val=append(a,b),
            class=class_,
            ID=append(seq_len(length(a)),
                      seq_len(length(b)))) # paired
    }

    if (is.null(main))
        main <- paste0(
            "boxplot (",
            ifelse(is.numeric(node_edge),
                   names(p)[node_edge], node_edge),").\n",
            "- ", type, " feature: ", feature ," (",summary_meta["feat"],").\n",
            "- p-value (p=", round(p[node_edge],3),"): ",
            summary_meta["test_name"], "\n- comparing ",
            summary_meta["class"], ", labels: ",
            "\n    ", summary_meta["label1"], " (mean=",
            round(pp$m1[node_edge],3),") & ", summary_meta["label2"],
            " (mean=",round(pp$m2[node_edge],3),").")

    # plot

    gp <- ggplot2::ggplot(
        dta, ggplot2::aes_(x=~class, y=~val, fill=~class)) +
        ggplot2::labs(y=paste0(
            type, " feature values (",ifelse(
                is.numeric(node_edge),
                names(p)[node_edge], node_edge),")"))

    gp <- gp + ggplot2::geom_boxplot(outlier.shape=ifelse(outlier,19,NA))

    if (paired)
        gp <- gp + ggplot2::geom_line(
            ggplot2::aes_(group=~ID), colour="black", linetype="11")

    if (dotplot)
        gp <- gp + ggplot2::geom_dotplot(
            binaxis='y', stackdir='center',
            position=ggplot2::position_dodge(1), dotsize=.5)

    gp <- gp + ggplot2::ggtitle(main)

    if (show_mean)
        gp <- gp + ggplot2::stat_summary(fun=mean, geom="point", shape=23, size=3)


    if (!outlier) {
        bstats <- purrr::map(unique(class_), function(x)
            grDevices::boxplot.stats(dta$val[class_==x])$stats)
        ymax <- max(purrr::map_dbl(bstats, utils::tail, 1))
        ymin <- min(purrr::map_dbl(bstats, utils::head, 1))

        gp <- gp + ggplot2::ylim(ymin, ymax)
    }

    if (!is.null(path))
        ggplot2::ggsave(
            ifelse(grepl("[.]png$",path, ignore.case=TRUE),
                   path, paste0(path, ".png")),
            plot=gp, scale=1, width=5, height=5,
            units="in", dpi=500, limitsize=TRUE)

    suppressMessages({ gp })
}


# plot a set of boxplots for SpecEnr
#' @importFrom gridExtra arrangeGrob
#' @importFrom ggplot2 element_blank coord_cartesian aes_
fg_plot_box_set <- function(
    fg, type="node", index=1, summary_meta=NULL,
    adjust_custom="byLayer", node_edge=1,
    filter_adjust0=.5, filter_es=.5,
    filter_btwn_tpthres=.05, filter_btwn_es=.5,
    paired=FALSE, dotplot=TRUE, outlier=TRUE, show_mean=TRUE,
    main=NULL, path=NULL
) {
    type <- match.arg(type, c("node", "edge"))

    index <- fg_get_summary_index(fg,type="node",index,summary_meta)
    summary_meta <- unlist(fg_get_summary_desc(fg)[[type]][index,])
    feature <- summary_meta["feat"]
    if (!grepl("SpecEnr",feature)) {
        fg_plot_box(
            fg, type=type, index=index, summary_meta=summary_meta,
            adjust_custom=adjust_custom, node_edge=node_edge,
            paired=paired, dotplot=dotplot, outlier=outlier, all_labels=FALSE,
            main=main, path=path,
            filter_adjust0=filter_adjust0, filter_es=filter_es,
            filter_btwn_tpthres=filter_btwn_tpthres,
            filter_btwn_es=filter_btwn_es
        )
    }
    node_edge <- ifelse(is.character(node_edge),
                        which(fg_get_graph(fg)$v$phenotype==node_edge),
                        node_edge)
    if (length(node_edge)==0) {
        warning("node/edge not found")
        return(NULL)
    }
    pp <- fg_get_summary(
        fg, type="node", index, summary_meta, adjust_custom=adjust_custom,
        filter_adjust0=filter_adjust0, filter_es=filter_es,
        filter_btwn_tpthres=filter_btwn_tpthres, filter_btwn_es=filter_btwn_es)
    p <- pp$values
    dfb <- pp$btwn
    rownames(dfb) <- dfb$phenotype

    features <- se_feats(feature)
    a1 <- fg_get_feature(fg, type, features[2])[pp$id1,node_edge]
    a2 <- fg_get_feature(fg, type, features[2])[pp$id2,node_edge]
    b1 <- fg_get_feature(fg, type, features[3])[pp$id1,node_edge]
    b2 <- fg_get_feature(fg, type, features[3])[pp$id2,node_edge]

    a = fg_get_feature(fg, type, feature)[pp$id1,node_edge]
    b = fg_get_feature(fg, type, feature)[pp$id2,node_edge]
    al <- length(a)
    bl <- length(b)

    classes <- c(summary_meta["label1"], summary_meta["label2"])
    dta <- data.frame(
        val=c(a,b, a1,a2, b1,b2),
        feature=rep(features, each=al+bl),
        class=rep(append(rep(classes[1],length(a)),
                         rep(classes[2],length(b))),3),
        ID=rep(append(seq_len(length(a)),
                      seq_len(length(b))),3) # paired
    )


    if (is.null(main))
        main <- paste0(
            "boxplot (",
            ifelse(is.numeric(node_edge),
                   names(p)[node_edge], node_edge),").\n",
            "- ", type, " feature: ", feature ," (",summary_meta["feat"],").\n",
            "- p-value (p=", round(p[node_edge],3),"): ",
            summary_meta["test_name"], "\n- comparing ",
            summary_meta["class"], ", labels: ",
            "\n    ", summary_meta["label1"], " (mean=",
            round(pp$m1[node_edge],3),") & ",
            summary_meta["label2"], " (mean=",
            round(pp$m2[node_edge],3),").")

    # specenr/actual/expect class 1 vs class 2
    gp <- ggplot2::ggplot(dta[dta$feature==feature,],
                          ggplot2::aes_(x=~class, y=~val, fill=~class)) +
        ggplot2::labs(y=paste0(type, " SpecEnr feature value")) +
        ggplot2::geom_boxplot(outlier.shape=ifelse(outlier,19,NA)) +
        ggplot2::ggtitle(paste0(
            "(",classes[2],"-",length(a)," vs ",
            classes[2],"-",length(b),")\n",main))

    if (show_mean)
        gp <- gp + ggplot2::stat_summary(fun=mean, geom="point", shape=23, size=3)

    gpae <- ggplot2::ggplot(dta[dta$feature!=features[1],],
                            ggplot2::aes_(x=~class, y=~val, fill=~class)) +
        ggplot2::labs(y=paste0(type, " raw feature values")) +
        ggplot2::ggtitle(paste0("feature: ", features[2])) +
        ggplot2::geom_boxplot(outlier.shape=ifelse(outlier,19,NA)) +
        ggplot2::theme(legend.position="none") +
        ggplot2::facet_grid(cols=ggplot2::vars(feature))

    if (show_mean)
        gpae <- gpae + ggplot2::stat_summary(fun=mean, geom="point", shape=23, size=3)

    # class 1/2 actual vs expected
    gp12 <- ggplot2::ggplot(dta[dta$feature!=feature,],
                            ggplot2::aes_(x=~feature, y=~val))+
        ggplot2::labs(y=paste0(type, " raw feature values")) +
        ggplot2::ggtitle(paste0(
            "diff btwn diff (p=",
            signif(dfb$btp[node_edge],3),
            ", CohenD=",signif(dfb$bcd[node_edge],3),")")) +
        ggplot2::geom_boxplot(outlier.shape=ifelse(outlier,19,NA)) +
        ggplot2::theme(legend.position="none") +
        ggplot2::facet_grid(cols=ggplot2::vars(class))

    if (show_mean)
        gp12 <- gp12 + ggplot2::stat_summary(fun=mean, geom="point", shape=23, size=3)

    if (paired) {
        gp <- gp + ggplot2::geom_line(
            ggplot2::aes_(group=~ID), colour="black", linetype="11")
        gpae <- gpae + ggplot2::geom_line(
            ggplot2::aes_(group=~ID), colour="black", linetype="11")
    }

    if (dotplot) {
        gp <- gp + ggplot2::geom_dotplot(
            binaxis='y', stackdir='center',
            position=ggplot2::position_dodge(1), dotsize=.3)
        gpae <- gpae + ggplot2::geom_dotplot(
            binaxis='y', stackdir='center',
            position=ggplot2::position_dodge(1), dotsize=.3)
        gp12 <- gp12 + ggplot2::geom_dotplot(
            binaxis='y', stackdir='center',
            position=ggplot2::position_dodge(1), dotsize=.3)
    }

    if (!outlier) {
        ylim1 = grDevices::boxplot.stats(a)$stats[c(1,5)]
        ylim2 = grDevices::boxplot.stats(b)$stats[c(1,5)]
        gp <- gp + ggplot2::coord_cartesian(
            ylim=c(min(ylim1[1],ylim2[1]), max(ylim1[2],ylim2[2])))
        ylim1 = grDevices::boxplot.stats(a1)$stats[c(1,5)]
        ylim2 = grDevices::boxplot.stats(a2)$stats[c(1,5)]
        ylim1_ = grDevices::boxplot.stats(b1)$stats[c(1,5)]
        ylim2_ = grDevices::boxplot.stats(b2)$stats[c(1,5)]
        gpae <- gpae + ggplot2::coord_cartesian(
            ylim=c(min(ylim1[1],ylim2[1],ylim1_[1],ylim2_[1]),
                   max(ylim1[2],ylim2[2],ylim1_[2],ylim2_[2])))
        ylim1 = grDevices::boxplot.stats(a1)$stats[c(1,5)]
        ylim2 = grDevices::boxplot.stats(b1)$stats[c(1,5)]
        ylim1_ = grDevices::boxplot.stats(a2)$stats[c(1,5)]
        ylim2_ = grDevices::boxplot.stats(b2)$stats[c(1,5)]
        gp12 <- gp12 + ggplot2::coord_cartesian(
            ylim=c(min(ylim1[1],ylim2[1],ylim1_[1],ylim2_[1]),
                   max(ylim1[2],ylim2[2],ylim1_[2],ylim2_[2])))
    }

    gp_grid <- gridExtra::arrangeGrob(
        gp, gpae, gp12,# gp1, gp2,
        layout_matrix=cbind(c(1,1,2,3),c(1,1,2,3)))

    if (!is.null(path)) {
        suppressMessages({
            ggplot2::ggsave(ifelse(grepl(
                "[.]png$",path, ignore.case=TRUE),
                path, paste0(path, ".png")),
                plot=gp_grid, scale=1, width=5, height=10,
                units="in", dpi=500, limitsize=TRUE)
        })
    }
    gp_grid
}



#' @title Creates a p value vs feature difference plot
#' @description Creates a p value vs feature difference plot where the
#'  difference is that of the
#'  features of samples belonging to different classes corresponding
#'  to an existing summary statistic.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{feature} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param logged A logical indicating whether or not to log the summary
#'  statistic p value.
#' @param label_max An integer indicating the maximum number of max difference
#'  and/or min p value nodes/edges that should be labelled.
#' @param p_thres A numeric variable indicating a p-value threshold; a line will
#'  be plotted at this threshold.
#' @param filter_adjust0 A numeric variable indicating what percentage of
#'  SpecEnr values compared (minimum) should be not close to 0.
#'  Set to 1 to not conduct filtering.
#' @param filter_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant.
#' @param filter_btwn_tpthres A numeric variable between 0 and 1 indicating the
#'  unadjusted T-test p-value threshold used to test whether the actual
#'  and expected feature values used to calculate the specified SpecEnr
#'  feature are significantly different for each sample class. Note this only
#'  needs to be specified for SpecEnr features. Combined
#'  with \code{filter_btwn_es}, we conduct three tests to understand if
#'  there is an actual large difference between actual and expected features:
#'  (1,2) T-test of significance between the actual and expected raw feature value
#'  (e.g. proportion) for samples in each of the compared classes, (3) and the
#'  T-test of significance between the differences of actual and
#'  expected feature values of the two classes. If any two of the three tests
#'  come out as insignificant, we set the p-value for the associated node/edge
#'  to 1.
#' @param filter_btwn_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant -- see \code{filter_btwn_tpthres}.
#' @param shiny_plot A logical indicating whether this plot is made for shiny;
#'  users don't need to change this.
#' @param nodes_max An integer indicating maximum number of nodes to plot;
#'  this limit is set for interactive plots only.
#' @param main A string or the title of the plot; if left as \code{NULL},
#'  a default title will be applied.
#' @param interactive A logical variable indicating whether the plot should be
#'  an interactive plot; see package \code{ggiraph}.
#' @param path A string indicating the path to where the function should save
#'  the plot; leave as \code{NULL} to not save the plot. Static plots are saved
#'  as PNG.
#' @return A static or interactive p value vs difference plot.
#' @details The interactive plot is made using the \code{ggiraph} package.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  gp <- fg_plot_pVSdiff(fg, type="node", summary_meta=NULL,
#'                        adjust_custom="byLayer", index=1, label_max=10)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_plot}}
#'  \code{\link[flowGraph]{plot_gr}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_plot_qq}}
#' @rdname fg_plot_pVSdiff
#' @export
#' @importFrom ggplot2 ggsave ggplot ggtitle geom_point labs geom_vline aes_
#' @importFrom ggrepel geom_text_repel
#' @importFrom ggiraph girafe geom_point_interactive
#' @importFrom htmlwidgets saveWidget
#' @importFrom utils head tail
fg_plot_pVSdiff <- function(
    fg, type="node", index=1, summary_meta=NULL, adjust_custom="byLayer",
    logged=TRUE, label_max=5, p_thres=.05,
    filter_adjust0=1, filter_es=0,
    filter_btwn_tpthres=1, filter_btwn_es=0,
    shiny_plot=FALSE, nodes_max=30,
    main=NULL, interactive=FALSE, path=NULL
) {
    type <- match.arg(type, c("node", "edge"))

    if (shiny_plot) interactive <- TRUE

    index <- fg_get_summary_index(fg, type, index, summary_meta)
    summary_meta <- unlist(fg_get_summary_desc(fg)[[type]][index,])
    if (!grepl("SpecEnr",unlist(summary_meta["feat"])))
        filter_adjust0 <- 1
    pp <- fg_get_summary(
        fg, type="node", index, summary_meta, default_p_thres=p_thres,
        filter_adjust0=filter_adjust0, filter_es=filter_es,
        filter_btwn_tpthres=filter_btwn_tpthres, filter_btwn_es=filter_btwn_es,
        adjust_custom=adjust_custom
    )
    p <- pp$values
    mse_c = pp$m1
    mse_m = pp$m2
    mse_ = mse_m - mse_c

    p_ <- p
    if (logged) {
        p_ = -log(p)
        p_[is.infinite(p_)] = max(p_[!is.infinite(p_)])
    }
    dta <- data.frame(log_p_value=p_, difference=mse_,
                      cohensd_size=pp$cohensd_size)
    dta <- cbind(dta, fg_get_graph(fg)$v)
    dta$phenotype_ <- names(p)
    dta$avg_count <- colMeans(fg_get_feature(fg, "node", "count"))

    if (is.null(main))
        main <- paste0(
            "(",sum(p<p_thres),"/",length(p),") ",
            "-ln(p-value) vs feature difference plot.\n",
            "- ", type, " feature: ", summary_meta["feat"], ".\n",
            "- p-value: ", summary_meta["test_name"],
            "\n- comparing ",
            summary_meta["class"], ", labels: ", summary_meta["label1"], " & ",
            summary_meta["label2"], ".")


    if (!interactive) {
        label_ind <- p<p_thres &
            (names(p) %in%
                 names(utils::head(sort(p),label_max)) |
                 names(p) %in%
                 names(utils::tail(
                     sort(abs(mse_)),label_max)))

        dta$phenotype <- ifelse(label_ind, names(p), "")

        gp <- ggplot2::ggplot(
            dta, ggplot2::aes_(x=~log_p_value, y=~difference,
                               color=~cohensd_size)) +
            ggplot2::geom_point(shape=1, ggplot2::aes_(size=~avg_count)) +
            ggplot2::ggtitle(main) +
            ggplot2::labs(x=ifelse(logged,"-ln(p-value)","p-value"),
                          y="difference between mean feature values",
                          col="cohen's d", size="mean count") +
            ggplot2::geom_vline(xintercept=ifelse(logged,-log(p_thres),p_thres))
        if (any(label_ind))
            gp <- gp + ggrepel::geom_text_repel(
                ggplot2::aes_(label=~phenotype),
                nudge_x      = -0.35,
                direction    = "y",
                hjust        = 1,
                segment.size = 0.2
            )
        if (!is.null(path))
            suppressMessages({
                ggplot2::ggsave(ifelse(grepl(
                    "[.]png$",path, ignore.case=TRUE),
                    path, paste0(path, ".png")), plot=gp)
            })
    } else {
        dta$phenotype <- names(p)
        dta$phenogroup <- fg_get_graph(fg)$v$phenogroup
        if (shiny_plot & nodes_max!=Inf) {
            node_ind <-
                (names(p) %in%
                     names(utils::head(sort(p),nodes_max)) |
                     names(p) %in%
                     names(utils::tail(sort(abs(mse_)),nodes_max)))
            if (sum(node_ind)==0) return(NULL)
            dta <- dta[node_ind,]
        }

        start1 <- Sys.time()
        gp <- ggplot2::ggplot(
            dta, ggplot2::aes_(x=~log_p_value, y=~difference,
                               color=~cohensd_size)) +
            ggplot2::ggtitle(main) +
            ggplot2::labs(x="-ln(p-value)",
                          y="difference between mean feature values",
                          col="cohen's d", size="mean count") +
            ggplot2::geom_vline(xintercept=-log(p_thres))
        if (shiny_plot) {
            gp <- gp + ggiraph::geom_point_interactive(
                ggplot2::aes_(tooltip=~phenotype, data_id=~phenotype,
                              size=~avg_count), alpha=.3)
        } else {
            gp <- gp + ggiraph::geom_point_interactive(
                ggplot2::aes_(tooltip=~phenotype, data_id=~phenogroup,
                              size=~avg_count))
            gp <- ggiraph::girafe(ggobj=gp)
        }
        gp

        if (!is.null(path))
            htmlwidgets::saveWidget(
                gp, ifelse(grepl("[.]html$", path, ignore.case=TRUE),
                           path, paste0(path, ".html")))
    }
    return(gp)

}





#' @title Saves numerous plots for all summary statistics to a folder.
#' @description Saves numerous plots for all summary statistics in a given
#'  flowGraph object to a user specified folder.
#' @param fg flowGraph object.
#' @param plot_path A string indicating the folder path to where the function
#'  should save the plots.
#' @param plot_types A string or a vector of strings indicating what feature
#'  types and their summaries the function should plot for: 'node' or 'edge'.
#' @param interactive A logical indicating whether the QQ plot, p-value vs
#'  difference plot, and the
#'  cell hierarchy plots should be interactive; see functions
#'  \code{fg_plot} and \code{fg_plot_qq}.
#' @param adjust_custom A function or a string indicating the
#' test adjustment method to use.
#'  If a string is provided, it should be one of
#'  \code{c("holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
#'  If a function is provided, it should take as input
#'  a numeric vector and output the
#'  same vector adjusted.
#' @param label_max An integer indicating how many labels should be shown
#'  in the functions \code{fg_plot_pVSdiff} and \code{fg_plot}.
#' @param box_no An integer indicating the maximum number of boxplots to save;
#'  used in function \code{fg_plot_box}.
#' @param paired A logical indicating whether the summary is paired; used in
#'  function \code{fg_plot_box}.
#' @param logged A logical indicating whether or not to log the summary
#'  statistic p value in the qq plots.
#' @param filter_adjust0 A numeric variable indicating what percentage of
#'  SpecEnr values compared (minimum) should be not close to 0.
#'  Set to 1 to not conduct filtering. This parameter is used for the QQ and the
#'  pVSdifference plots.
#' @param filter_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant.
#' @param filter_btwn_tpthres A numeric variable between 0 and 1 indicating the
#'  unadjusted T-test p-value threshold used to test whether the actual
#'  and expected feature values used to calculate the specified SpecEnr
#'  feature are significantly different for each sample class. Note this only
#'  needs to be specified for SpecEnr features. Combined
#'  with \code{filter_btwn_es}, we conduct three tests to understand if
#'  there is an actual large difference between actual and expected features:
#'  (1,2) T-test of significance between the actual and expected raw feature value
#'  (e.g. proportion) for samples in each of the compared classes, (3) and the
#'  T-test of significance between the differences of actual and
#'  expected feature values of the two classes. If any two of the three tests
#'  come out as insignificant, we set the p-value for the associated node/edge
#'  to 1.
#' @param filter_btwn_es A numeric variable between 0 and 1 indicating what the
#'  Cohen's D value of the nodes/edges in question must be greater or
#'  equal to, to be significant -- see \code{filter_btwn_tpthres}.
#' @param overwrite A logical variable indicating whether or not to replace
#'  old plots if they exist under the same folder name.
#' @param node_labels Parameter for the \code{fg_plot} function.
#' @param ... Other parameters for the \code{fg_plot} function.
#' @return No return; plots are saved to file.
#' @details The interactive plots are made using the \code{ggiraph} package.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos2)
#'  fg <- flowGraph(fg_data_pos2$count,
#'                  class=fg_data_pos2$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg_save_plots(fg, "temp")
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_plot}}
#'  \code{\link[flowGraph]{plot_gr}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_plot_qq}}
#'  \code{\link[flowGraph]{fg_plot_pVSdiff}}
#'  \code{\link[flowGraph]{fg_plot_box}}
#' @rdname fg_save_plots
#' @importFrom stringr str_pad
#' @importFrom utils head tail
#' @export
fg_save_plots <- function(
    fg, plot_path, plot_types="node", interactive=FALSE,
    adjust_custom="byLayer",
    label_max=10, box_no=20, paired=FALSE, logged=FALSE,
    filter_adjust0=1, filter_es=0,
    filter_btwn_tpthres=1, filter_btwn_es=0, overwrite=TRUE,
    node_labels="NONE", ...) {
    for (type in plot_types) {
        fg_summary <- fg_get_summary_all(fg)

        if (is.null(fg_summary[[type]])) next
        fg_summary_desc <- fg_get_summary_desc(fg)

        for (index in seq_len(length(fg_summary[[type]]))) {
            sm <- unlist(fg_summary_desc[[type]][index,])
            sm[2] <- paste0(sm[2], "-", ifelse(
                is.function(adjust_custom),"adjusted",adjust_custom))
            plot_path_ <- paste0(
                plot_path, "/", type, "/", paste0(sm, collapse="_"))
            while (dir.exists(plot_path_) & !overwrite)
                plot_path_ <- paste0(plot_path_,"_")
            dir.create(plot_path_, recursive=TRUE, showWarnings=FALSE)

            try ({
                ## P value vs SpecEnr difference
                gp <- fg_plot_pVSdiff(
                    fg, type=type, summary_meta=NULL,
                    adjust_custom=adjust_custom, index=index,
                    filter_adjust0=filter_adjust0, filter_es=filter_es,
                    filter_btwn_tpthres=filter_btwn_tpthres,
                    filter_btwn_es=filter_btwn_es,
                    label_max=label_max, interactive=interactive,
                    path=paste0(plot_path_, "/pVSdifference.png"))
            })

            try ({
                ## boxplots for each node
                if (box_no>0) {
                    pp = fg_get_summary(
                        fg, type="node", index=index, adjust_custom=adjust_custom,
                        filter_adjust0=filter_adjust0, filter_es=filter_es,
                        filter_btwn_tpthres=filter_btwn_tpthres,
                        filter_btwn_es=filter_btwn_es)
                    seda = abs(pp$m1 - pp$m2)
                    p = pp$values
                    node_edges = union(
                        names(utils::head(sort(p),box_no)),
                        names(utils::tail(sort(seda),box_no)))
                    rdir_ <- paste0(plot_path_, "/boxplots")
                    dir.create(rdir_, recursive=TRUE, showWarnings=FALSE)
                    for (node_edgei in seq_len(length(node_edges))) {
                        node_edge = node_edges[node_edgei]
                        if (node_edge=="") next
                        # if (p[node_edge]==1) next
                        # node_edge = "A+B+C+"
                        path <- paste0(
                            rdir_,"/",stringr::str_pad(node_edgei,width=3,pad="0"),
                            "_",node_edge,".png")
                        gp <- fg_plot_box_set(
                            fg, type="node", index=index, node_edge=node_edge,
                            adjust_custom=adjust_custom,
                            path=path, paired=paired,
                            filter_adjust0=filter_adjust0, filter_es=filter_es,
                            filter_btwn_tpthres=filter_btwn_tpthres,
                            filter_btwn_es=filter_btwn_es)
                    }
                }
            })

            try ({
                ## qq plot
                gp <- fg_plot_qq(
                    fg, type=type, index=index, adjust_custom=adjust_custom,
                    filter_adjust0=filter_adjust0, filter_es=filter_es,
                    filter_btwn_tpthres=filter_btwn_tpthres,
                    filter_btwn_es=filter_btwn_es,
                    path=paste0(plot_path_,"/qq.png"),
                    interactive=interactive, logged=logged)
            })

            if (type=="node")
                try ({
                    ## cell hierarchy plot
                    gr <- fg_plot(
                        fg, type="node", index=index,
                        adjust_custom=adjust_custom,
                        path=paste0(plot_path_,"/cell_hierarchy.png"),
                        label_max=label_max, interactive=interactive,
                        node_labels=node_labels, ...)

                    feat_index <- fg_summary_desc$node$feat[[index]]
                    if (grepl("SpecEnr",feat_index)) {
                        gp <- plot_gr(gr, label_coloured=FALSE)
                        feats <- se_feats(feat_index)

                        m1 <- fg_get_feature_means(
                            fg, "node", feats[2],
                            class=fg_summary_desc$node$class[[index]],
                            label=fg_summary_desc$node$label1[[index]])
                        m2 <- fg_get_feature_means(
                            fg, "node", feats[2],
                            class=fg_summary_desc$node$class[[index]],
                            label=fg_summary_desc$node$label2[[index]])

                        gr$v$v_ind[(m2>m1 & gr$v$colour<0) | (m2<m1 & gr$v$colour>0)] <- FALSE
                        gr$v$colour <- m2-m1
                        gr$main <- paste0(gr$main,"\n- colour: mean ",feats[2]," diff")
                        gp <- plot_gr(gr, label_coloured=FALSE)
                        ggplot2::ggsave(
                            paste0(plot_path_,"/cell_hierarchy_",
                                   feats[2],"_colours.png"),
                            plot=gp)
                    }

                })
        }
    }
}

#' @title Formats time into string.
#' @description Formats time into a string HH:MM:SS given time zone.
#' @param time A time variable of class \code{POSIXct}, \code{POSIXt}.
#' @return Time formatted as a string; used in \code{time_output} function.
#' @examples
#'  # NOT EXPORTED
#'  flowGraph:::tstr(Sys.time())
#'
#' @rdname tstr
tstr <- function(time) format(.POSIXct(time), "%H:%M:%S")


#' @title Outputs elapsed time.
#' @description Given a time, prints the time elapsed from that time until now.
#' @param start A time variable of class \code{POSIXct}, \code{POSIXt}.
#' @param msg A string with a message to print out after the elapsed time.
#' @return Prints to console, the time from which process
#'  started \code{start} - ended, and > time elapsed from
#'  \code{start} until now.
#' @examples
#'
#'  start <- Sys.time()
#'  flowGraph:::time_output(start,'start - now > time elapsed')
#'
#' @rdname time_output
time_output <- function(start, msg="") {
    start <- as.POSIXct(start)
    end <- Sys.time()
    time_elapsed <- difftime(end, start, units="secs")
    message(msg, ifelse(msg == "", "", ": "),
            tstr(start), "-", tstr(end), " > ", tstr(time_elapsed))
}


#' @title Prepares parallel loop indices.
#' @description \code{loop_ind_f} is a helper function that splits
#'  a vector of loop indices into a list of multiple loop indices
#'  for use in parallel processes within the flowGraph package.
#' @param x A vector of loop indices.
#' @param n An integer, or the number of vectors to split \code{x} into.
#' @return list of \code{n} vectors with elements from \code{x}.
#' @examples
#'
#'  old_loop_inds <- 1:10
#'  no_cores <- 5
#'
#'  new_loop_inds <- flowGraph:::loop_ind_f(old_loop_inds, no_cores)
#'  # future::plan(future::multisession)
#'  # example_indices <- furrr::future_map(new_loop_inds, function(ii) {
#'  #     purrr::map(ii, function(i) i )
#'  # s})
#'
#' @rdname loop_ind_f
loop_ind_f <- function(x, n) {
    if (n == 1) return(base::list(x))
    return(base::split(x, ceiling(seq_along(x)/ceiling(base::length(x)/n))))
}


#' @title Summarizes a numeric matrix.
#' @description Summarizes a numeric matrix.
#' @param m A numeric matrix.
#' @param feat_type Name of the matrix \code{m}.
#' @return A data frame containing one row summarizing \code{m};
#'  see \code{\link[flowGraph]{fg_get_feature_desc}}.
#' @examples
#'
#'  summary_table(matrix(rnorm(12),nrow=3), feat_type='random')
#'
#' @rdname summary_table
#' @export
summary_table <- function(m, feat_type="") {
    m <- as.matrix(m)
    base::data.frame(feat=feat_type, nrow=base::nrow(m),
                     ncol=base::ncol(m), inf=sum(is.infinite(m)),
                     neginf=sum(m == -Inf), na=sum(base::is.na(m)),
                     nan=sum(is.nan(m)), neg=sum(m < 0),
                     pos=sum(m > 0), zero=sum(m == 0), max=max(m[is.finite(m)]),
                     min=min(m[is.finite(m)]))
}


#' @title Normalizes matrix values by class.
#' @description Used only in the \code{\link[flowGraph]{fg_feat_mean_class}}
#'  function; for each class in the \code{classes} vector,
#'  \code{meandiff} takes the column mean
#'  of the rows in the given matrix associated with that class;
#'  it then takes the difference point by point between these means and
#'  the original rows for that class.
#' @param m0 A numeric matrix.
#' @param classes A vector whose length is equal to the number of
#'  rows in the given matrix.
#' @return A numeric matrix whose dimensions equate to that of the input
#'  and whose values are normalized per class.
#' @examples
#'
#'  classes <- append(rep('apples',4), rep('oranges',3))
#'  m0 <- matrix(rnorm(35), nrow=7)
#'  m <- flowGraph:::mean_diff(m0, classes)
#'
#' @seealso
#'  \code{\link[flowGraph]{fg_feat_mean_class}}
#' @rdname meandiff
#' @importFrom purrr map
mean_diff <- function(m0, classes) {
    m <- m0
    for (pi in base::unique(classes)) {
        pii <- classes == pi
        m_ <- m[pii, , drop=FALSE]
        m_m <- base::colMeans(m_)
        m[pii, ] <- base::do.call(rbind,
                                  purrr::map(base::seq_len(base::nrow(m_)),
                                             function(i) m_[i, ] - m_m))
    }
    base::dimnames(m) <- base::dimnames(m0)
    return(m)
}

#' @title Wrapper for map
#' @description Wrapper for \code{purrr::map} and \code{furrr::future_map}
#'  to handle parallel-ization
#' @param x Variable to recurse over; must be indices!
#' @param f Function to recurse over.
#' @param no_cores Number of cores to use; \code{future} must have
#'  been ran already.
#' @param prll If set to FALSE, forces use of purrr::map instead of
#' furrr::future_map, Default: TRUE
#' @param ... Other parameters used by \code{f}.
#' @return Unnested named list.
#' @details Wrapper for \code{purrr::map} and \code{furrr::future_map} to handle
#' parallel-ization easily; note that \code{future} must have been ran already
#' outside of the function and outputs will always be a list.
#' @seealso
#'  \code{\link[purrr]{map}}
#'  \code{\link[furrr]{future_map}}
#' @rdname fpurrr_map
# #' @export
#' @importFrom purrr map
#' @importFrom furrr future_map
fpurrr_map <- function(x, f, no_cores=1, prll=TRUE, ...) {
    if (length(x)==0) return(NULL)
    if (no_cores==1 | !prll) {
        res <- purrr::map(x, f)
    } else if (length(x)/no_cores < 2) {
        res <- furrr::future_map(x, f)
    } else {
        xxx <- loop_ind_f(x, no_cores)
        res <- furrr::future_map(xxx, function(xx) {
            a <- purrr::map(xx, f)
            if (is.list(a)) return(a)
            list(a)
        })
        res <- unlist(res, recursive=FALSE)
    }
    if (is.character(x))
        names(res) <- x
    return(res)
}


#' @title Extracts markers from cell population phenotypes
#' @description Extracts all unique markers from cell population phenotypes
#' @param phen A vector of cell population phenotypes.
#' @return A vector of unique markers
#' @seealso
#'  \code{\link[stringr]{str_split}}
#' @rdname extract_markers
# #' @export
#' @importFrom stringr str_split
extract_markers <- function(phen) {
    phen <- gsub("_not","_",phen)
    markers <- unique(unlist(stringr::str_split(phen,"[-+_]+")))
    markers[markers!=""]
}


#' @title Reformats phenotype
#' @description Reformats cell population phenotypes into flowGraph format
#' @param phen Vector of cell population phenotype names as character strings.
#' @param markers markers extracted from \code{phen}.
#' @return Vector with the same length as \code{phen} containing reformatted
#' and not necessarily changed cell population phenotype names.
#' @seealso
#'  \code{\link[stringr]{str_extract}},\code{\link[stringr]{str_split}}
#' @examples
#'
#' # fg_clean_phen(c("A+_B+","B+_notC","A-_C"))
#'
#' @rdname fg_clean_phen
# #' @export
#' @importFrom stringr str_extract str_split
fg_clean_phen <- function(phen, markers=NULL) {
    if (is.null(markers))
        markers <- extract_markers(phen)
    if (any(grepl("[_]not",phen))) {
        gate_markers <- gsub("not","",unique(stringr::str_extract(
            unlist(stringr::str_split(phen,"[-+_]+")),
            paste0("^not", paste0(markers),collapse="|") )))
        gate_markers <- gate_markers[!is.na(gate_markers)]
        for (gm in gate_markers) {
            phen <- gsub(paste0("^",gm,"$|_",gm,"$|_",gm,"_|^",gm), paste0("_",gm,"+"), phen)
            phen <- gsub(paste0("not",gm), paste0(gm,"-"), phen)
            phen <- gsub("^_", "", phen)
        }
    }
    if (any(grepl("[_]",phen)))
        phen <- gsub("_","",phen)
    phen
}

#' @title Converts input into a significance test function
#' @param test_custom a string \code{c("t", "wilcox","ks","var","chisq")} or a
#'  function.
#' @return a statistical significance test function.
#' @seealso
#'  \code{\link[stats]{t.test}},\code{\link[stats]{wilcox.test}},
#'  \code{\link[stats]{ks.test}},\code{\link[stats]{var.test}},
#'  \code{\link[stats]{chisq.test}}
#' @rdname test_c
# #' @export
#' @importFrom stats t.test wilcox.test ks.test var.test chisq.test
test_c <- function(test_custom) {
    if (!is.function(test_custom))
        if (test_custom=="t") {
            test_custom <- function(x,y)
                tryCatch(stats::t.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="wilcox") {
            test_custom <- function(x,y)
                tryCatch(stats::wilcox.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="ks") {
            test_custom <- function(x,y)
                tryCatch(stats::ks.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="var") {
            test_custom <- function(x,y)
                tryCatch(stats::var.test(x,y)$p.value,error=function(e)1)
        } else if (test_custom=="chisq") {
            test_custom <- function(x,y)
                tryCatch(stats::chisq.test(x,y)$p.value,error=function(e)1)
        }
    test_custom
}


#' @title Calcuate SpecEnr from proportion and expected proportion
#' @description FUNCTION_DESCRIPTION
#' @param mp_ Numerical sample x cell population matrix w/ proportions.
#' @param me_ Numerical sample x cell population matrix w/ expected proportions.
#' @return Numerical sample x cell population matrix w/ SpecEnr.
# #' @rdname ms_create
# #' @export
ms_create <- function(mp_, me_) {
    mp_ <- as.matrix(mp_)
    me_ <- as.matrix(me_)
    ms_ <- as.matrix(mp_/me_)
    if (is.na(dim(ms_)[1]))
        ms_ <- matrix(ms_, ncol=ncol(mp_))
    ms_nan <- is.nan(ms_)
    ms_[is.infinite(ms_)] <- max(ms_[is.finite(ms_)])
    ms_[ms_nan] <- 0
    e0 <- as.matrix(me_)==0
    suppressWarnings({ ms_ <- as.matrix(log(ms_)) })
    ms_[is.nan(ms_)] <- 0
    ms_[e0] <- log(as.matrix(mp_)[e0])
    ms_[mp_==0] <- 0

    dimnames(ms_) <- dimnames(mp_)
    return(ms_)
}


#' @title Gets child populations of given cell populations
#' @description Gets the child populations of a vector of given cell populations
#'  \code{parens} and updates \code{pchild} the edge list if edge list doesn't contain
#'  the requested information.
#' @param parens Character vector of cell population phenotypes.
#' @param pchild Edge list where the name of the list is the cell population
#'  and the vector in each element contains the child cell populations of the
#'  named cell population.
#' @param pc_i A cell population x marker matrix where the values are 0/1/2/...
#'  correspondng to marker conditions /-/+/... for possible PARENT populations.
#' @param ac__ A list where the elements are marker index > "0"/"1"/"2"/... >
#'  a logical vector the same length as the number of cell population phenotypes
#'  indicating whether or not the marker condition exists in them; this is for
#'  the possible CHILD cell populations
#' @param meta_cell__ data frame with meta data for cell population phenotypes
#'  from the flowGraph object; this is for the possible CHILD cell populations.
#' @return A list containing child populations of \code{parens}; also globally
#'  updates \code{pchild}.
#' @seealso
#'  \code{\link[purrr]{map}},\code{\link[purrr]{keep}}
# #' @rdname get_child
# #' @export
#' @importFrom purrr map compact
get_child <- function(parens, pchild, pc_i, ac__, meta_cell__) {
    parens_ <- parens[!parens%in%names(pchild)]
    if (length(parens_)==0)
        return(pchild[match(parens,names(pchild))])

    res <- fpurrr_map(seq_len(nrow(pc_i)), function(j) {
        mcgrow <- pc_i[j,]
        colj1 <- which(mcgrow > 0)
        chi <- Reduce("&", purrr::map(colj1, function(coli)
            ac__[[coli]][[as.character(mcgrow[coli])]] ))
        meta_cell__$phenotype[chi]
    }, no_cores, prll=nrow(pc_i)>1000)#, pc=pc_i, ac=ac__, mcell=meta_cell__)
    names(res) <- rownames(pc_i)

    pchild <- append(pchild, purrr::compact(res))
    pchild <<- pchild
    return(pchild[match(parens,names(pchild))])
}

#' @title Gets parent populations of given cell populations
#' @description Gets the parent populations of a vector of given cell
#'  \code{childs} and updates \code{pparen} the edge list if edge list doesn't
#'  contain the requested information.
#' @param childs Character vector of cell population phenotypes.
#' @param pparen Edge list where the name of the list is the cell population
#'  and the vector in each element contains the parent cell populations of the
#'  named cell population.
#' @param pc__i A cell population x marker matrix where the values are 0/1/2/...
#'  correspondng to marker conditions /-/+/...;
#'  this is for the possible CHILD cell populations.
#' @param ac_ A list where the elements are marker index > "0"/"1"/"2"/... >
#'  a logical vector the same length as the number of cell population phenotypes
#'  indicating whether or not the marker condition exists in them;
#'  this is for the possible PARENT cell populations.
#' @param meta_cell_ data frame with meta data for cell population phenotypes
#'  from the flowGraph object; this is for the possible PARENT cell populations.
#' @return A list containing parent populations of \code{childs}; also globally
#'  updates \code{pparen}.
#' @rdname get_paren
# #' @export
#' @importFrom purrr map compact
get_paren <- function(childs, pparen, pc__i, ac_, meta_cell_) {
    childs_ <- childs[!childs%in%names(pparen)]
    if (length(childs_)==0)
        return(pparen[match(childs,names(pparen))])

    res <- fpurrr_map(seq_len(nrow(pc__i)), function(j) {
        mcgrow <- pc__i[j, ]
        colj1 <- which(mcgrow > 0)
        chidf <- do.call(cbind, purrr::map(colj1, function(coli)
            ac_[[coli]][[as.character(mcgrow[coli])]] ))
        chi <- apply(chidf, 1, function(x) sum(!x) == 1)
        meta_cell_$phenotype[chi]
    }, no_cores, prll=nrow(pc__i)>1000)
    names(res) <- rownames(pc__i)

    pparen <- append(pparen, purrr::compact(res))
    pparen <<- pparen
    return(pparen[match(childs,names(pparen))])
}

#' @title Gets edge proportions of a given edge matrix
#' @description Gets the edge proportions of the edges in edge matrix
#'  \code{edf_} and updates \code{ep} edge proportion matrix if it didn't
#'  contain the requested information.
#' @param edf_ edge x from&to data frame containing edges and their from and to
#'  cell population phenotypes.
#' @param ep sample x edge (parent_child) matrix with edge proportions.
#' @param mp_ sample x phenotype matrix with proportions.
#' @param no_cores Number of cores to use, Default: 1
#' @return \code{ep} with only the specific columns (edges) requested; also updates
#' \code{ep} globally.
#' @rdname get_eprop
# #' @export
get_eprop <- function(edf_, ep, mp_, no_cores=1) {
    clnm <- apply(edf_, 1, paste0, collapse="_")
    todoi <- which(!clnm%in%colnames(ep))
    if (length(todoi)==0)
        return(ep[,clnm,drop=FALSE])
    ep_ <- do.call(cbind, fpurrr_map(todoi, function(i) {
        if (edf_$from[i]=="") return(mp_[,edf_$to[i]])
        mp_[,edf_$to[i]]/mp_[,edf_$from[i]]
    }, no_cores, prll=length(todoi)>1000))
    colnames(ep_) <- clnm[todoi]
    if (is.null(ep))
        rownames(ep_) <- rownames(mp_)
    ep <- cbind(ep, ep_)
    ep <<- ep

    return(ep[,clnm,drop=FALSE])
}


#' @title Determines which phenotypes are statistically significant
#' @description Determines which phenotypes are statistically significant based
#'  on SpecEnr.
#' @param ms_ sample x phenotype SpecEnr matrix
#' @param summary_pars See \code{flowGraphSubset}.
#' @param summary_adjust See \code{flowGraphSubset}.
#' @param test_cust Final significance test function.
#' @param test_custom Raw significance test function.
#' @param lyrno An integer indicating total number of layers in the cell
#'  hierarchy including layer 0.
#' @param mp_ sample x phenotype proportion matrix.
#' @param me_ sample x phenotype expected proportion matrix.
#' @return A logical vector the same length as the number of columns in
#'  \code{ms_} indicating whether or not each phenotype is significant;
#'  used only for the fast version of flowGraph to determine whether or not
#'  to keep testing the phenotypes' children.
#' @rdname ms_psig
# #' @export
ms_psig <- function(ms_, summary_pars, summary_adjust,
                    test_cust, test_custom, lyrno, mp_, me_) {
    if (is.null(summary_adjust$filter_btwn_tpthres)) {
        warning("no p threshold specified in `summary_adjust$filter_btwn_tpthres`, aplying default threshold 0.05")
        summary_adjust$filter_btwn_tpthres <- .05
    }
    btwn_test_custom <- test_c(summary_adjust$btwn_test_custom)

    ## calculate p values
    p <- apply(ms_,2,test_cust,meta[[summary_pars$class]],test_custom)
    names(p) <- colnames(ms_)

    ## adjust p values
    dual <- FALSE
    if (!is.null(summary_pars$labels) & !is.null(summary_adjust))
        if (length(summary_pars$labels)==2)
            dual <- TRUE

    if (summary_adjust$adjust_custom=="byLayer")
        p <- sapply(p*ncol(ms_)*lyrno, min, 1)
    p[is.na(p)] <- 1
    ptf <- rep(FALSE, length(p))
    sigcands <- rep(TRUE, length(p))

    # cohen's d, number of 0s, pvalue in between
    if (dual & !is.null(btwn_test_custom) &
        summary_adjust$filter_btwn_es>0 |
        summary_adjust$filter_btwn_tpthres<1) {
        id1 <- summary_pars$labels[[1]]
        id2 <- summary_pars$labels[[2]]

        pp3v1 <- sapply(seq_len(ncol(mp_)), function(ci)
            btwn_test_custom(mp_[id1,ci], me_[id1,ci]) )
        pp3v2 <- sapply(seq_len(ncol(mp_)), function(ci)
            btwn_test_custom(mp_[id2,ci], me_[id2,ci]) )

        pp3b_raw <- lapply(seq_len(ncol(mp_)), function(ci) {
            a <- mp_[id1,ci] - mp_[id2,ci]
            b <- me_[id1,ci] - me_[id2,ci]
            return(list(a=a,b=b))
        })
        pp3bt <- sapply(seq_len(ncol(mp_)), function(ci)
            btwn_test_custom( pp3b_raw[[ci]]$a, pp3b_raw[[ci]]$b) )
        pp3bt[is.na(pp3bt)] <- 1

        sig1 <- pp3v1<summary_adjust$filter_btwn_tpthres
        sig2 <- pp3v2<summary_adjust$filter_btwn_tpthres
        sig3 <- pp3bt<summary_adjust$filter_btwn_tpthres
        sigcands <- (sig1 | sig2) & sig3
    }
    ptf[sigcands & p<summary_adjust$filter_btwn_tpthres] <- TRUE
    return(ptf)
}


#' @name flowGraphSubset_summary_pars
#' @title Default for flowGraphSubset's summary_pars
#' @description Default input for flowGraphSubset's \code{summary_pars} parameter.
#' @return Default list parameter flowGraphSubset's \code{summary_pars} parameter.
#' @rdname flowGraphSubset_summary_pars
#' @examples
#'
#'  flowGraphSubset_summary_pars()
#'
#' @export
flowGraphSubset_summary_pars <- function()
    list(
        node_feature="SpecEnr",
        edge_feature="NONE",
        test_name="t_diminish",
        test_custom="t",
        diminish=TRUE,
        class="class",
        labels=NULL)


#' @name flowGraphSubset_summary_adjust
#' @title Default for flowGraphSubset's summary_adjust
#' @description Default input for flowGraphSubset's \code{summary_adjust}
#'  parameter. ONLY USE THIS OVER flowGraph IF: 1) your data set has more than
#'  10,000 cell populations and you want to speed up your calculation time AND
#'  2) you only have one set of classes you want to test on the
#'  SAME SET OF SAMPLES (e.g. control vs experiment).
#'  As flowGraphSubset does not
#'  calculate the SpecEnr for all cell populations, so if you want to test other
#'  sets of classes on the same samples, you will not be able to test all
#'  possible cell populations on the new set of classes.
#' @return Default list parameter flowGraphSubset's \code{summary_adjust} parameter.
#' @rdname flowGraphSubset_summary_adjust
#' @examples
#'
#'  flowGraphSubset_summary_adjust()
#'
#' @export
flowGraphSubset_summary_adjust <- function()
    list(
        adjust_custom="byLayer", btwn_test_custom="t",
        adjust0_lim=c(-.1,.1), filter_adjust0=1, filter_es=0,
        filter_btwn_tpthres=.05, filter_btwn_es=.5)


#' @name flowGraphSubset
#' @title flowGraph object constructor.
#' @description Initializes a \code{flowGraph} object given the cell counts
#'  for one or more flow cytometry sample(s).
#'  The flowGraph object returned holds meta data
#'  for each sample, each cell population node,
#'  edges representing how each cell population node relate to one another,
#'  and features for these nodes and edges.
#' @param input_ a numeric matrix of the cell counts;
#'   its column/names must be the phenotype names and its rownames
#'   must be sample ID's.
#' @param meta A data frame with meta data for each \code{Phenotypes} or sample;
#'  One of its column names should be "id" whose values correspond to
#'  the name of each \code{Phenotypes} object. We also recommend for it to have
#'  a column named "class" where one of its unique values is "control".
#' @param class A string corresponding to the column name or index
#'  of \code{meta} whose values represent
#'  the class label of each sample, Default: 'class'
#' @param no_cores An integer indicating how many cores to parallelize on,
#'  Default: 1
#' @param markers A string vector of marker names used in \code{input_},
#'  Default: NULL
#' @param layout_fun A string of a function from the \code{igraph} package that
#'  indicates what layout should be used if a cell hierarchy is to be ploted;
#'  all such functions have prefix \code{layout_}. This is defaulted to
#'  e.g. \code{layout_fun="layout.reingold.tilford"}.
#' @param max_layer And integer indicating the maximum layer in the cell
#'  hierarchy to analyze; set to `NULL` to analyze all layers.
#' @param cumsumpos A logical variable indicating whether
#'  or not to cumulate cell counts;
#'  this applies only when \code{partitionsPerMarker > 3} and will convert
#'  e.g. the count of A+ or A++ into the sum of the counts of
#'  A+, A++, A+++, ..., or A++, A+++, ... , Default: FALSE
#' @param path A string indicating the folder path to where the flowGraph
#'  object should save its elements, Default = NULL (don't save).
#' @param summary_pars A list containing parameters for calculating the
#'  statistical significance summary significance that will determine whether
#'  to trim out phenotypes for this fast version of flowGraph. The lists'
#'  elements are:
#' \itemize{
#'   \item{\code{node_feature}: "SpecEnr"; this is the feature we will be
#'    testing, don't change this.}
#'   \item{\code{edge_feature}: "NONE"; this unneeded for now.}
#'   \item{\code{test_name}: "t_diminish"; this unneeded for now.}
#'   \item{\code{test_custom}: "t"; a string or a function indicating the
#'    statistical test desires. These tests can be
#'    \code{c("t", "wilcox","ks","var","chisq")} corresponding to functions
#'    \code{\link[stats]{t.test}},\code{\link[stats]{wilcox.test}},
#'    \code{\link[stats]{ks.test}},\code{\link[stats]{var.test}},
#'    \code{\link[stats]{chisq.test}}}
#'   \item{\code{diminish}: TRUE; whether or not to continue testing
#'    phenotypes whos parent phenotypes are all insignificant.}
#'   \item{\code{class}: "class"; the column name in \code{meta} that contains
#'    class labels you want to test.}
#'   \item{\code{labels}: c("aml", "control") for the flowcap data set;
#'   SET THIS!! to the class labels you want to test using \code{test_custom}.}
#' }
#' @param summary_adjust A list of parameters on how to adjust the p-values;
#'  this also affects which phenotypes are tested. The elements in the list are:
#' \itemize{
#'   \item{\code{adjust_custom}: "byLayer"; this is a string (corresponding to
#'    an option in \code{\link[stats]{p.adjust}}) or a function used to
#'    adjust p-values.}
#'   \item{\code{btwn_test_custom}: "t"; see \code{test_custom} in
#'    \code{summary_pars}; this statistical significance test is used in
#'    the filters.}
#'   \item{\code{adjust0_lim}: see \code{fg_get_summary}.}
#'   \item{\code{filter_adjust0}: see \code{fg_get_summary}.}
#'   \item{\code{filter_es}: see \code{fg_get_summary}.}
#'   \item{\code{filter_btwn_tpthres}: see \code{fg_get_summary}.}
#'   \item{\code{filter_btwn_es}: see \code{fg_get_summary}.}
#' }
#' @param save_plots A logical indicating whether or not to save plots.
#' @return flowGraph object
#' @details All node and edge features are trimmed such that only the
#'  significant phenotypes are left; the original input is stored in the slot
#'  \code{etc$original_count} of the returned flowGraph object.
#' @examples
#' \dontrun{
#' if(interactive()){
#'   data(fg_data_pos2)
#'   fg <- flowGraph(fg_data_pos2$count, meta=fg_data_pos2$meta, no_cores=1)
#'  }
#' }
#' @rdname flowGraphSubset
#' @export
#' @importFrom Matrix Matrix
#' @importFrom purrr map_lgl map map_dfr compact
#' @importFrom methods new
#' @importFrom future plan multisession sequential
#' @importFrom data.table as.data.table setattr ".I"
#' @importFrom stringr str_split
#' @importFrom matrixStats rowMins rowMaxs
#' @importFrom stringi stri_rand_strings
flowGraphSubset <- function(
    input_,
    meta=NULL,
    class="class",
    no_cores=1,
    markers=NULL, # including gate names
    layout_fun="layout.reingold.tilford", # layout.circle

    max_layer=NULL,

    # feature parameters
    cumsumpos=FALSE, # whether to make positives +=+/++ (cumsum)

    # save flowGraph object as folder
    path=NULL,

    # summary parameters
    summary_pars=flowGraphSubset_summary_pars(),
    summary_adjust=flowGraphSubset_summary_adjust(),

    # plotting parameters
    save_plots=TRUE
) {

    warning("The fast version of flowGraph is in beta <(@u@<)")

    ## check parameters/input ####
    if (!is.null(meta))
        if (!"id"%in%colnames(meta))
            stop("'meta' must have a column named \"id\" with sample ID's.")

    if (is.na(dim(input_)[1]))
        stop("input must be a numeric sample x cell population matrix.")

    start <- start1 <- Sys.time()
    message("preparing input; ")

    options(stringsAsFactors=FALSE)


    ## feature: count (sample x cell population)
    if (is.null(dim(input_)[1]))
        input_ <- matrix(input_, nrow=1, dimnames=list("s1",names(input_)))
    if (is.null(rownames(input_)))
        rownames(input_) <- paste0("s",seq_len(nrow(input_)))

    # remove columns with all 0's (these won't have children so it's ok)
    keepinds <- apply(input_, 2, function(x) any(x>0))
    input_ <- Matrix::Matrix(input_[,keepinds,drop=FALSE], sparse=TRUE)


    ## meta (samples)
    if (is.null(meta))
        meta <- data.frame(id=rownames(input_))
    if (nrow(meta)!= nrow(input_))
        stop("meta data and cell count matrix have different row counts")
    if (!"id"%in%colnames(meta))
        stop("meta must have an \'id\' column, id,
             whose values corresponding with sample names")
    if (!summary_pars$class%in%colnames(meta))
        stop("significance test class not found (summary_pars$class)")

    meta <- as.data.frame(meta)
    factor_col <- purrr::map_lgl(meta, is.factor)
    if (any(factor_col))
        warning("converting factors in meta into strings")
    for (j in which(factor_col))
        meta[[j]] <- as.character(meta[[j]])
    rownames(input_) <- meta$id


    ## meta (cell populations) ####
    # remove "_" between marker conditions,
    # replace "not" with "-",
    # if no "not", add "+"
    if (!any(grepl("[+-]", colnames(input_))))
        stop("make sure input type is correct")
    if (is.null(markers))
        markers <- extract_markers(colnames(input_))
    colnames(input_) <- fg_clean_phen(colnames(input_), markers)

    # meta (cell populations)
    meta_cell <- get_phen_meta(colnames(input_))
    input_ <- input_[,match(meta_cell$phenotype,colnames(input_)),drop=FALSE]

    # get rid of lower layers if requested
    if (!is.null(max_layer)) {
        phen_id <- meta_cell$phenolayer <= max_layer
        meta_cell <- meta_cell[phen_id,,drop=FALSE]
        input_ <- input_[,phen_id,drop=FALSE]
    }
    time_output(start1)
    start1 <- Sys.time()
    message("preparing to calculate features")

    ## flowGraph ####
    fg <- methods::new(
        "flowGraph",
        feat=list(node=list(count=input_), edge=list()),
        feat_desc=list(node=summary_table(input_,"count")),
        meta=meta,
        markers=markers,

        graph=list(v=meta_cell), edge_list=list(child=NULL,parent=NULL),
        plot_layout=as.character(substitute(layout_fun)),
        etc=list(cumsumpos=FALSE, class_mean_normalized=FALSE)
    )

    # feature: make + cpops' count a cumulated sum "++"="++"+"+++"
    cumsumpos <- cumsumpos & any(grepl("3",meta_cell$phenocode))
    if (cumsumpos)
        fg <- fg_feat_cumsum(fg, no_cores=no_cores)
    mc <- fg_get_feature(fg, "node", "count")

    # feature: proportion
    mp <- mc/mc[,colnames(mc)==""]
    dimnames(mp) <- dimnames(mc)


    ## START ####
    if (no_cores>1) future::plan(future::multisession)


    ## initialize significance test ####
    labels <- NULL
    if (is.null(summary_pars$class) & length(class)==1)
        summary_pars$class <- class
    test_custom <- test_c(summary_pars$test_custom)
    btwn_test_custom <- NULL
    if (!is.null(summary_adjust$btwn_test_custom))
        btwn_test_custom <- test_c(summary_adjust$btwn_test_custom)
    us <- unique(meta[,summary_pars$class])
    if (length(summary_pars$labels)==2 & !is.list(summary_pars$labels)) {
        labels <- summary_pars$labels
        summary_pars$labels <- lapply(summary_pars$labels, function(x)
            meta[,summary_pars$class]==x)
    }
    if (length(us)==2 &
        is.null(summary_pars$labels)) {
        summary_pars$labels <- lapply(us, function(x)
            meta[,summary_pars$class]==x)
    }
    # if it's a 2 way significance test AND labels is specified,
    # x and y are the two groups we're comparing
    # ELSE x are the values, y are the classes
    test_cust <- function(x,y, test_custom) {
        if (is.null(summary_pars$labels))
            return(test_custom(x,y))
        test_custom(x[summary_pars$labels[[1]]],x[summary_pars$labels[[2]]])
    }


    ## initialize cell population meta material for making edge list ####
    # indices for each layer
    dt <- data.table::as.data.table(meta_cell$phenolayer)[
        , list(list(.I)), by=meta_cell$phenolayer]
    lyril <- data.table::setattr(dt$V1, 'names', dt$meta_cell)
    lyrno <- length(lyril)
    lyrs <- as.numeric(names(lyril))
    lyrstf <- sapply(lyrs, function(x) (x-1)%in%lyrs & (x-2)%in%lyrs)

    # make phenocode matrix
    pc <- Matrix::Matrix(Reduce(
        rbind, lapply(stringr::str_split(meta_cell$phenocode,""),
                      as.numeric)), sparse=TRUE)
    rownames(pc) <- meta_cell$phenotype
    allcolu <- apply(pc, 2,unique)
    if (!is.list(allcolu))
        allcolu <- split(allcolu, seq(ncol(allcolu)))

    # make allcol > markeri > 0/1/2 > T/F per phenotype
    allcol <- fpurrr_map(seq_len(length(allcolu)), function(ci) {
        a <- purrr::map(allcolu[[ci]], function(ui) pc[, ci]==ui)
        names(a) <- allcolu[[ci]]; return(a)
    }, no_cores=no_cores, prll=length(allcolu)>1000)

    # split everything above by layer
    pcs <- acs <- meta_cells <- list()
    for (lyri in names(lyril)) {
        li <- lyril[[lyri]]
        pcs[[lyri]] <- pc[li,,drop=FALSE]
        acs[[lyri]] <- purrr::map(allcol, purrr::map, function(y) y[li])
        meta_cells[[lyri]] <- meta_cell[li,, drop=FALSE]
    }
    time_output(start1)
    start1 <- Sys.time()
    message("calculating features + edge lists by + p-values for cell populations with significant parents only (this is the fast version of flowGraph);")


    ## initialize features (SpecEnr, expect_prop, edge prop) for lyr0/1 ####
    # expected prop/SpecEnr for node(m), edge(e)
    me <- ep <- ms <- p <- lyrp <- sig_phens <- p1 <- NULL
    pchild <- pparen <- list()

    # root proportion is just 1, but for completeness we add it in
    if ("0"%in%names(lyril)) {
        me <- mp[,colnames(mp)=="",drop=FALSE]
        ms <- matrix(0, ncol=1, nrow=nrow(me), dimnames=dimnames(me))
        p <- 1; names(p) <- ""
        p1 <- sig_phens <- ""
        lyrp <- 0
    }
    if ("1"%in%names(lyril)) {
        p1 <- meta_cells[["1"]]$phenotype
        me <- cbind(me, matrix(.5, nrow=nrow(mp), ncol=length(p1),
                               dimnames=list(rownames(mp),p1)))
        ep <- mp[,p1,drop=FALSE]
        colnames(ep) <- paste0("_",p1)

        ## initiate child and parent edge lists for layers 0/1
        pchild <- list(p1)
        names(pchild) <- ""
        pparen <- purrr::map(seq_len(length(pchild[[1]])), function(x) "")
        names(pparen) <- pchild[[1]]

        ms_ <- ms_create(mp[,p1,drop=FALSE], me[,p1,drop=FALSE])
        ms <- cbind(ms, ms_)

        # FAST trim
        p1 <- p1[ms_psig(ms_, summary_pars, summary_adjust,
                         test_cust, test_custom, lyrno,
                         mp[,colnames(ms_),drop=FALSE],
                         me[,colnames(ms_),drop=FALSE])]
        sig_phens <- append(sig_phens, p1)
        lyrp <- 1
    }


    ## calculate features for each layer ####
    for (lyr in sort(lyrs[lyrstf])) {
        if (length(p1)==0) break

        start2 <- Sys.time()
        message("- ", length(p1), "/", length(lyril[[as.character(lyr)]]),
                " pops @ layer ", lyr)

        lyrc_ <- as.character(lyr-1)
        lyrc__ <- as.character(lyr)

        # get cell pops
        p2 <- meta_cells[[lyrc__]]$phenotype
        if (!is.null(lyrp)) #  prev layer exists?
            if (lyrp+1==lyr)
                p2 <- unique(unlist(get_child(
                    p1, pchild, pc_i=pcs[[lyrc_]], ac__=acs[[lyrc__]],
                    meta_cell__=meta_cells[[lyrc__]])))
        if (length(p2)==0) break
        p1 <- p2

        # get cell pops' parents
        pparen_ofchd <- get_paren(
            p2, pparen, pc__i=pcs[[lyrc__]], ac_=acs[[lyrc_]],
            meta_cell_=meta_cells[[lyrc_]])

        # get cell pops' grandparents
        grprnt_ofchd <- get_paren(
            unique(unlist(pparen_ofchd)), pparen,
            pc__i=pcs[[lyrc_]], ac_=acs[[as.character(lyr-2)]],
            meta_cell_=meta_cells[[as.character(lyr-2)]])

        # get min edge proportions between parent and grandparents
        edf_ <- purrr::map_dfr(names(grprnt_ofchd), function(paren)
            data.frame(from=grprnt_ofchd[[paren]], to=paren))
        ep_ <- get_eprop(
            edf_, ep,
            mp[,colnames(mp)%in%unique(append(edf_$from,edf_$to)),drop=FALSE],
            no_cores)
        ep_min <- do.call(cbind, fpurrr_map(names(grprnt_ofchd), function(x) {
            ep_col <- grepl(paste0("_",x), colnames(ep_))
            matrixStats::rowMins(ep_[,ep_col,drop=FALSE])
        }, no_cores, prll=length(grprnt_ofchd)>1000))

        # calculate expected proportion
        me_ <- do.call(cbind, fpurrr_map(p2, function(phen) {
            matrixStats::rowMins(ep_min[,pparen[[phen]],drop=FALSE]) *
                matrixStats::rowMaxs(mp[,pparen[[phen]],drop=FALSE])
        }, no_cores, prll=length(p2)>1000))
        if (is.na(dim(me_)[1]))
            me_ <- matrix(me_, ncol=length(pparen_ofchd))
        me_[is.nan(me_)] <- 0
        me_[as.matrix(me_)<0] <- 0
        colnames(me_) <- p2
        me <- cbind(me, me_)

        # calculte SpecEnr
        ms_ <- ms_create(mp[,p2,drop=FALSE], me[,p2,drop=FALSE])

        # FAST trim
        p1 <- p2[ms_psig(ms_, summary_pars, summary_adjust,
                         test_cust, test_custom, lyrno,
                         mp[,colnames(ms_),drop=FALSE],
                         me[,colnames(ms_),drop=FALSE])]
        sig_phens <- append(sig_phens, p1)
        ms <- cbind(ms, ms_)

        lyrp <- lyr
        time_output(start2)
    }
    time_output(start1, "total time used to calcuate features + edge lists")
    start1 <- Sys.time()
    if (length(sig_phens[!sig_phens%in%""])==0) {
        warning("no significant cell population phenotypes found. Try again with another set of class labels or use `flowGraph` the ful constructor instead.")
        future::plan(future::sequential)
        return(NULL)
    }

    ## trim everything so there are only significant cell populations ####
    ms <- ms[,match(sig_phens, colnames(ms)),drop=FALSE]
    me <- me[,match(sig_phens, colnames(me)),drop=FALSE]
    mp <- mp[,match(sig_phens, colnames(mp)),drop=FALSE]
    rownames(ms) <- rownames(me) <- rownames(mp) <- rownames(input_)
    mc <- input_[,match(sig_phens, colnames(input_)),drop=FALSE]
    pparen <- pparen[names(pparen)%in%sig_phens]
    pparen <- purrr::map(pparen, function(x) x[x%in%sig_phens])
    pparen <- purrr::compact(pparen)
    pchild <- pchild[names(pchild)%in%sig_phens]
    pchild <- purrr::map(pchild, function(x) x[x%in%sig_phens])
    pchild <- purrr::compact(pchild)
    eps <- stringr::str_split(colnames(ep),"_")
    ep <- ep[,sapply(eps, function(x) all(x%in%sig_phens) ),drop=FALSE]
    edf <- as.data.frame(do.call(rbind, eps))
    names(edf) <- c("from","to")
    meta_cell <- meta_cell[meta_cell$phenotype%in%sig_phens,,drop=FALSE]
    markers <- extract_markers(sig_phens)

    # graph
    gr <- list(e=edf, v=meta_cell)
    gr <- set_layout_graph(gr, layout_fun) # layout cell hierarchy


    ## flowGraph ####
    desc <- do.call(rbind, list(
        summary_table(mc,"count"),
        summary_table(mp,"prop"),
        summary_table(me,"expect_prop"),
        summary_table(ms,"SpecEnr_prop")))
    desc_ <- summary_table(ep,"prop")
    fg <- methods::new(
        "flowGraph",
        feat=list(node=list(
            count=mc, prop=mp, expect_prop=me, SpecEnr_prop=ms),
            edge=list(prop=ep)),
        feat_desc=list(node=desc, edge=desc_),
        meta=meta,
        markers=markers,

        graph=gr, edge_list=list(child=pchild,parent=pparen),
        plot_layout=as.character(substitute(layout_fun)),
        etc=list(cumsumpos=fg@etc$cumsumpos, class_mean_normalized=FALSE,
                 original_count=input_)
    )


    ## calculate summary
    time_output(start1)
    start1 <- Sys.time()
    message("calculating summary statistics for SpecEnr")
    try({
        fg <- fg_summary(
            fg, no_cores=no_cores,
            class=summary_pars$class,
            label1=labels[1], label2=labels[2],
            node_features="SpecEnr",
            edge_features="NONE",
            overwrite=TRUE,
            test_custom=summary_pars$test_custom,
            test_name=summary_pars$test_name,
            diminish=summary_pars$diminish)
    })
    time_output(start1)


    ## save flowGraph object
    fg@etc$save <- list(id=stringi::stri_rand_strings(1,5))
    saved <- FALSE
    if (!is.null(path))
        tryCatch({
            fg_save(fg, path, save_plots=FALSE)
            saved <- TRUE
            fg@etc$save$path <- path
        }, error=function(e) {
            message("flowGraph object not saved; check path and try again with fg_save!")
        })

    if (save_plots & saved)
        fg_save_plots(fg, plot_path=paste0(path, "/plots"))

    time_output(start, "total time used")
    future::plan(future::sequential)
    return(fg)
}

## GRAPH -----------------------------------------------------------------------

#' @title Determines the layer on which a phenotype resides.
#' @description Determines the layer on which the given phenotypes reside.
#' @param phen A string vector of phenotype or cell population name labels.
#' @return A numeric vector with the same length as \code{phen} indicating which
#'    layer each phenotype resides on.
#' @details Given a vector of phenotypes, returns an equal length vector of
#'    the number of markers in each phenotype.
#' @examples
#'
#'    phen <- c('A+B+C-D++', 'A+B-', '', 'B++D-E+')
#'    cell_type_layers(phen)
#'
#' @seealso
#'    \code{\link[flowGraph]{get_phen_list}}
#'    \code{\link[flowGraph]{get_phen_meta}}
#' @rdname cell_type_layers
#' @export
#' @importFrom purrr map_int
#' @importFrom stringr str_split
cell_type_layers <- function(phen)
    purrr::map_int(stringr::str_split(phen, "[+-]+"), function(x) sum(x!=""))


#' @title Genrates phenotype meta data.
#' @description Generates phenotype meta data given a vector of
#'    phenotypes and optionally phenocodes.
#' @param phen A string vector of phenotype or cell population name labels.
#' @param phenocode A string vector of phenocodes corresponding
#'    to the phenotypes in \code{phen}.
#' @return A data frame with columns containing meta data on
#'    cell poulation nodes with columns:
#'    \itemize{
#'        \item{\code{phenotype}: cell population node label e.g. "A+B+".}
#'        \item{\code{phenocode}: a string penocode containing a numeric
#'         corresponding to the phenotype column e.g. "2200".}
#'        \item{\code{phenolayer}: a numeric layer on which a cell population
#'         resides in e.g. 2.}
#'    }
#' @examples
#'
#'    phen <- c('A+B+C-D++', 'A+B-', '', 'B++D-E+')
#'    phenc <- c('22130','21000','00000','03012')
#'    get_phen_meta(phen, phenc)
#'
#' @seealso
#'    \code{\link[flowGraph]{get_phen_list}}
#'    \code{\link[flowGraph]{cell_type_layers}}
#' @rdname get_phen_meta
#' @export
#' @importFrom stringr str_split str_extract_all str_count
#' @importFrom purrr map_chr map_int map
get_phen_meta <- function(phen, phenocode=NULL) {
    pm <- data.frame(phenotype=phen)
    markers <- unlist(stringr::str_split(phen, "[-+]"))
    markers <- gsub("_","",markers) # underscore between marker conditions
    markers <- unique(markers[markers != ""])
    if (is.null(phenocode)) {
        phenocode <- purrr::map_chr(phen, function(x) {
            if (x == "") return(paste0(rep(0, length(markers)), collapse=""))
            b <- stringr::str_split(x, "[+-]+[_]*")[[1]]
            b <- b[-length(b)]
            bo <- match(markers, b)
            phec <- purrr::map_int(
                unlist(stringr::str_extract_all(x, "[+-]+")), function(y)
                    stringr::str_count(y, "[+]")) + 1
            c <- phec[bo]
            c[is.na(c)] <- 0
            paste0(c, collapse="")
        })
    }
    pm$phenocode <- phenocode
    pm$phenolayer <- cell_type_layers(phen)
    pm$phenotype <- as.character(pm$phenotype)
    pm$phenogroup <- gsub("[+-]+", "_", pm$phenotype)

    return(pm)
}


#' @title Creates edge lists.
#' @description Creates edge lists indicating relationships
#' between cell populations
#'    given meta data on these cell populations
#'    produced by the \code{get_phen_meta} function.
#' @param meta_cell A data frame containing meta data on cell populations
#'    as produced by the \code{get_phen_meta} function.
#' @param phen A string vector of phenotype or cell population name labels.
#'    Cannot be set to \code{NULL} if \code{meta_cell} is set to \code{NULL}.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @return A list containing 'pchild', an edge list
#'    indicating where edges point to,
#'    'pparen', an edge list indicating where edges point from, and
#'    'edf', a data frame where each row contains
#'    the nodes an edge points 'from' and 'to'.
#' @examples
#'
#'    phen <- c('A+B-C+', 'A+B-', 'A+')
#'    get_phen_list(phen=phen)
#'
#' @seealso
#'    \code{\link[flowGraph]{get_phen_meta}}
#'    \code{\link[flowGraph]{cell_type_layers}}
#' @rdname get_phen_list
#' @export
#' @importFrom future plan multisession sequential
#' @importFrom stringr str_split str_extract_all
#' @importFrom Matrix Matrix
#' @importFrom purrr map compact map_chr
#' @importFrom furrr future_map
#' @importFrom data.table as.data.table setattr
get_phen_list <- function(meta_cell=NULL, phen=NULL, no_cores=1) {
    if (no_cores>1) future::plan(future::multisession)

    if (is.null(phen) & is.null(meta_cell))
        stop("give me something!")
    if (is.null(meta_cell))
        meta_cell <- get_phen_meta(phen)


    ## initialize cell population meta material for making edge list ####
    # indices for each layer
    dt <- data.table::as.data.table(meta_cell$phenolayer)[
        , list(list(.I)), by=meta_cell$phenolayer]
    lyril <- data.table::setattr(dt$V1, 'names', dt$meta_cell)
    lyrs <- length(lyril)

    # make phenocode matrix
    pc <- Matrix::Matrix(Reduce(
        rbind, lapply(stringr::str_split(meta_cell$phenocode,""),
                                    as.numeric)), sparse=TRUE)
    rownames(pc) <- meta_cell$phenotype
    allcolu <- apply(pc, 2,unique)
    if (!is.list(allcolu))
        allcolu <- split(allcolu, seq(ncol(allcolu)))

    # make allcol > markeri > 0/1/2 > T/F per phenotype
    allcol <- fpurrr_map(seq_len(length(allcolu)), function(ci) {
        a <- purrr::map(allcolu[[ci]], function(ui) pc[, ci]==ui)
        names(a) <- allcolu[[ci]]; return(a)
    }, no_cores=no_cores)

    # split everything above by layer
    pcs <- acs <- meta_cells <- list()
    for (lyri in names(lyril)) {
        li <- lyril[[lyri]]
        pcs[[lyri]] <- pc[li,,drop=FALSE]
        acs[[lyri]] <- purrr::map(allcol, purrr::map, function(y) y[li])
        meta_cells[[lyri]] <- meta_cell[li,, drop=FALSE]
    }

    ## initialize
    pchild <- pparen <- list()

    # root proportion is just 1, but for completeness we add it in
    if (all(c("0", "1")%in%names(lyril))) {
        ## initiate child and parent edge lists for layers 0/1
        pchild <- list(meta_cells[["1"]]$phenotype)
        names(pchild) <- ""
        pparen <- purrr::map(seq_len(length(pchild[[1]])), function(x) "")
        names(pparen) <- pchild[[1]]
    }


    ## calculate features for each layer ####
    lyrs <- sort(unique(meta_cell$phenolayer))
    lyrstf <- sapply(lyrs, function(x) (x-1)%in%lyrs)

    for (lyr in lyrs[lyrstf]) {
        start2 <- Sys.time()
        message("- ", length(lyril[[as.character(lyr)]]),
                        " pops @ layer ", lyr)

        lyrc_ <- as.character(lyr-1)
        lyrc__ <- as.character(lyr)
        meta_cell_grid_ <- pcs[[lyrc_]]
        meta_cell_grid__ <- pcs[[lyrc__]]
        allcol_ <- acs[[lyrc_]]
        allcol__ <- acs[[lyrc__]]
        meta_cell_ <- meta_cells[[lyrc_]]
        meta_cell__ <- meta_cells[[lyrc__]]

        # child
        pchildl <- fpurrr_map(seq_len(nrow(meta_cell_)), function(j) {
            mcgrow <- meta_cell_grid_[j, ]
            colj1 <- which(mcgrow > 0)
            chi <- Reduce("&", purrr::map(colj1, function(coli)
                allcol__[[coli]][[as.character(mcgrow[coli])]]))
            meta_cell__$phenotype[chi]
        }, no_cores=no_cores, prll=nrow(meta_cell__) > 5000)
        names(pchildl) <- meta_cell_$phenotype
        pchildl <- purrr::compact(pchildl)
        pchild <- append(pchild, pchildl)

        # paren
        pparenl <- fpurrr_map(seq_len(nrow(meta_cell__)), function(j) {
            mcgrow <- meta_cell_grid__[j, ]
            colj1 <- which(mcgrow > 0)
            chidf <- do.call(cbind, purrr::map(colj1, function(coli)
                allcol_[[coli]][[as.character(mcgrow[coli])]] ))
            chi <- apply(chidf, 1, function(x) sum(!x) == 1)
            meta_cell_$phenotype[chi]
        }, no_cores=no_cores, prll=nrow(meta_cell__) > 5000)
        names(pparenl) <- meta_cell__$phenotype
        pchildl <- purrr::compact(pparenl)
        pparen <- append(pparen, pparenl)

        time_output(start2)
    }
    edf <- do.call(rbind,purrr::map(seq_len(length(pchild)), function(x)
        data.frame(from=names(pchild)[x], to=pchild[[x]])) )

    temp_se <- function(x) stringr::str_extract_all(x, "[^_^+^-]+[+-]+")
    from_ <- temp_se(edf$from)
    to_ <- temp_se(edf$to)
    edf$marker <- unlist(fpurrr_map(seq_len(length(from_)), function(x)
        setdiff(to_[[x]], from_[[x]]), no_cores=no_cores, prll=nrow(edf)>5000))

    future::plan(future::sequential)
    return(list(pchild=pchild, pparen=pparen, edf=edf))
}



#' @title Determines cell hierarchy layout.
#' @description Determines cell hierarchy layout and returns the X, Y coordinate
#'    of each cell population.
#' @param gr A list containing data frames \code{e} and \code{v}.
#' @param layout_fun A string of a function from the \code{igraph} package that
#'    indicates what layout should be used if a cell hierarchy is to be ploted;
#'    all such functions have prefix \code{layout_}
#'    e.g. \code{layout_fun="layout.reingold.tilford"}.
#' @return A list containing data frames \code{e} and \code{v}; each data frame
#'    contains an X, Y column or coordinate for each node and edge.
#' @examples
#'
#'    no_cores <- 1
#'    data(fg_data_pos30)
#'    fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                                    prop=FALSE, specenr=FALSE,
#'                                    no_cores=no_cores)
#'
#'    head(set_layout_graph(fg_get_graph(fg)))
#'
#' @rdname set_layout_graph
#' @export
#' @import igraph
#' @importFrom igraph graph_from_data_frame layout.reingold.tilford V
#' @importFrom purrr map_int
set_layout_graph <- function(gr, layout_fun="layout.reingold.tilford") {
    # layout.circle, layout.reingold.tilford FUN is a layout
    # function from the igraph package assume
    # graph is connected, used internally

    gr_e <- gr$e
    gr_v <- gr$v

    # edit layout
    gr0 <- igraph::graph_from_data_frame(gr_e[,c("from","to")])
    if (layout_fun!="layout.reingold.tilford") {
        layout_fun <- get(layout_fun)
        gr_vxy_ <- layout_fun(gr0)
        gr_vxy <- as.data.frame(gr_vxy_)
    } else {
        gr_vxy_ <- igraph::layout.reingold.tilford(gr0)
        gr_vxy <- as.data.frame(gr_vxy_)

        ## edit layout manually
        gr_vxy <- space_hierarchy(gr_vxy)
        # turn plot sideways
        gr_vxy <- gr_vxy[, 2:1]
        xmax <- max(gr_vxy[, 1])
        if (gr_vxy[1, 1] == xmax)
            gr_vxy[, 1] <- xmax - gr_vxy[, 1]
    }

    # get node
    colnames(gr_vxy) <- c("x", "y")
    gr_v_xy <- gr_vxy[match(gr_v$phenotype, names(igraph::V(gr0)[[]])), ]
    gr_v$x <- gr_v_xy$x
    gr_v$y <- gr_v_xy$y

    # get edge
    e_match <- match(gr_e$from, gr_v$phenotype)
    gr_e$from.x <- gr_v$x[e_match]
    gr_e$from.y <- gr_v$y[e_match]
    e_match2 <- match(gr_e$to, gr_v$phenotype)
    gr_e$to.x <- gr_v$x[e_match2]
    gr_e$to.y <- gr_v$y[e_match2]

    return(list(v=gr_v, e=gr_e))
}

space_hierarchy <- function(gr_vxy) {
    gys <- sort(unique(gr_vxy[, 2]))
    gxns <- purrr::map_int(gys, function(y) sum(gr_vxy[, 2] == y))
    maxlayern <- max(gxns)
    maxlayer <- gys[which.max(gxns)]
    maxlayertf <- gr_vxy[, 2] == maxlayer
    gr_vxy[maxlayertf, 1] <- rank(gr_vxy[maxlayertf, 1]) - 1
    for (gy in gys) {
        if (gy == maxlayer)
            next()
        layertf <- gr_vxy[, 2] == gy
        gr_vxy[layertf, 1]=rank(gr_vxy[layertf, 1]) - 1 +
            floor((maxlayern - sum(layertf))/2)
    }
    return(gr_vxy)
}


#' @title Determines cell hierarchy layout.
#' @description Determines cell hierarchy layout and returns the X, Y coordinate
#'    of each cell population. This function is a wrapper for
#'    \code{\link[flowGraph]{set_layout_graph}}.
#' @param fg flowGraph object.
#' @param layout_fun A string version of a function name from
#'    the \code{igraph} package that
#'    indicates what layout should be used if a cell hierarchy is to be ploted;
#'    all such functions have prefix \code{layout_}
#'    e.g. \code{layout_fun="layout.reingold.tilford"}.
#' @return flowGraph object with coordinate meta data on cell populations
#'    and edges for plotting use.
#' @details Given a flowGraph object, modifies the \code{graph} slot such that
#'    it contains X, Y axes for each node in accordance to a user specified
#'    layout.
#' @examples
#'
#'    no_cores <- 1
#'    data(fg_data_pos30)
#'    fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                                    prop=FALSE, specenr=FALSE,
#'                                    no_cores=no_cores)
#'
#'    fg <- fg_set_layout(fg)
#'    head(fg_get_graph(fg)$v)
#'
#' @rdname fg_set_layout
#' @export
fg_set_layout <- function(fg, layout_fun="layout.reingold.tilford") {
    fg@plot_layout <- layout_fun
    fg@graph <- set_layout_graph(fg_get_graph(fg), layout_fun)
    return(fg)
}

#' @title Adds a feature.
#' @description Adds a feature created using \code{feat_fun} from \code{fg} OR
#'  \code{m} into a given flowGraph object. Only use this function if
#'  you cannot generate the desired features using the existing flowGraph
#'  functions starting with \code{fg_feat_<feature name>}.
#' @param fg flowGraph object.
#' @param type A string specifying the type of the feature being
#'  added i.e. 'node' or 'edge'.
#' @param feature A string indicating the unique name of the feature added.
#' @param m A numeric matrix with feature values; it should contain the
#'  same sample id's on row names as in \code{fg_get_meta(fg)$id}
#'  and node or edge names
#'  as column names (i.e. if \code{m} is a node feature, it would have the same
#'  column names as those in \code{fg_get_graph(fg)$v$phenotype};
#'  if it is an edge
#'  feature, its column names should be the same as
#'  \code{paste0(fg_get_graph(fg)$e$from, '_', fg_get_graph(fg)$e$to)}).
#' @param feat_fun A function that ouputs a feature matrix as in \code{m} given
#'  \code{fg} and other optional parameters.
#' @param overwrite A logical variable indicating whether or not the function
#'  should replace the existing feature with the same name if
#'  one is already in \code{fg}.
#' @param ... Other parameters that would be used as input into \code{feat_fun}.
#' @return flowGraph object.
#' @details \code{fg_add_feature} adds the given new feature matrix to the
#'  given flowGraph object \code{fg} updating slots
#'  \code{feat} and \code{feat_desc}.
#'  See \code{\link[flowGraph]{flowGraph-class}}
#'  slot \code{feat} and \code{feat_desc} for what should be in these slots.
#'  We do not recommend users to directly use this method unless there is
#'  a clear understanding on how the row and column names should be specified.
#'  Instead, we recommend users to use the functions listed in the "See also"
#'  sections prefixed with "fg_feat_".
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  fg_get_feature_desc(fg)
#'
#'  fg <- fg_add_feature(fg, type="node", feature="count_copy",
#'                       m=fg_data_pos30$count)
#'  fg_get_feature_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_feat_node_prop}}
#'  \code{\link[flowGraph]{fg_feat_node_specenr}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#' @rdname fg_add_feature
#' @export
fg_add_feature <- function(
    fg, type="node", feature, m=NULL, feat_fun=NULL, overwrite=FALSE, ...
) {
    type <- match.arg(type, c("node", "edge"))
    fg_feat <- fg_get_feature_all(fg)
    fg_feat_desc <- fg_get_feature_desc(fg)

    feat_names <- names(fg_feat[[type]])
    exists_ <- feature %in% feat_names
    if (exists_) {
        if (!overwrite) {
            message("skipped")
            return(fg)
        }
        f_ind <- feat_names != feature
        fg_feat_desc[[type]] <- fg_feat_desc[[type]][f_ind,,drop=FALSE]
        fg_feat[[type]] <- fg_feat[[type]][f_ind]
    }

    if (is.null(m)) {
        if (is.null(feat_fun))
            stop("please provide a feature matrix or a function to create one")
        m <- feat_fun(fg, ...)
    }

    fg@feat_desc[[type]] <-
        rbind(fg_feat_desc[[type]], summary_table(m, feature))
    fg@feat[[type]][[feature]] <- m

    return(fg)
}


#' @title Removes a feature.
#' @description Removes a feature from a given flowGraph object.
#' @param fg flowGraph object.
#' @param type A string specifying the type of the feature being
#'  removed i.e. 'node' or 'edge'.
#' @param feature A string indicating the unique name of the feature removed;
#'  note we cannot remove the 'node' 'count' feature type.
#' @return flowGraph object with specified feature removed.
#' @details \code{fg_rm_feature} removes a specified feature matrix from the
#'  given flowGraph object \code{fg} updating slots
#'  \code{feat} and \code{feat_desc}.
#'  See \code{\link[flowGraph]{flowGraph-class}}
#'  slot \code{feat} and \code{feat_desc} for what should be in these slots.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'  fg_get_feature_desc(fg)
#'
#'  fg <- fg_rm_feature(fg, type="node", feature="prop")
#'  fg_get_feature_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#' @rdname fg_rm_feature
#' @export
fg_rm_feature <- function(fg, type="node", feature=NULL) {
    type <- match.arg(type, c("node", "edge"))
    if (feature=="count" & type=="node")
        stop("cannot remove the count node feature from a flowGraph object.")
    ft_ind <- which(names(fg_get_feature_all(fg)[[type]]) == feature)
    fg@feat[[type]][[feature]] <- NULL
    if (length(ft_ind)==1) {
        fg@feat_desc[[type]] <-
            fg_get_feature_desc(fg)[[type]][-ft_ind,, drop=FALSE]
    } else {
        warning("feature not found, nothing was dropped")
    }
    # don't need the drop part, but just in case.
    return(fg)
}


#' @title Adds a feature summary.
#' @description Adds a feature summary into a given flowGraph object.
#'  Only use this function if your summary statistic cannot be calcuated
#'  using the \code{\link[flowGraph]{fg_summary}} function.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for;
#'  'node' or 'edge'.
#' @param summary_meta The user must provide \code{type} and
#'  \code{summary_meta}.
#'
#'  \code{summary_meta} is a list containing
#'  \code{feature} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param p A list containing summary values; this list contains elements:
#'  \code{values} (a vector containing summary statistics e.g. p-values;
#'   this vector should be named by their associated phenotype or edge name),
#'  \code{test_custom} (a function of the statistical test used), and
#'  \code{adjust_custom} (a function of the p-value correction method used).
#'  This list must contain the \code{values} element.
#' @param summ_fun  A function that ouputs a feature summary matrix
#'  as in \code{p} given \code{fg} and other optional parameters.
#' @param overwrite A logical variable indicating whether or not the function
#'  should replace the existing feature summary with the
#'  same name if one is already in \code{fg}.
#' @param ... Other parameters that would be used as input into \code{summ_fun}.
#' @return flowGraph object.
#' @details \code{fg_add_summary} adds the given feature summary list \code{p}
#'  or the output of the given function \code{summ_fun} to the
#'  given flowGraph object \code{fg} updating slots
#'  \code{summary} and \code{summary_desc}.
#'  See \code{\link[flowGraph]{flowGraph-class}}
#'  slot \code{summary} and \code{summary_desc}
#'  for what should be in these slots. We do not recommend users directly use
#'  this function unless what is required is duly in the above slots is
#'  well understood --- note these slots are used in plotting functions
#'  e.g. \code{\link[flowGraph]{fg_plot}}. We instead recommend users to use
#'  the \code{\link[flowGraph]{fg_summary}} function.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  # get samples that we are going to compare
#'  m <- fg_get_feature(fg, type="node", feature="prop")
#'  m1_ <- m[fg_data_pos30$meta$class=="control",,drop=FALSE]
#'  m2_ <- m[fg_data_pos30$meta$class=="exp",,drop=FALSE]
#'
#'  # define test or summary function to conduct comparison
#'  test_custom <- function(x,y)
#'      tryCatch(stats::t.test(x,y)$p.value, error=function(e) 1)
#'  values_p <- sapply(seq_len(ncol(m)), function(j)
#'      test_custom(m1_[,j], m2_[,j]) )
#'  values_p <- p.adjust(values_p , method="BY")
#'
#'  # the user can choose to fill either parameter "p" or "summ_fun",
#'  # the latter of which must output a list with the same elements as "p".
#'  # see documentation for ?flowGraph-class, slot "summary" for
#'  # details on what should be in "p".
#'  p <- list(values=values_p, test_fun=test_custom, adjust_fun="BY")
#'  fg <- fg_add_summary(fg, type="node", summary_meta=list(
#'       feature="prop", test_name="wilcox_BY",
#'       class="class", label1="control", label2="exp"), p=p)
#'
#'  fg_get_summary_desc(fg)
#'
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_summary}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#' @rdname fg_add_summary
#' @export
#' @importFrom purrr map_lgl compact
fg_add_summary <- function(
    fg, type="node", summary_meta=NULL,
    p=NULL, summ_fun=NULL, overwrite=FALSE, ...
) {
    type <- match.arg(type, c("node", "edge"))
    options(stringsAsFactors=FALSE)
    try({
        index <- fg_get_summary_index(
            fg, type=type, summary_meta=summary_meta)
        if (!overwrite) {
            message("summary exists, skipped")
            return(fg)
        }
        fg <- fg_rm_summary(fg, type=type, index=index)
    }, silent=TRUE)

    if (is.null(p)) {
        if (is.null(summ_fun))
            stop("provide summary statistic values or a function to create one")
        p <- summ_fun(fg, type=type, ...)  # list(values, test, adjust)
    }

    # legacy, left here just in case
    if ("m1"%in%names(p)) p[["m1"]] <- NULL
    if ("m2"%in%names(p)) p[["m2"]] <- NULL
    p <- purrr::compact(p)

    sm <- data.frame(matrix(summary_meta, nrow=1))
    colnames(sm) <- c("feat", "test_name","class","label1", "label2")
    if (length(fg_get_summary_desc(fg)[[type]])==0) {
        fg@summary_desc[[type]] <- sm
    } else {
        fg@summary_desc[[type]] <- rbind(fg_get_summary_desc(fg)[[type]], sm)
        rownames(fg@summary_desc[[type]]) <- NULL
    }

    if (is.null(fg_get_summary_all(fg)[[type]])) fg@summary[[type]] <- list()
    fg@summary[[type]][[nrow(fg_get_summary_desc(fg)[[type]])]] <- p

    return(fg)
}


#' @title Removes a feature summary.
#' @description Removes a feature summary from a given flowGraph object;
#'  while \code{fg} is required, the user can choose to input parameters
#'  \code{summary_meta}, \code{index}, or all of \code{type},
#'  \code{feat}, \code{test_name}, \code{class}, \code{label1},
#'   and \code{label2}.
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @param fg flowGraph object.
#' @param type A string indicating feature type the summary was created for;
#'  'node' or 'edge'.
#' @param index The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{index} is an integer indicating the row in
#'  \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
#'  summary the user would like to retrieve.
#' @param summary_meta The user must provide \code{type} and
#'  additionally, one of \code{summary_meta} or \code{index}.
#'
#'  \code{summary_meta} is a list containing

#'  \code{feat} (feature name), \code{test_name} (summary statistic name),
#'  \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
#'  See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
#' @return flowGraph object.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
#'                   overwrite=FALSE, test_name="wilcox_byLayer", diminish=FALSE,
#'                   node_features=NULL, edge_features=NULL)
#'  fg_get_summary_desc(fg)
#'
#'  fg <- fg_rm_summary(fg, summary_meta=c(
#'      feature="count",test_name="wilcox_byLayer",
#'      class="class", label1="control", label2="exp"))
#'  fg_get_summary_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#' @rdname fg_rm_summary
#' @export
#' @importFrom purrr map_lgl
fg_rm_summary <- function(fg, type="node", index=NULL, summary_meta=NULL) {
    type <- match.arg(type, c("node", "edge"))
    index <- fg_get_summary_index(fg,type=type, index,summary_meta)
    fg@summary[[type]][[index]] <- NULL
    # don't need the drop part, but just in case.
    fg@summary_desc[[type]] <-
        fg_get_summary_desc(fg)[[type]][-index,, drop=FALSE]
    return(fg)
}

#' @title Clears all featuresin a flowGraph object.
#' @description Returns a flowGraph object with only the \code{count} feature.
#' @param fg flowGraph object.
#' @return flowGraph object with only the \code{count} \code{node} feature.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_clear_features(fg)
#'  fg_get_summary_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#' @rdname fg_clear_features
#' @export
#' @importFrom purrr compact
fg_clear_features <- function(fg) {
    if (length(fg_get_feature_all(fg)$node)>1)
        fg@feat$node[!names(fg_get_feature_all(fg)$node)%in%"count"] <- NULL
    fg@feat$node <- purrr::compact(fg_get_feature_all(fg)$node)
    fg@feat$edge <- fg@feat_desc$edge <- list()
    fg@feat_desc$node <-
        fg_get_feature_desc(fg)$node[fg_get_feature_desc(fg)$node$feat=="count",]
    return(fg)
}


#' @title Removes all summary statistics.
#' @description Removes all summary statistics in a flowGraph object;
#'  we recommend doing this to save space.
#' @param fg flowGraph object.
#' @return flowGraph object with an empty \code{summary} slot.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores, node_features="count")
#'  fg_get_summary_desc(fg)
#'
#'  fg <- fg_clear_summary(fg)
#'  fg_get_summary_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_summary}}
#' @rdname fg_clear_summary
#' @export
fg_clear_summary <- function(fg) {
    fg@summary <- fg@summary_desc <- list()
    fg@etc$actualVSexpect <- list()
    return(fg)
}


#' @title Clears all features and feature summaries in a flowGraph object.
#' @description Returns a flowGraph object with only the \code{count} feature
#'  and meta data. This function clears all other features and
#'  feature summaries to save space.
#' @param fg flowGraph object.
#' @return flowGraph object with all summary statistics and feature values
#'  removed except for the node count feature.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_extract_raw(fg)
#'  show(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#' @rdname fg_extract_raw
#' @export
fg_extract_raw <- function(fg) {
    fg <- fg_clear_summary(fg)
    fg <- fg_clear_features(fg)
    return(fg)
}

#' @title Replace marker names.
#' @description Replace marker names in a flowGraph object.
#' @param fg flowGraph object.
#' @param markers_new A string vector of new marker names;
#'  if \code{markers_old} is set to \code{NULL},
#'  each marker in \code{markers_new} should correspond to
#'  each marker in the \code{markers} slot of the \code{flowGraph} object.
#' @param markers_old A string vector of old marker names user wants to replace;
#' these marker names corresponding to those
#' in \code{fg_get_markers(fg)} with the same length as \code{markers_new}.
#' If \code{markers_old=NULL}, \code{markers_new} should be the same length as
#' \code{fg_get_markers(fg)}.
#' @return flowGraph object with marker names replaced.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_gsub_markers(fg, c("Anew", "Bnew", "Cnew", "Dnew"))
#'  fg_get_feature_desc(fg)
#'
#' @rdname fg_gsub_markers
#' @export
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_gsub_ids}}
#' @importFrom purrr map
fg_gsub_markers <- function(fg, markers_new, markers_old=NULL) {

    if (is.null(markers_old)) {
        if (length(markers_new) != length(fg_get_markers(fg)))
            stop("incorrect number of markers\n")

        markers_old <- fg_get_markers(fg)
    } else {
        if (length(markers_old) != length(markers_new))
            stop("incorrect number of markers\n")
    }
    for (mi in seq_len(length(markers_old))) {
        fg_graph <- fg_get_graph(fg)
        fg@graph$v$phenotype <-
            gsub(markers_old[mi], markers_new[mi], fg_graph$v$phenotype)
        fg@graph$e$from <-
            gsub(markers_old[mi], markers_new[mi], fg_graph$e$from)
        fg@graph$e$to <-
            gsub(markers_old[mi], markers_new[mi], fg_graph$e$to)
        fg@graph$e$marker <-
            gsub(markers_old[mi], markers_new[mi], fg_graph$e$marker)

        fg@edge_list$parent <-
            purrr::map(fg@edge_list$parent, function(x)
                gsub(markers_old[mi], markers_new[mi], x))
        names(fg@edge_list$parent) <-
            gsub(markers_old[mi], markers_new[mi], names(fg@edge_list$parent))
        fg@edge_list$child <-
            purrr::map(fg@edge_list$child, function(x)
                purrr::map(x, function(y)
                    gsub(markers_old[mi], markers_new[mi], y) ))
        names(fg@edge_list$child) <-
            gsub(markers_old[mi], markers_new[mi], names(fg@edge_list$child))
    }
    fg_graph <- fg_get_graph(fg)
    fg@feat$node <- purrr::map(fg_get_feature_all(fg)$node, function(x) {
        colnames(x) <- fg_graph$v$phenotype
        x
    })
    if (length(fg_get_feature_all(fg)$edge) > 0) {
        ecn <- paste0(fg_graph$e$from, "__", fg_graph$e$to)
        fg@feat$edge <- purrr::map(fg_get_feature_all(fg)$edge, function(x) {
            colnames(x) <- ecn
            x
        })
    }

    fg@markers[match(markers_old, fg_get_markers(fg))] <- markers_new
    return(fg)
}

#' @title Replace sample id's.
#' @description Replace sample id's in a flowGraph object.
#' @param fg flowGraph object.
#' @param ids_new A string vector of new sample id's; if \code{ids_old} is
#' set to \code{NULL}, each id in \code{ids_new} should correspond to
#' each id in \code{fg_get_meta(fg)$id}.
#' @param ids_old A string vector of old sample id's the user wants to replace;
#' these marker names corresponding to those
#' in \code{fg_get_meta(fg)$id} with the same length as \code{ids_new}.
#' If \code{ids_old=NULL}, \code{ids_new} should be the same length as
#' \code{fg_get_meta(fg)$id}.
#' @return flowGraph object with sample id's replaced.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg <- fg_gsub_ids(fg, ids_new=paste0(fg_get_meta(fg)$id, "_new"))
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_gsub_markers}}
#' @rdname fg_gsub_ids
#' @export
#' @importFrom purrr map
fg_gsub_ids <- function(fg, ids_new, ids_old=NULL) {

    fg_meta <- fg_get_meta(fg)
    fg_feat <- fg_get_feature_all(fg)
    if (is.null(ids_old)) {
        if (length(ids_new) != nrow(fg_meta))
            stop("incorrect number of ids\n")

        ids_old <- fg_meta$id
    } else if (length(ids_old) != length(ids_new)) {
        stop("incorrect number of ids\n")
    }

    ids_ind <- match(ids_old, fg_meta$id)
    fg@meta$id[ids_ind] <- ids_new
    fg@feat$node <- purrr::map(fg_feat$node, function(x) {
        rownames(x)[ids_ind] <- ids_new
        x
    })
    if (length(fg_feat$edge) > 0) {
        fg@feat$edge <- purrr::map(fg_feat$edge, function(x) {
            rownames(x)[ids_ind] <- ids_new
            x
        })
    }

    fg@feat_desc <- fg_get_feature_desc(fg, re_calc=TRUE)
    return(fg)
}


#' @title Merges the samples from two flowGraph objects.
#' @description Merges the samples from two flowGraph objects together;
#'  we recommend removing all summary statistics from the new flowGraph object
#'  as those won't be adjusted: \code{\link[flowGraph]{fg_clear_summary}}.
#' @param fg1 flowGraph object.
#' @param fg2 flowGraph object.
#' @return flowGraph object.
#' @details Appends the samples from \code{fg2} onto those in \code{fg1}.
#'  This function requires that the two flowGraph objects must have the
#'  same phenotypes. Therefore, we recommend users to use,
#'  instead, \code{\link[flowGraph]{fg_merge}}.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg1 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[1:5])
#'  fg2 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[4:7])
#'  fg <- fg_merge_samples(fg1, fg2)
#'  fg_get_feature_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_merge}}
#'  \code{\link[flowGraph]{fg_extract_samples}}
#' @rdname fg_merge_samples
#' @export
#' @importFrom purrr map
fg_merge_samples <- function(fg1, fg2) {
    fg <- fg1

    fg1_meta <- fg_get_meta(fg1)
    fg2_meta <- fg_get_meta(fg2)
    fg1_feat <- fg_get_feature_all(fg1)
    fg2_feat <- fg_get_feature_all(fg2)

    cnames = intersect(colnames(fg1_meta), colnames(fg2_meta))

    id2 <- setdiff(fg2_meta$id, fg1_meta$id)
    meta2 <- fg2_meta[fg2_meta$id%in%id2,cnames,drop=FALSE]
    meta1 <- fg1_meta[,cnames,drop=FALSE]

    fg@meta <- rbind(meta1, meta2)
    nfs <- intersect(names(fg1_feat$node),
                           names(fg2_feat$node))
    fg@feat$node=purrr::map(nfs, function(xi) {
        a <- fg1_feat$node[[xi]]
        b <- fg2_feat$node[[xi]][id2, , drop=FALSE]
        abcol <- intersect(colnames(a), colnames(b))
        ab <- rbind(a[,match(abcol, colnames(a)), drop=FALSE],
                          b[match(setdiff(rownames(b),
                                                      rownames(a)),
                                        rownames(b)),
                            match(abcol, colnames(b)), drop=FALSE])
    })
    names(fg@feat$node) <- nfs
    if (length(fg@feat$edge) > 0) {
        efs <- intersect(names(fg1_feat$edge),
                               names(fg2_feat$edge))
        fg@feat$edge <- purrr::map(efs, function(xi) {
            a <- fg1_feat$edge[[xi]]
            b <- fg2_feat$edge[[xi]][id2,, drop=FALSE]
            abcol <- intersect(colnames(a),
                                     colnames(b))
            ab <- rbind(
                a[, match(abcol, colnames(a)), drop=FALSE],
                b[match(setdiff(rownames(b),
                                            rownames(a)),
                              rownames(b)),
                  match(abcol, colnames(b)),drop=FALSE])
        })
        names(fg@feat$edge) <- efs
    }

    fg@feat_desc <- fg_get_feature_desc(fg, re_calc=TRUE)
    fg <- fg_clear_summary(fg)
    warning("merging two flowGraph objects will clear the summary statistics")
    return(fg)
}


#' @title Extracts a set of samples from a flowGraph object.
#' @description Extracts or removes a specified set of samples from
#'  a flowGraph object.
#' @param fg flowGraph object.
#' @param sample_ids A string vector of sample id's that the user wants to
#'  keep in \code{fg}.
#' @param rm_summary A logical indicating whether or not to clear summary.
#' @return flowGraph object.
#' @details The summaries in \code{fg} will not be modified;
#'  we recommend the user recalculates them.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  fg_get_feature_desc(fg0)
#'
#'  fg <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[1:5])
#'  fg_get_feature_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_merge}}
#'  \code{\link[flowGraph]{fg_extract_phenotypes}}
#' @rdname fg_extract_samples
#' @export
#' @importFrom purrr map
fg_extract_samples <- function(fg, sample_ids, rm_summary=TRUE) {
    fg_meta <- fg_get_meta(fg)
    if (!any(sample_ids %in% fg_meta$id))
        stop("please provide valid sample id's; see @meta$id\n")


    id_inds <- match(sample_ids, fg_meta$id)

    fg@meta <- fg_meta[id_inds, , drop=FALSE]

    fg_feat <- fg_get_feature_all(fg)
    fg@feat$node <- purrr::map(fg_feat$node, function(x)
        x[id_inds, , drop=FALSE])
    if (length(fg_feat$edge) > 0)
        fg@feat$edge <- purrr::map(fg_feat$edge, function(x)
            x[id_inds,, drop=FALSE])

    fg@feat_desc <- fg_get_feature_desc(fg, re_calc=TRUE)

    fg_summary <- fg_get_summary_all(fg)
    if (!is.null(fg_summary$node) | !is.null(fg_summary$edge))
        if (rm_summary) {
            warning("subsetting samples mean that summary statistics will no longer be valide, removing summary statistics.")
            fg <- fg_clear_summary(fg)
        }
    return(fg)
}


#' @title Extracts a set of phenotypes from a flowGraph object.
#' @description Extracts or removes a specified set of
#'  phenotypes from a flowGraph object.
#' @param fg flowGraph object.
#' @param phenotypes A string vector of phenotype or
#'  cell population name labels.
#' @return flowGraph object.
#' @details The \code{summary} in \code{fg} will not be modified;
#'  we recommend users recalculate them.
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  fg_get_feature_desc(fg0)
#'
#'  fg <- fg_extract_phenotypes(fg0, fg_get_graph(fg0)$v$phenotype[1:10])
#'  fg_get_feature_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_merge}}
#'  \code{\link[flowGraph]{fg_extract_samples}}
#'  \code{\link[flowGraph]{fg_merge_samples}}
#' @rdname fg_extract_phenotypes
#' @export
#' @importFrom purrr map compact
fg_extract_phenotypes <- function(fg, phenotypes) {
    if (!any(phenotypes %in% fg@graph$v$phenotype))
        stop("please provide valid phenotypes; see @graph$v\n")

    fg_graph <- fg_get_graph(fg)

    id_inds <- fg_graph$v$phenotype %in% phenotypes
    id_inds_ <- fg_graph$e$from %in% phenotypes &
        fg_graph$e$to %in% phenotypes

    fg@graph$v <- fg_graph$v[id_inds,, drop=FALSE]
    fg@graph$e <- fg_graph$e[id_inds_,, drop=FALSE]
    fg <- fg_set_layout(fg, fg_get_plot_layout(fg))

    fg_feat <- fg_get_feature_all(fg)
    fg@feat$node <- purrr::map(fg_feat$node, function(x)
        x[, id_inds, drop=FALSE])
    if (length(fg_feat$edge) > 0)
        fg@feat$edge <- purrr::map(fg_feat$edge, function(x)
            x[, id_inds_, drop=FALSE])

    fg@edge_list$child <-
        fg@edge_list$child[phenotypes[phenotypes %in%
                                          names(fg@edge_list$child)]]
    fg@edge_list$child <- purrr::map(fg@edge_list$child, function(x)
        purrr::map(x, function(y) {
            a <- y[y %in% phenotypes]
            if (length(a) == 0)
                return(NULL)
            a
        }))

    fg@edge_list$child <- purrr::compact(fg@edge_list$child)
    fg@edge_list$parent <-
        fg@edge_list$parent[phenotypes[phenotypes %in%
                                           names(fg@edge_list$parent)]]
    fg@edge_list$parent <- purrr::map(fg@edge_list$parent, function(x) {
        a <- x[x %in% phenotypes]
        if (length(a) == 0)
            return(NULL)
        a
    })
    fg@edge_list$parent <- purrr::compact(fg@edge_list$parent)

    if (length(fg@summary)>0) {
        if (length(fg@summary$node)>0)
            fg@summary$node <- purrr::map(fg@summary$node, function(x) {
                x$values <- x$values[id_inds]
                x
            })
        if (length(fg@summary$edge)>0)
            fg@summary$edge <- purrr::map(fg@summary$edge, function(x) {
                x$values <- x$values[id_inds_]
                x
            })
    }

    fg@feat_desc <- fg_get_feature_desc(fg, re_calc=TRUE)
    return(fg)
}


#' @title Merges two flowGraph objects together.
#' @description Merges two flowGraph objects together.
#' @param fg1 flowGraph object.
#' @param fg2 flowGraph object.
#' @param method_sample A string indicating how samples from flowGraph objects
#'  should be merged:
#'  \itemize{
#'    \item{\code{union}: keep all samples from both flowGraph objects;
#'     in this case \code{method_phenotype} must be \code{intersect}.}
#'    \item{\code{intersect}: keep only samples that exist
#'     in both \code{fg1} and \code{fg2}.}
#'    \item{\code{setdiff}: keep only samples that exist
#'     in \code{fg1} and not in \code{fg2}.}
#'    \item{\code{none}: keep all samples in \code{fg1}.}
#'  }
#' @param method_phenotype  A string indicating how phenotypes from
#' flowGraph objects should be merged:
#'  \itemize{
#'   \item{\code{intersect}: keep only phenotypes that exist in both
#'    \code{fg1} and \code{fg2}.}
#'   \item{\code{setdiff}: keep only phenotypes that exist in
#'    \code{fg1} and not in \code{fg2}.}
#'   \item{\code{none}: keep all phenotypes in \code{fg1}.}
#'  }
#' @return flowGraph object.
#' @details \code{fg_merge} is a generic function that merges the samples and
#'  phenotypes of two flowGraph objects.
#'  Note that if \code{method_sample="union"}
#'  then \code{method_phenotype} must be set to "intersect".
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'
#'  fg1 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[1:5])
#'  fg2 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[4:7])
#'  fg <- fg_merge(fg1, fg2, method_sample="intersect",
#'                           method_phenotype="intersect")
#'  fg_get_feature_desc(fg)
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_extract_samples}}
#'  \code{\link[flowGraph]{fg_extract_phenotypes}}
#'  \code{\link[flowGraph]{fg_merge_samples}}
#' @rdname fg_merge
#' @export
fg_merge <- function(
    fg1, fg2, method_sample=c("union", "intersect", "setdiff", "none"),
    method_phenotype=c("intersect", "setdiff", "none")
) {
    method_sample <- match.arg(method_sample)
    method_phenotype <- match.arg(method_phenotype)

    if (method_sample == "union" & method_phenotype != "intersect")
        stop("if method_sample=union,
             then we must set method_phenotype=intersect;\n
             otherwise, features become difficult to compare.")
    if (method_sample == "union")
        return(fg_merge_samples(fg1, fg2))
    if (method_sample == "none" & method_phenotype == "none")
        return(fg1)

    sample_id1 <- fg_get_meta(fg1)$id
    sample_id2 <- fg_get_meta(fg2)$id
    sample_id1_int <- intersect(sample_id1, sample_id2)
    sample_id1_new <- setdiff(sample_id1, sample_id2)
    sample_id1_uni <- union(sample_id1, sample_id2)

    phen1 <- fg_get_graph(fg1)$v$phenotype
    phen2 <- fg_get_graph(fg2)$v$phenotype
    phen1_int <- intersect(phen1, phen2)
    phen1_new <- setdiff(phen1, phen2)
    phen1_uni <- union(phen1, phen2)

    if (method_sample == "none")
        sample1 <- fg1

    if (method_sample == "intersect") {
        if (length(sample_id1_int) == 0)
            stop("no intersecting samples")
        sample1 <- fg_extract_samples(fg1, sample_id1_int)
    }

    if (method_sample == "setdiff") {
        if (length(sample_id1_new) == 0)
            stop("no setdiff samples")
        sample1 <- fg_extract_samples(fg1, sample_id1_new)
    }

    if (method_phenotype == "none")
        return(sample1)

    if (method_phenotype == "intersect") {
        if (length(phen1_int) == 0)
            stop("no intersecting phenotypes")
        if (all(phen1_int %in% phen2))
            return(sample1)
        return(fg_extract_phenotypes(sample1, phen1_int))
    }

    if (method_phenotype == "setdiff") {
        if (length(phen1_new) == 0)
            stop("no setdiff phenotypes")
        if (all(phen1_new %in% phen1))
            return(sample1)
        return(fg_extract_phenotypes(sample1, phen1_new))
    }

}


#' @title Replaces sample meta.
#' @description Replaces sample meta in a given flowGraph object.
#' @param fg flowGraph object.
#' @param meta A data frame containing meta data; see details in
#'  \code{\link[flowGraph]{flowGraph-class}}.
#' @return A flowGraph object with an updated sample meta.
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_meta}}
#' @examples
#'
#'  no_cores <- 1
#'  data(fg_data_pos30)
#'  fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                  prop=FALSE, specenr=FALSE,
#'                  no_cores=no_cores)
#'  head(fg_get_meta(fg))
#'
#'  new_df <- fg_data_pos30$meta
#'  new_df$id[1] <- "newID"
#'
#'  fg <- fg_replace_meta(fg, new_df)
#'  head(fg_get_meta(fg))
#'
#' @rdname fg_replace_meta
#' @export
#' @importFrom purrr map
fg_replace_meta <- function(fg, meta) {
    if (nrow(meta)!=nrow(fg_get_meta(fg)))
        stop("meta must have same number of rows as the number of samples")
    if (!"id"%in%colnames(meta)) stop("meta must have id column")
    fg@meta <- meta

    fg_feat <- fg_get_feature_all(fg)
    fg@feat$node <- purrr::map(fg_feat$node, function(x) {
        rownames(x) <- meta$id
        x
    })
    if (length(fg_feat$edge) > 0)
        fg@feat$edge <- purrr::map(fg_feat$edge, function(x) {
            rownames(x) <- meta$id
            x
        })

    return(fg)
}

#' @name flowGraph
#' @title flowGraph object constructor.
#' @description Initializes a \code{flowGraph} object given the cell counts
#'  for one or more flow cytometry sample(s).
#'  The flowGraph object returned holds meta data
#'  for each sample, each cell population node,
#'  edges representing how each cell population node relate to one another,
#'  and features for these nodes and edges.
#' @param input_ Any of the following:
#' \itemize{
# #'   \item{a \code{Phenotypes} object from the \code{ftf***} package.}
#'   \item{a numeric matrix or vector of the cell counts;
#'   its column/names must be the phenotype names and its rownames
#'   must be sample ID's.}
# #'   \item{a list of \code{Phenotypes} object(s).}
# #'   \item{a vector of \code{Phenotypes} object full paths.}
#' }
#' All input samples should have the same \code{markers} and
#' \code{partitionsPerMarker}.
# #' If given \code{Phenotype} object(s),
# #' the function will look to generate phenotype names from the
# #' \code{Phenotypes@@PhenoCodes} or \code{rownames(Phenotypes@@MFIs)} slots.
#' @param meta A data frame with meta data for each \code{Phenotypes} or sample;
#'  One of its column names should be "id" whose values correspond to
#'  the name of each \code{Phenotypes} object. We also recommend for it to have
#'  a column named "class" where one of its unique values is "control".
#' @param class A string corresponding to the column name or index
#'  of \code{meta} whose values represent
#'  the class label of each sample; OR a vector the same length
#'  as the the number of samples in \code{input_} specifiying the class
#'  of each given sample --- this vector will be appended to \code{meta} under
#'  column name \code{class}.
#' @param no_cores An integer indicating how many cores to parallelize on.
#' @param markers A string vector of marker names used in \code{input_}.
#' @param layout_fun A string of a function from the \code{igraph} package that
#'  indicates what layout should be used if a cell hierarchy is to be ploted;
#'  all such functions have prefix \code{layout_}. This is defaulted to
#'  e.g. \code{layout_fun="layout.reingold.tilford"}.
#' @param max_layer And integer indicating the maximum layer in the cell
#'  hierarchy to analyze; set to `NULL` to analyze all layers.
#' @param cumsumpos A logical variable indicating whether
#'  or not to cumulate cell counts;
#'  this applies only when \code{partitionsPerMarker > 3} and will convert
#'  e.g. the count of A+ or A++ into the sum of the counts of
#'  A+, A++, A+++, ..., or A++, A+++, ... .
#' @param prop A logical variable indicating whether or not to
#'  calculate the proportion feature;
#'  this can be done later on with \code{flowGraph_prop}.
#' @param specenr logical variable: whether or not to calculate
#'  the SpecEnr feature, Default: T
#' @param path A string indicating the folder path to where the flowGraph
#'  object should save its elements, Default = NULL (don't save).
#' @param calculate_summary A logical variable indicating whether or not to
#'  calculate the summary statistics for SpecEnr based on default parameters
#'  using the \code{fg_summary} summary function on class
#'  specified in parameter \code{class}.
#' @param node_features A string vector indicating which node feature(s)
#'  to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
#'  and the function will perform summary statistics on all or no node features.
#' @param edge_features A string vector indicating which edge feature(s)
#'  to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
#'  and the function will perform summary statistics on all or no edge features.
#' @param test_name A string with the name of the test you are performing.
#' @param test_custom See \code{\link[flowGraph]{fg_summary}}.
#' @param diminish A logical variable; applicable if \code{calculate_summary} is
#'  \code{TRUE}; see \code{\link[flowGraph]{fg_summary}}.
#' @param label1 A string indicating a class label in
#'  \code{fg_get_meta(fg)[,class]}; set to \code{NULL} if you would
#'  like to compare all classes aganst all classes;
#'  applicable if \code{calculate_summary} is \code{TRUE}.
#' @param label2 A string indicating a class label in
#'  \code{fg_get_meta(fg)[,class]};
#'  applicable if \code{calculate_summary} is \code{TRUE}.
# #' @param normalize A logical variable indicating whether or not to
# #'  calculate normalized cell count;
# #'  this can be done later on with \code{fg_feat_node_norm}.
# #' @param norm_ind See \code{\link[flowGraph]{fg_feat_node_norm}}.
# #' @param norm_layer See \code{\link[flowGraph]{fg_feat_node_norm}}.
# #' @param norm_path See \code{\link[flowGraph]{fg_feat_node_norm}}.
#' @param save_plots A logical indicating whether or not to save plots.
# #' @param ... Other parameters used in the \code{fg_save_plots} function
# #'  of the \code{edgeR} package. See \code{\link[flowGraph]{tmm}}.
#' @return flowGraph object
#' @details \code{flowGraph} is the constructor for the \code{flowGraph} object.
#'  The user can choose to input as \code{input_} a vector, a \code{Phenotypes}
#'  object (meaning there is only one sample), a matrix, or a \code{Phenotypes}
#'  object list. If the user is also inputting a sample meta data frame, it
#'  must contain a \code{id} column corresponding to sample names.
#' @examples
#'
#' no_cores <- 1
#' samplen <- 10
#' meta_file <- data.frame(
#'     id=1:samplen,
#'     class=append(rep("control", samplen/2), rep("exp", samplen/2)),
#'     stringsAsFactors=FALSE
#' )
#'
#'
#' ## using the constructor -----------------------
#'
#' data(fg_data_pos30)
#'
#' # input: vector of load-able Phenotypes paths
#' fg <- flowGraph(fg_data_pos30$count[1,], no_cores=no_cores)
#'
#' # input: matrix + vector of class corresponding to samples
#' fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#'                 no_cores=no_cores)
#' # - save to file directly
#' # fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#' #                no_cores=no_cores, path="path_to_folder)
#'
#' # input: matrix + meta data frame
#' # fg <- flowGraph(fg_data_pos30$count, meta=fg_data_pos30$meta,
#' #                 no_cores=no_cores)
#'
#'
#' @seealso
#'  \code{\link[flowGraph]{flowGraph-class}}
#'  \code{\link[flowGraph]{fg_get_feature}}
#'  \code{\link[flowGraph]{fg_get_feature_desc}}
#'  \code{\link[flowGraph]{fg_get_summary}}
#'  \code{\link[flowGraph]{fg_get_summary_desc}}
#'  \code{\link[flowGraph]{fg_add_feature}}
#'  \code{\link[flowGraph]{fg_rm_feature}}
#'  \code{\link[flowGraph]{fg_add_summary}}
#'  \code{\link[flowGraph]{fg_rm_summary}}
#'  \code{\link[flowGraph]{fg_gsub_markers}}
#'  \code{\link[flowGraph]{fg_gsub_ids}}
#'  \code{\link[flowGraph]{fg_merge_samples}}
#'  \code{\link[flowGraph]{fg_extract_samples}}
#'  \code{\link[flowGraph]{fg_extract_phenotypes}}
#'  \code{\link[flowGraph]{fg_merge}}
#'  \code{\link[doParallel]{registerDoParallel}}
#'  \code{\link[Matrix]{Matrix}}
#' @rdname flowGraph
#' @export
#' @importFrom stringr str_split
#' @importFrom stringi stri_rand_strings
#' @importFrom future plan multisession sequential
#' @importFrom furrr future_map
#' @importFrom purrr map_lgl map_chr map_dfr map compact map_int
#' @importFrom Matrix Matrix
#' @importFrom methods new
#' @importFrom igraph layout.reingold.tilford
flowGraph <- function(
    input_, meta=NULL, class="class", no_cores=1, markers=NULL,
    layout_fun="layout.reingold.tilford", # layout.circle

    max_layer=NULL,
    cumsumpos=FALSE, # whether to make positives +=+/++ (cumsum)
    prop=TRUE, specenr=TRUE,

    path=NULL,

    # summary parameters
    calculate_summary=TRUE, node_features="SpecEnr", edge_features="NONE",
    test_name="t_diminish",
    test_custom="t",
    diminish=TRUE, label1=NULL, label2=NULL,

    # plotting parameters
    save_plots=FALSE

) {
    options(stringsAsFactors=FALSE)

    if (!is.null(meta))
        if (!"id"%in%colnames(meta))
            stop("meta must have a column named \"id\" containing sample ID's")

    start <- start1 <- Sys.time()
    message("preparing input; ")

    # convert input_ into a cell count matrix ----------------------------------
    msg <- "make sure input type is correct"

    phenocode <- NULL
    if (any(class(input_)%in%c("integer","numeric")) |
        grepl("matrix",class(input_), ignore.case=TRUE)) {
        ## feature: count (sample x cell population)
        if (is.null(dim(input_))) {
            mc <- mc0 <- matrix(input_,nrow=1)
            colnames(mc) <- names(input_)
            rownames(mc) <- "s1"
        } else {
            mc <- mc0 <- input_
            if (is.null(rownames(input_)))
                rownames(input_) <-
                    paste0("s",seq_len(nrow(input_)))
            rownames(mc) <- rownames(input_)
        }

        ## make meta for samples
        if (is.null(meta)) sample_id <- rownames(mc)

        ## make meta for cell populations
        if (!any(grepl("[+-]",colnames(mc)))) stop(msg)

        phen <- colnames(mc)
        if (is.null(markers)) {
            markers <- unlist(stringr::str_split(phen,"[-+]+"))
            markers <- gsub("_","",markers) # underscore between marker conditions
            markers <- unique(markers[markers!=""])
        }

    } else {
        stop("input must be a numeric sample x cell population matrix.")
    }

    # prepare sample meta ----------------------------------
    if (!is.null(meta)) {
        if (nrow(meta)!= nrow(mc))
            stop("meta data and cell count matrix have different row counts")
        if (!"id"%in%colnames(meta))
            stop("meta must have an \'id\' column, id,
                 whose values corresponding with sample names")
        meta <- as.data.frame(meta)
        factor_col <- purrr::map_lgl(meta, is.factor)
        if (any(factor_col)) warning("converting factors in meta into strings")
        for (j in which(factor_col)) meta[[j]] <- as.character(meta[[j]])
    } else {
        meta <- data.frame(id=sample_id)
    }
    # insert class if there is one
    if (!identical(class, "class")) {
        if (length(class)==length(sample_id)) {
            if ("class"%in%colnames(meta)) {
                meta$class_ <- class
            } else {
                meta$class <- class
            }
        }
    }

    ## feature: count (sample x cell population) -----------
    message("preparing edge lists: mapping out cell population relations")

    keepinds <- apply(mc, 2, function(x) any(x>0))
    keepinds <- apply(mc, 2, function(x) any(x>0))
    mc <- mc[,keepinds,drop=FALSE]
    colnames(mc) <- phen[keepinds]
    rownames(mc) <- meta$id
    mc <- Matrix::Matrix(mc, sparse=TRUE) # rm all 0 cols
    if (!is.null(phenocode)) phenocode <- phenocode[keepinds]

    ## make meta for cell populations FINAL -----------------
    meta_cell <- get_phen_meta(phen[keepinds],phenocode)
    if (any(grepl("_",meta_cell$phenotype))) {
        meta_cell$phenotype_ <- meta_cell$phenotype
        meta_cell$phenotype <- gsub("_","", meta_cell$phenotype)
        colnames(mc) <- meta_cell$phenotype
    }
    # get rid of lower layers if requested
    if (!is.null(max_layer)) {
        phen_id <- meta_cell$phenolayer <= max_layer
        meta_cell <- meta_cell[phen_id,,drop=FALSE]
        mc <- mc[,phen_id,drop=FALSE]
    }


    # make parent list (for each cell popultion, list its parents)
    pccell <- get_phen_list(meta_cell=meta_cell, no_cores=no_cores)
    edf <- pccell$edf
    pchild <- pccell$pchild # not used, returned
    pparen <- pccell$pparen

    # extract cell populations that have parents to calculate
    # expected proportions for; just to be safe
    cells1 <- meta_cell$phenotype[meta_cell$phenolayer==1]
    cells1 <- append("",cells1[cells1%in%names(pparen)])
    cells <- meta_cell$phenotype[meta_cell$phenolayer>1]
    cells <- cells[cells%in%names(pparen)]
    cells_ <- append(cells1,cells)

    meta_cell <- meta_cell[match(cells_,meta_cell$phenotype),]
    rooti <- which(meta_cell$phenolayer==0)

    mc <- mc[,match(cells_, colnames(mc)),drop=FALSE] # final cpops

    # trim/order list of parents and children
    pparen_ <- pparen[names(pparen)%in%cells_]
    pparen_[cells] <- purrr::map(pparen_[cells], function(x) {
        a <- x[x%in%cells_]
        if (length(a)==0) return(NULL)
        a
    })
    pparen_ <- purrr::compact(pparen_)

    pchild_ <- pchild[names(pchild)%in%cells_]
    pchild_ <- purrr::map(pchild_, function(x) {
        a <- x
        for (i in seq_len(length(a)))
            a[[i]] <- x[[i]][x[[i]]%in%cells_]
        if (all(purrr::map_int(a, length)==0)) return(NULL)
        a
    })
    pchild_ <- purrr::compact(pchild_)

    edf <- edf[edf$to%in%cells_ & edf$from%in%cells_,,drop=FALSE]

    time_output(start1)
    start1 <- Sys.time()
    message("calculating features")


    ## list of outputs
    gr <- list(e=edf, v=meta_cell)
    gr <- set_layout_graph(gr, layout_fun) # layout cell hierarchy

    if (is.null(meta)) meta <- data.frame(id=sample_id)

    desc <- summary_table(mc,"count")
    fg <- methods::new(
        "flowGraph",
        feat=list(node=list(count=mc), edge=list()),
        feat_desc=list(node=desc),
        markers=markers,
        graph=gr, edge_list=list(child=pchild_,parent=pparen_),
        meta=meta, plot_layout=as.character(substitute(layout_fun)),
        etc=list(cumsumpos=FALSE, class_mean_normalized=FALSE)
    )
    time_output(start1)
    start1 <- Sys.time()

    ## features ----------------------------
    cumsumpos <- cumsumpos & any(grepl("3",meta_cell$phenocode))
    if (cumsumpos)
        fg <- fg_feat_cumsum(fg, no_cores=no_cores)

    if (prop) { # included in specenr
        fg <- fg_feat_node_prop(fg)
        fg <- fg_feat_edge_prop(fg, no_cores=no_cores)
    }

    if (specenr) {
        fg <- fg_feat_node_specenr(fg, no_cores=no_cores)
    }

    try({
        if (calculate_summary &
            ifelse(length(class)==1, class%in%colnames(meta), TRUE) &
            all(node_features%in%c(names(fg_get_feature_all(fg)$node), "NONE")) &
            all(edge_features%in%c(names(fg_get_feature_all(fg)$edge), "NONE"))) {

            time_output(start1)
            start1 <- Sys.time()
            message("calculating summary statistics")

            fg <- fg_summary(
                fg, no_cores=no_cores,
                class=ifelse(length(class)==1, class, "class"),
                label1=label1, label2=label2,
                node_features=node_features,
                edge_features=edge_features,
                overwrite=FALSE,
                test_custom=test_custom,
                test_name=test_name,
                diminish=diminish)
        }
    })
    time_output(start1)

    saved <- FALSE
    if (!is.null(path))
        tryCatch({
            saved <- fg_save(fg, path, save_plots=FALSE)
        }, error=function(e) {
            message("flowGraph object not saved; check path and try again with fg_save!")
        })

    if (calculate_summary & save_plots & saved)
        fg_save_plots(fg, plot_path=paste0(path, "/plots"))

    time_output(start, "total time used")
    return(fg)
}

---
title: "_**flowGraph**_: Identifying differential cell populations in flow cytometry data accounting for marker frequency"
shorttitle: "flowGraph"
author: Alice Yue
email: aya43@sfu.ca
package: flowGraph
date: September 2020
output: 
    BiocStyle::html_document
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{flowGraph}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```


[![DOI](https://zenodo.org/badge/DOI/10.1101/837765.svg)](https://doi.org/10.1101/837765)

[Summary blog post](https://aya49.github.io/2020/09/30/flowGraph/)

flowGraph is an R package used to identify candidate biomarkers for disease diagnosis in flow cytometry data. It does so by identifying driver cell populations whose abundance changes significantly and independently given a disease.

flowGraph takes cell counts as input and outputs SpecEnr values for each cell population within a flow cytometry sample, based on their expected proportion. SpecEnr accounts for dependencies between cell populations such that we can use it to flag only cell populations whose abundance change is incurred wholly or in part because of its association with a sample class (e.g. healthy vs sick).

# Installing the package

**flowGraph** can be installed via Bioconductor.

You can also install the development version directly from Github using BiocManager:

```{r, eval=FALSE, label=library1, warning=FALSE, message=FALSE, echo=TRUE, fig=FALSE, results='hide'}
if (!require("BiocManager")) install.packages('BiocManager') 
BiocManager::install("aya49/flowGraph")
```

# Citation

The theory, proof, and algorithm behind the SpecEnr statistic used in the 
flowGraph package can be found in the following [paper](https://www.biorxiv.org/content/10.1101/837765v3.abstract). 
Please consider citing if you found it helpful.

bibtex:
```
@article{yue2019identifying,
  title={Identifying differential cell populations in flow cytometry data accounting for marker frequency},
  author={Yue, Alice and Chauve, Cedric and Libbrecht, Maxwell and Brinkman, Ryan},
  journal={BioRxiv},
  pages={837765},
  year={2019},
  publisher={Cold Spring Harbor Laboratory}
}
```

The scripts and data from the paper can be downloaded ons [Zenodo](https://zenodo.org/record/3991166).

# Introduction

The main goal of _**flowGraph**_ is to help users interpret which cell populations 
are differential between samples of different etiologies. To understand how 
flowGraph defines cell populations, we introduce the cell hierarchy model below.
This model will serve as the basis for the analysis done in the 
_**flowGraph**_ package.

```{r, label=library2, warning=FALSE, message=FALSE, echo=TRUE, fig=FALSE, message=FALSE}
library(flowGraph)
```

## Cell population naming convention

A **cell hierarchy** is a directed acyclic graph that maps out all possible 
cell populations. In this graph, each cell population is a node while relations 
between cell populations are represented by edges. Each node is labelled by 
whether or not its corresponding cell population contains cells that does/not 
contain a subset of markers. For example, if the markers used in an experiment 
is $A$, $B$, $C$, $D$, then $A^-$ and $A^+$ would represent the cell population 
with cells that does/not contain marker $A$ (e.g. Figure \ref{fig:ch}).

If you are using threshold gates, $A^+$/$A^-$ is the count of cells with a 
fluorescent intensity (FI) value higher/lower than the threshold. If you have 
multiple thresholds, for example 3 thresholds, for $A$, then you would have
$A^-$, $A^+$, $A^++$, $A^+++$ with thresholds in between.

If you are using polygon gates, then $A$ would be the name of your polygon gate
on any two dimension and $A^-$ would be the cells inside the gate and $A^+$ 
would be the cells outside the gate.

We also accept cell population names where marker conditions are separated by an
underscore e.g. $A^+\_B^+$.


```{r, message=FALSE}
no_cores <- 1
data(fg_data_pos2)

meta_cell <- get_phen_meta(colnames(fg_data_pos2$count))
suppressWarnings({ pccell <- flowGraph:::get_phen_list(meta_cell, no_cores) })
gr <- set_layout_graph(list(e=pccell$edf, v=meta_cell)) # layout cell hierarchy

gr <- ggdf(gr)
gr$v$colour <- ifelse(!grepl("[-]",gr$v$phenotype), 1,0)
                     # "nodes with only marker conditions", "other nodes")
gr$v$label <- gr$v$phenotype
gr$v$v_ind <- gr$v$label_ind <- TRUE
gr$e$e_ind <- !grepl("[-]",gr$e$from) & !grepl("[-]",gr$e$to)
```
```{r, fig.wide=TRUE}
knitr::opts_template$set(figure1=list(fig.height=9, fig.width=9))
plot_gr(gr, main="Example cell hierarchy with markers A, B, C, D")
```


Traditionally, cell populations are quantified by their proportion, or their 
cell count over the total cell count. A downside to this analysis is that if 
one cell population is differential, then all cell populations that contain 
cells that are also in that differential cell population would also be flagged 
as significantly differential. By incorporating information on relations 
between cell populations, _**flowGraph**_ uses the notion of 
expected proportions and **SpecEnr** or specific enrichment as a replacement 
for proportions to isolate only differential cell populations.


# Workflow: a simple example

This section will run through a simple example of how _**flowGraph**_ can be 
used to analyze a set of flow cytometry samples.

Typically, one would input a sample x cell population matrix and 
a directory where one wants to save all of the flowGraph resuts and plots:

```{r, eval=FALSE}
no_cores <- 1 # number of cores to parallelize on.
data(fg_data_pos2)
fg <- flowGraph(
    fg_data_pos2$count, # sample x cell population matrix
    meta=fg_data_pos2$meta, # a data frame with each sample's meta data
    path="flowGraph_example", # a directory for flowGraph to output results to
    no_cores=no_cores) # number of cores to use, typically 1 for small data
```

The flowGraph output can be loaded into R from the specified directory as 
a flowGraph object `fg`. 

```{r, eval=FALSE}
fg <- fg_load("flowGraph_example")
```

This flowGraph object can be further analyzed and
modified by using methods described in the following sections.


## Data sets contained in the package

The package contains two data sets:
- `fg_data_fca` [@aghaeepour2013critical]: a real data set comparing healthy 
and AML (acute myeloid leukemia) positive subjects; it is known that there is 
an outlier sample and that cell population node $CD34^+$ increases in production 
in the AML positive subjects' samples.
- `fg_data_pos2`: a positive control data set where cell population node 
   $A^{+}B^{+}C^+$ is artificially increased by 50\%.
- `fg_data_pos30`: a positive control data set where cell population node 
   $A^{+...}B^{+...}C^+$ is artificially increased by 50\%. Note this data set 
   contains multiple thresholds for markers $A$ and $B$.


Both of these are lists containing elements:
- `count`: a sample x cell population numeric matrix containing cell count data.
- `meta`: a data frame containing meta data on the samples in `count`. 
   The sample names in the `id` corresponds with the row names in `count`.

```{r,label=Load_Data, fig=FALSE, message=FALSE}
# data(fg_data_fca)
data(fg_data_pos2)
# data(fg_data_pos30)

# ?fg_data_fca
# ?fg_data_pos2
# ?fg_data_pos30
```


## Initializing a flowGraph object

To contain information regarding cell population quantification and the 
cell hierarchy structure in one place, we use a **flowGraph** object to conduct 
analysis. To initialize a flowGraph object, the user can give as input, 
a numeric vector or matrix. 
For examples on how all of these options 
can be used, see `?flowGraph`. 


For our example, we will directly use a numeric matrix as provided by our 
`fg_data_pos2` data set. By default, `flowGraph` will calculate all of the 
proportion `prop`, and SpecEnr `specenr`.


```{r, message=FALSE}
# no_cores <- 1 # number of cores to parallelize on.
data(fg_data_pos2)
fg <- flowGraph(fg_data_pos2$count, meta=fg_data_pos2$meta, no_cores=no_cores)

```

By default, `calculate_summary` is set to `TRUE` so that a default set of
summary statistics will be calculated for the SpecEnr
node feature and `prop` edge feature. Note that if the user decides to do this,
the `class` in `summary_pars`  must be the column name in data frame `meta` 
with the class labels. This is set to `"class"` by default.
A class in this context is, for example, an experiment or control sample.
If the user does not wish for this to be calculated during construction
of the flowGraph object, the user can set `summary_pars` to `NULL` in the
`flowGraph` function. Note that`summary_pars` must be specified if the fast
version of flowGraph is used.

`meta` can be given/modified at a later time. 
Just make sure the meta data is a data frame
containing a `id` column where its values correspond to the row names in 
the flowGraph objects' feature matrices.

```{r}
meta <- fg_get_meta(fg)
head(meta)

mcount <- fg_get_feature(fg, "node", "count")
head(rownames(mcount))
```


### Input format

The input to flowGraph is a sample x cell population phenotype matrix containing
the cell count of each cell population for each sample.

The row names of the input matrix should be sample ID, otherwise, 
flowGraph will create sample IDs for you. 

The column names of the input matrix should be cell population phenotype names
that follow cell population naming conventions.
Markers/measurements must not contain underscores, dashes, or pluses 
(`_`, `+`, `-`). If underscores were
used to separate marker/measurement conditions e.g. ssc+_cd45-, the underscores
will be removed from the column names of the features matrices but will be
saved in `fg_get_meta(fg)$phenotype_`.

To calculate the SpecEnr of a cell population (e.g. A+B+C+), 
flowGraph requires that all of its parent (A+B+, B+C+, A+C+) 
and grandparent cell populations (A+, B+, C+) are available.



## Retrieving results from a flowGraph object

The default summary statistics is calculated using the Wilcoxan signed-rank test and adjusted
using the `byLayer` adjustment method. This adjustment method is a family-wise method that 
multiplies the p-value for each cell population by the number of nodes in 
its layer and the total number of layers in the cell hierarchy on which there 
exists a cell population node.

Below, we retrieve this summary statistic and list out the cell
populations with the most significant p-values as per below.

```{r}
# get feature descriptions
fg_get_summary_desc(fg)
# get a summary statistic
fg_sum <- fg_get_summary(fg, type="node", summary_meta=list(
    node_feature="SpecEnr", 
    test_name="t_diminish", 
    class="class", 
    labels=c("exp","control"))
)
# fg_sum <- fg_get_summary(fg, type="node", index=1) # same as above

# list most significant cell populations
p <- fg_sum$values # p values
head(sort(p),30)
```

To make changes to the flowGraph object, see functions that start with `fg_`.

Once we have made all the changes necessary, we can save the flowGraph object to
a folder. Inside the folder, all the feature values and summary statistics
are saved as csv files. Plots for each summary statistic can also optionally 
be saved to this folder.

The same folder directory is used to load the flowGraph object when needed again.

```{r, eval=FALSE}
fg_save(fg, "path/to/user/specified/folder/directory") # save flowGraph object
fg <- fg_load("path/to/user/specified/folder/directory") # load flowGraph object
```

See other flowGraph object initialization options in the Appendix 2.

The following sections will go over additional options for summary statistics,
and result interpretation.


# Accessing and modifying data in a flowGraph object


## Flow cytometry sample meta data

The flowGraph object initially contains meta data on the samples and 
cell population nodes (phenotypes). The most basic way of understanding what 
is inside a flowGraph object is by using `show`. This shows a description of 
the flowGraph object and returns a list of data frames containing information 
on the node and edge features and the summary statitics performed on them 
shortly in this vignette.

```{r}
show(fg)
```

One can obtain meta data on samples and cell populations as follows. 
Note that information on cell populations is given to the user in the form of a 
`graph` or a list contianing data frames `v` and `e`. The former represents the 
nodes or the cell populatins, and the latter represent edges or the relation 
between cell population --- note that edges always point from parent to 
child cell populations indicative of whether or not a cell population is a 
sub-population of another.

```{r}
# get sample meta data
head(fg_get_meta(fg))

# modify sample meta data
meta_new <- fg_get_meta(fg)
meta_new$id[1] <- "new_sample_id1"
fg <- fg_replace_meta(fg, meta_new)
```
```{r}
# get cell population meta data
gr <- fg_get_graph(fg)
head(gr$v)
head(gr$e)
```


## Feature values

The user can also extract or modify the features inside a flowGraph object. 
For adding new features, unless needed, we recommend users stick with the 
default feature generation methods that starts with `fg_feat_`.

```{r}
# get feature descriptions
fg_get_feature_desc(fg)
# get count node feature
mc <- fg_get_feature(fg, type="node", feature="count")
dim(mc)
```
```{r}
# add a new feature; 
# input matrix must contain the same row and column names as existing features;
# we recommend users stick with default feature generation methods 
# that start with fg_feat_
fg <- fg_add_feature(fg, type="node", feature="count_copy", m=mc)
fg_get_feature_desc(fg)
```
```{r}
# remove a feature; note, the count node feature cannot be removed
fg <- fg_rm_feature(fg, type="node", feature="count_copy")
fg_get_feature_desc(fg)
```


## Feature summary statistics

Once a flowGraph object is created, the user can calculate summary statistics 
for any of the features it contains.

We recommend the user use the `fg_summary` function. Its default 
summary statistic is the significance T-test along with a `byLayer` 
p-value adjustment method. The user can specify other summary statistics or 
adjustment methods by providing the name of the method or a function to 
parameters `test_custom` or `adjust_custom`.

```{r}
fg_get_summary_desc(fg)

# calculate summary statistic
fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                 node_features="count", edge_features="NONE",
                 overwrite=FALSE, test_name="t", diminish=FALSE)
fg_get_summary_desc(fg)
```
```{r}
# get a summary statistic
fg_sum1 <- fg_get_summary(fg, type="node",  summary_meta=list(
    feature="count", test_name="t", 
    class="class", label1="control", label2="exp"))
names(fg_sum1)
```
```{r}
# remove a summary statistic
fg <- fg_rm_summary(fg, type="node", summary_meta=list(
    feature="count", test_name="t", 
    class="class", label1="control", label2="exp"))
fg_get_summary_desc(fg)
```
```{r}
# add a new feature summary; 
# input list must contain a 'values', 'id1', and 'id2' containing summary 
# statistic values and the sample id's compared;
# we recommend users stick with default feature generation method fg_summary
fg <- fg_add_summary(fg, type="node",  summary_meta=list(
    feature="SpecEnr", test_name="t_copy",
    class="class", label1="control", label2="exp"), p=fg_sum1)
fg_get_summary_desc(fg)
```


A summary static statistic, once obtained, is a list containing:
- `values`: a vector of p-values for each node or edge.
- `id1` and `id2`: a vector of sample id's that were compared.
- `test_fun` and `adjust_fun`: the functions used to test and adjust the 
   summary statistic.
- `m1` and `m2`: a vector that summarizes one of the sets of samples compared. 
   These are not contained inside a flowGraph object but can be calculated on 
   the spot when retreiving a summary using `fg_get_summary` by setting 
   parameter `summary_fun` to a matrix function (default: `colSums`) and not 
   `NULL`. This usually does not need to be adjusted.


# Plotting and visualizing results

All of the calculated summaries can be visualized in the form of a 
cell hierarchy plot using function `fg_plot`. The plot can also be saved as a 
PNG file if the path to this PNG file is provided as a string for its 
`path` parameter. Here, we do not save the plot, but we plot the returned 
`graph` list `gr` given by `fg_plot` that contains all the plotting columns 
using the `plot_gr` function.

```{r}
# plotting functions default to plotting node feature SpecEnr 
# labelled with mean expected/proportion (maximum 30 labels are shown for clarity)
# and only significant nodes based on the wilcox_byLayer_diminish summary statistic
# are shown.
# gr <- fg_plot(fg, p_thres=.01, show_bgedges=TRUE, # show background edges
#               node_feature="SpecEnr", edge_feature="prop",
#               test_name="t_diminish", label_max=30)
gr <- fg_plot(fg, index=1, p_thres=.01, show_bgedges=TRUE)
# plot_gr(gr)
```

While through `plot_gr`, `fg_plot` uses the `ggplot2` package to create static
plot, the user can 
also choose to plot `gr` as an interactive plot by setting the `interactive` 
parameter to `TRUE` using the `ggiraph` package.

```{r}
# interactive version in beta
plot_gr(gr, interactive=TRUE)
```



# Appendix

## Appendix 1: OTher useful plots

Summary statistics can also be analyzed using other plots.

### QQ plot
For example, the user can plot a static/interactive 
`ggiraph` QQ plot of a chosen summary statistic. This plots the p-values against
a uniform distribution.

```{r, message=FALSE}
data(fg_data_pos2)
fg1 <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class, 
                 no_cores=no_cores)
```
```{r}
fg_get_summary_desc(fg)

fg_plot_qq(fg, type="node", index=1)
fg_plot_qq(fg, type="node", index=1, logged=TRUE)

# interactive version
fg_plot_qq(fg, type="node", index=1, interactive=TRUE)
```

### Boxplot
To understand how each p-value was obtained, the user can also plot the 
distribution of values as boxplots for a specific feature between features of different
class labels.

```{r}
fg_plot_box(fg, type="node", summary_meta=NULL, index=1, node_edge="A+")
```

### Logged p-value vs feature difference
Another useful plot is to compare the p-value and the difference between the
mean of a feature value between samples of different classes. This should
look like a volcano plot.

```{r}
fg_plot_pVSdiff(fg, type="node", summary_meta=NULL, index=1)

# interactive version
fg_plot_pVSdiff(fg, type="node", summary_meta=NULL, index=1, interactive=TRUE)
```

### Customizing the cell hierarchy plot

The user can also manually specify how the cell hierarchy plot should look. The columns 
needed for plotting in `plot_gr` can be attached onto the `graph` slot of the 
`fg` the `flowGraph` object using the `ggdf` function. For more information on 
these columns, see `?ggdf`.

```{r}
gr <- fg_get_graph(fg)
gr <- ggdf(gr)

gr$v$colour <- ifelse(!grepl("[-]",gr$v$phenotype), 1, 0)
                     # "nodes with only marker conditions", "other nodes")
gr$v$label <- gr$v$phenotype
gr$v$v_ind <- gr$v$label_ind <- TRUE
gr$e$e_ind <- !grepl("[-]",gr$e$from) & !grepl("[-]",gr$e$to)

plot_gr(gr, main="Example cell hierarchy with markers A, B, C, D")
```


## Appendix 2: flowGraphSubset, a fast version of the flowGraph constructor

If the user is only interested in one set of class labels for a set of samples,
they can choose to use `flowGraphSubset`, a faster version of the default constructor
`flowGraph`. It is fast because the edge list, proportion, expected proportion, 
and SpecEnr features are only calculated for cell populations who are in the
0'th and 1st layer, or have a significant parent population. The assumption here
is that cell populations who are significantly differentially abundant 
must also have at least one significantly differentially abundant parent 
population, which is true for almost all cases.

However, if the user wants to test different sets of sample class labels on the 
same set of samples, we recommend using the default `flowGraph` constructor
as it calculates SpecEnr for all cell populations. Since SpecEnr only has to be
calculated once, the user can apply multiple statistically significance tests
and ask questions about different class sets on the same SpecEnr values.

So in summary, ONLY USE THIS OVER flowGraph IF: 1) your data set has more than
10,000 cell populations and you want to speed up your calculation time AND
2) you only have one set of classes you want to test on the 
SAME SET OF SAMPLES (e.g. control vs experiment).

The parameters for `flowGraphSubset` is a bit different than those in `flowGraph`.
It is currently in beta, so we recommend reading the manual for it carefully.

```{r, evaluate=FALSE}
data(fg_data_pos2)
fg <- flowGraphSubset(fg_data_pos2$count, meta=fg_data_pos2$meta, no_cores=no_cores,
                 summary_pars=flowGraphSubset_summary_pars(),
                 summary_adjust=flowGraphSubset_summary_adjust())
```




# System information

The following is an output of `sessionInfo()` on the system on which this 
document was compiled.

```{r}
sessionInfo()
```


# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_merge_samples}
\alias{fg_merge_samples}
\title{Merges the samples from two flowGraph objects.}
\usage{
fg_merge_samples(fg1, fg2)
}
\arguments{
\item{fg1}{flowGraph object.}

\item{fg2}{flowGraph object.}
}
\value{
flowGraph object.
}
\description{
Merges the samples from two flowGraph objects together;
 we recommend removing all summary statistics from the new flowGraph object
 as those won't be adjusted: \code{\link[flowGraph]{fg_clear_summary}}.
}
\details{
Appends the samples from \code{fg2} onto those in \code{fg1}.
 This function requires that the two flowGraph objects must have the
 same phenotypes. Therefore, we recommend users to use,
 instead, \code{\link[flowGraph]{fg_merge}}.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg1 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[1:5])
 fg2 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[4:7])
 fg <- fg_merge_samples(fg1, fg2)
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_merge}}
 \code{\link[flowGraph]{fg_extract_samples}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{flowGraphSubset}
\alias{flowGraphSubset}
\title{flowGraph object constructor.}
\usage{
flowGraphSubset(
  input_,
  meta = NULL,
  class = "class",
  no_cores = 1,
  markers = NULL,
  layout_fun = "layout.reingold.tilford",
  max_layer = NULL,
  cumsumpos = FALSE,
  path = NULL,
  summary_pars = flowGraphSubset_summary_pars(),
  summary_adjust = flowGraphSubset_summary_adjust(),
  save_plots = TRUE
)
}
\arguments{
\item{input_}{a numeric matrix of the cell counts;
its column/names must be the phenotype names and its rownames
must be sample ID's.}

\item{meta}{A data frame with meta data for each \code{Phenotypes} or sample;
One of its column names should be "id" whose values correspond to
the name of each \code{Phenotypes} object. We also recommend for it to have
a column named "class" where one of its unique values is "control".}

\item{class}{A string corresponding to the column name or index
of \code{meta} whose values represent
the class label of each sample, Default: 'class'}

\item{no_cores}{An integer indicating how many cores to parallelize on,
Default: 1}

\item{markers}{A string vector of marker names used in \code{input_},
Default: NULL}

\item{layout_fun}{A string of a function from the \code{igraph} package that
indicates what layout should be used if a cell hierarchy is to be ploted;
all such functions have prefix \code{layout_}. This is defaulted to
e.g. \code{layout_fun="layout.reingold.tilford"}.}

\item{max_layer}{And integer indicating the maximum layer in the cell
hierarchy to analyze; set to `NULL` to analyze all layers.}

\item{cumsumpos}{A logical variable indicating whether
or not to cumulate cell counts;
this applies only when \code{partitionsPerMarker > 3} and will convert
e.g. the count of A+ or A++ into the sum of the counts of
A+, A++, A+++, ..., or A++, A+++, ... , Default: FALSE}

\item{path}{A string indicating the folder path to where the flowGraph
object should save its elements, Default = NULL (don't save).}

\item{summary_pars}{A list containing parameters for calculating the
 statistical significance summary significance that will determine whether
 to trim out phenotypes for this fast version of flowGraph. The lists'
 elements are:
\itemize{
  \item{\code{node_feature}: "SpecEnr"; this is the feature we will be
   testing, don't change this.}
  \item{\code{edge_feature}: "NONE"; this unneeded for now.}
  \item{\code{test_name}: "t_diminish"; this unneeded for now.}
  \item{\code{test_custom}: "t"; a string or a function indicating the
   statistical test desires. These tests can be
   \code{c("t", "wilcox","ks","var","chisq")} corresponding to functions
   \code{\link[stats]{t.test}},\code{\link[stats]{wilcox.test}},
   \code{\link[stats]{ks.test}},\code{\link[stats]{var.test}},
   \code{\link[stats]{chisq.test}}}
  \item{\code{diminish}: TRUE; whether or not to continue testing
   phenotypes whos parent phenotypes are all insignificant.}
  \item{\code{class}: "class"; the column name in \code{meta} that contains
   class labels you want to test.}
  \item{\code{labels}: c("aml", "control") for the flowcap data set;
  SET THIS!! to the class labels you want to test using \code{test_custom}.}
}}

\item{summary_adjust}{A list of parameters on how to adjust the p-values;
 this also affects which phenotypes are tested. The elements in the list are:
\itemize{
  \item{\code{adjust_custom}: "byLayer"; this is a string (corresponding to
   an option in \code{\link[stats]{p.adjust}}) or a function used to
   adjust p-values.}
  \item{\code{btwn_test_custom}: "t"; see \code{test_custom} in
   \code{summary_pars}; this statistical significance test is used in
   the filters.}
  \item{\code{adjust0_lim}: see \code{fg_get_summary}.}
  \item{\code{filter_adjust0}: see \code{fg_get_summary}.}
  \item{\code{filter_es}: see \code{fg_get_summary}.}
  \item{\code{filter_btwn_tpthres}: see \code{fg_get_summary}.}
  \item{\code{filter_btwn_es}: see \code{fg_get_summary}.}
}}

\item{save_plots}{A logical indicating whether or not to save plots.}
}
\value{
flowGraph object
}
\description{
Initializes a \code{flowGraph} object given the cell counts
 for one or more flow cytometry sample(s).
 The flowGraph object returned holds meta data
 for each sample, each cell population node,
 edges representing how each cell population node relate to one another,
 and features for these nodes and edges.
}
\details{
All node and edge features are trimmed such that only the
 significant phenotypes are left; the original input is stored in the slot
 \code{etc$original_count} of the returned flowGraph object.
}
\examples{
\dontrun{
if(interactive()){
  data(fg_data_pos2)
  fg <- flowGraph(fg_data_pos2$count, meta=fg_data_pos2$meta, no_cores=1)
 }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_helpers.R
\name{mean_diff}
\alias{mean_diff}
\title{Normalizes matrix values by class.}
\usage{
mean_diff(m0, classes)
}
\arguments{
\item{m0}{A numeric matrix.}

\item{classes}{A vector whose length is equal to the number of
rows in the given matrix.}
}
\value{
A numeric matrix whose dimensions equate to that of the input
 and whose values are normalized per class.
}
\description{
Used only in the \code{\link[flowGraph]{fg_feat_mean_class}}
 function; for each class in the \code{classes} vector,
 \code{meandiff} takes the column mean
 of the rows in the given matrix associated with that class;
 it then takes the difference point by point between these means and
 the original rows for that class.
}
\examples{

 classes <- append(rep('apples',4), rep('oranges',3))
 m0 <- matrix(rnorm(35), nrow=7)
 m <- flowGraph:::mean_diff(m0, classes)

}
\seealso{
\code{\link[flowGraph]{fg_feat_mean_class}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_helpers.R
\name{summary_table}
\alias{summary_table}
\title{Summarizes a numeric matrix.}
\usage{
summary_table(m, feat_type = "")
}
\arguments{
\item{m}{A numeric matrix.}

\item{feat_type}{Name of the matrix \code{m}.}
}
\value{
A data frame containing one row summarizing \code{m};
 see \code{\link[flowGraph]{fg_get_feature_desc}}.
}
\description{
Summarizes a numeric matrix.
}
\examples{

 summary_table(matrix(rnorm(12),nrow=3), feat_type='random')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_add_feature}
\alias{fg_add_feature}
\title{Adds a feature.}
\usage{
fg_add_feature(
  fg,
  type = "node",
  feature,
  m = NULL,
  feat_fun = NULL,
  overwrite = FALSE,
  ...
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string specifying the type of the feature being
added i.e. 'node' or 'edge'.}

\item{feature}{A string indicating the unique name of the feature added.}

\item{m}{A numeric matrix with feature values; it should contain the
same sample id's on row names as in \code{fg_get_meta(fg)$id}
and node or edge names
as column names (i.e. if \code{m} is a node feature, it would have the same
column names as those in \code{fg_get_graph(fg)$v$phenotype};
if it is an edge
feature, its column names should be the same as
\code{paste0(fg_get_graph(fg)$e$from, '_', fg_get_graph(fg)$e$to)}).}

\item{feat_fun}{A function that ouputs a feature matrix as in \code{m} given
\code{fg} and other optional parameters.}

\item{overwrite}{A logical variable indicating whether or not the function
should replace the existing feature with the same name if
one is already in \code{fg}.}

\item{...}{Other parameters that would be used as input into \code{feat_fun}.}
}
\value{
flowGraph object.
}
\description{
Adds a feature created using \code{feat_fun} from \code{fg} OR
 \code{m} into a given flowGraph object. Only use this function if
 you cannot generate the desired features using the existing flowGraph
 functions starting with \code{fg_feat_<feature name>}.
}
\details{
\code{fg_add_feature} adds the given new feature matrix to the
 given flowGraph object \code{fg} updating slots
 \code{feat} and \code{feat_desc}.
 See \code{\link[flowGraph]{flowGraph-class}}
 slot \code{feat} and \code{feat_desc} for what should be in these slots.
 We do not recommend users to directly use this method unless there is
 a clear understanding on how the row and column names should be specified.
 Instead, we recommend users to use the functions listed in the "See also"
 sections prefixed with "fg_feat_".
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 fg_get_feature_desc(fg)

 fg <- fg_add_feature(fg, type="node", feature="count_copy",
                      m=fg_data_pos30$count)
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_feat_node_prop}}
 \code{\link[flowGraph]{fg_feat_node_specenr}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_flowgraph_plots.R
\name{plot_gr}
\alias{plot_gr}
\title{Plots a cell hierarchy.}
\usage{
plot_gr(
  gr,
  main = NULL,
  show_bgedges = TRUE,
  colour_palette = NULL,
  label_coloured = TRUE,
  shiny_plot = FALSE,
  interactive = FALSE,
  visNet_plot = TRUE,
  colour_edges = FALSE,
  ...
)
}
\arguments{
\item{gr}{A list containing data frames \code{e} and \code{v}.}

\item{main}{A string containing the plot title. If this is set to NULL, the
function will look for a plot title in the \code{main} slot of \code{gr};
otherwise, this defaults to "".}

\item{show_bgedges}{A logical variable indicating whether or not
edges not specified for plotting should be plotted as light grey
in the background. If this is \code{NULL}, the function will look for a
\code{show_bgedges} in the \code{show_bgedges} slot of \code{gr};
otherwise, this defaults to \code{TRUE}.}

\item{colour_palette}{A colour palette e.g. the default palette if the user
sets this to \code{NULL} is \code{c('blue','cyan','yellow','red')}.}

\item{label_coloured}{A logical indicating whether to colour the node
labels using the same colours as the nodes in the non-interactive plot.}

\item{shiny_plot}{A logical indicating whether this plot is made for shiny;
users don't need to change this.}

\item{interactive}{A logical variable indicating whether the plot should be
an interactive plot; see package \code{ggiraph}.}

\item{visNet_plot}{A logical variable indicating if an interactive plot is
chosen, if function should output a visNetwork plot; if set to \code{FALSE},
ggplot's girafe will be used instead.}

\item{colour_edges}{A logical variable indicating whether to colour edges if
plotting a node feature summary.}

\item{...}{Other parameters for \code{ggplot} if \code{interactive}
is set to \code{FALSE}; other parameters for \code{plot_ly}
if \code{interactive} is set to \code{TRUE}.}
}
\value{
A \code{ggplot} object if \code{interactive} is set to \code{FALSE};
 a \code{ggiraph} object if \code{interactive} is set to \code{TRUE}.
}
\description{
Plots a cell hierarchy given the output from \code{fg_plot}, a list of nodes and edges.
}
\examples{

 no_cores <- 1
 data(fg_data_pos2)
 fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
                 no_cores=no_cores)

 # fg <- fg_summary(fg, no_cores=no_cores, class="class", control="control",
 #                  overwrite=FALSE, test_name="t_byLayer", diminish=FALSE)

 gr_summary <- fg_plot(
   fg, type="node", p_thres=.05, show_bgedges=TRUE,
   path=NULL) # set path to a full path to save plot as a PNG

 plot_gr(gr_summary, main=gr_summary$main, show_bgedges=TRUE)

 plot_gr(gr_summary, main=gr_summary$main, show_bgedges=TRUE, interactive=TRUE)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_plot}}
 \code{\link[flowGraph]{get_phen_meta}}
 \code{\link[flowGraph]{ggdf}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_get_summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_feature_means}
\alias{fg_get_feature_means}
\title{Retrieves feature summaries.}
\usage{
fg_get_feature_means(
  fg,
  type = c("node", "edge"),
  feature = "count",
  class = NULL,
  label = NULL,
  id = NULL,
  summary_fun = colMeans
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for
'node' or 'edge'.}

\item{feature}{A string indicating feature name the summary was created for;}

\item{class}{A string corresponding to a column name of the \code{meta} slot of
\code{fg} whose values represent the class label of each sample
on which the summary was created to compare or analyze;}

\item{label}{A string indicating a class label.}

\item{id}{A string vector containing the sample id's corresponding to the
\code{id} column of the \code{meta} slot of \code{fg}.}

\item{summary_fun}{A function that takes in a matrix and outputs a
vector the same length as the number of columns this matrix has.}
}
\value{
A list containing two numeric vectors calculated using the
 \code{summary_fun} function on the subset of samples specified by
 sample id's \code{id} OR class label
 \code{label} for class \code{class} from a feature matrix
 specified by \code{type} and \code{feat}.
}
\description{
Retrieves a feature summary (e.g. \code{colMeans}) for samples
 specified by sample id's \code{id} OR class label
 \code{label} for class \code{class} given a feature specified
 by \code{type} and \code{feat}.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)
 fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                  overwrite=FALSE, test_name="t", diminish=FALSE)
 show(fg)
 feat_mean <- fg_get_feature_means(fg, type="node", feature="count",
                                   class="class", label="control")

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_get_summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{fpurrr_map}
\alias{fpurrr_map}
\title{Wrapper for map}
\usage{
fpurrr_map(x, f, no_cores = 1, prll = TRUE, ...)
}
\arguments{
\item{x}{Variable to recurse over; must be indices!}

\item{f}{Function to recurse over.}

\item{no_cores}{Number of cores to use; \code{future} must have
been ran already.}

\item{prll}{If set to FALSE, forces use of purrr::map instead of
furrr::future_map, Default: TRUE}

\item{...}{Other parameters used by \code{f}.}
}
\value{
Unnested named list.
}
\description{
Wrapper for \code{purrr::map} and \code{furrr::future_map}
 to handle parallel-ization
}
\details{
Wrapper for \code{purrr::map} and \code{furrr::future_map} to handle
parallel-ization easily; note that \code{future} must have been ran already
outside of the function and outputs will always be a list.
}
\seealso{
\code{\link[purrr]{map}}
 \code{\link[furrr]{future_map}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_load}
\alias{fg_load}
\title{Load a flowGraph object from a specified folder path.}
\usage{
fg_load(folder_path)
}
\arguments{
\item{folder_path}{A string indicating the folder path to where a flowGraph
object was saved using the \code{fg_save} function.}
}
\value{
flowGraph object
}
\description{
Load a flowGraph object from a specified folder path.
}
\details{
see function \code{fg_save}
}
\examples{

 no_cores <- 1
 data(fg_data_pos2)
 fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
                 no_cores=no_cores)

 fg_save(fg, "tmp")
 fg <- fg_load("tmp")
}
\seealso{
\code{\link[flowGraph]{fg_save}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{get_eprop}
\alias{get_eprop}
\title{Gets edge proportions of a given edge matrix}
\usage{
get_eprop(edf_, ep, mp_, no_cores = 1)
}
\arguments{
\item{edf_}{edge x from&to data frame containing edges and their from and to
cell population phenotypes.}

\item{ep}{sample x edge (parent_child) matrix with edge proportions.}

\item{mp_}{sample x phenotype matrix with proportions.}

\item{no_cores}{Number of cores to use, Default: 1}
}
\value{
\code{ep} with only the specific columns (edges) requested; also updates
\code{ep} globally.
}
\description{
Gets the edge proportions of the edges in edge matrix
 \code{edf_} and updates \code{ep} edge proportion matrix if it didn't
 contain the requested information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_summary_desc}
\alias{fg_get_summary_desc}
\title{Retrieves a feature summary description table.}
\usage{
fg_get_summary_desc(fg)
}
\arguments{
\item{fg}{flowGraph object.}
}
\value{
A data frame where each row contains information
 on a feature summary from \code{fg}:
\itemize{
  \item{\code{type}: feature type (i.e. 'node' or 'edge').}
  \item{\code{feat}: feature name.}
  \item{\code{test_name}: summary name.}
  \item{\code{class}: class or the column name of \code{fg_get_meta(fg)}
   whose values represent the class label of each sample on which
   the summary was created for.}
  \item{\code{label1}: A string from the \code{class} column of the
   \code{meta} slot indicating the label of samples compared.}
  \item{\code{label2}: A string from the \code{class} column of the
   \code{meta} slot indicating the label of samples compared.}
}
}
\description{
Retrieves a feature summary description table for
 a given flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 fg_get_summary_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_graph.R
\name{cell_type_layers}
\alias{cell_type_layers}
\title{Determines the layer on which a phenotype resides.}
\usage{
cell_type_layers(phen)
}
\arguments{
\item{phen}{A string vector of phenotype or cell population name labels.}
}
\value{
A numeric vector with the same length as \code{phen} indicating which
   layer each phenotype resides on.
}
\description{
Determines the layer on which the given phenotypes reside.
}
\details{
Given a vector of phenotypes, returns an equal length vector of
   the number of markers in each phenotype.
}
\examples{

   phen <- c('A+B+C-D++', 'A+B-', '', 'B++D-E+')
   cell_type_layers(phen)

}
\seealso{
\code{\link[flowGraph]{get_phen_list}}
   \code{\link[flowGraph]{get_phen_meta}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_flowgraph_features.R
\name{fg_feat_node_specenr}
\alias{fg_feat_node_specenr}
\title{Generates the SpecEnr node feature.}
\usage{
fg_feat_node_specenr(fg, no_cores = 1, feature = "prop", overwrite = FALSE)
}
\arguments{
\item{fg}{flowGraph object}

\item{no_cores}{An integer indicating how many cores to parallelize on.}

\item{feature}{A string indicating feature name; this is the feature
SpecEnr will be calculated on.}

\item{overwrite}{A logical variable indicating whether to
overwrite the existing SpecEnr node feature if it exists.}
}
\value{
flowGraph object containing the SpecEnr node feature.
}
\description{
Generates the SpecEnr node feature and returns it
 inside the returned flowGraph object.
}
\details{
Given a flowGraph object, \code{fg_feat_node_specenr}
 returns the same
 flowGraph object with an additional \code{SpecEnr} and \code{expect_prop}
 \code{node} feature and its meta data.
 The expected proportions feature is made using the \code{prop} \code{node}
 and \code{edge} features; therefore, the returned flowGraph will also
 contain these two features. For details on how these feature is calculated.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 # SpecEnr is by default calculated based on proportions
 fg <- fg_feat_node_specenr(fg, no_cores=no_cores)

 # SpecEnr can be calculated for other feature values too
 fg <- fg_feat_node_specenr(fg, feature="count")

 show(fg)

}
\references{
\insertRef{yue2019identifying}{flowGraph}
}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_feat_node_prop}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_replace_meta}
\alias{fg_replace_meta}
\title{Replaces sample meta.}
\usage{
fg_replace_meta(fg, meta)
}
\arguments{
\item{fg}{flowGraph object.}

\item{meta}{A data frame containing meta data; see details in
\code{\link[flowGraph]{flowGraph-class}}.}
}
\value{
A flowGraph object with an updated sample meta.
}
\description{
Replaces sample meta in a given flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 head(fg_get_meta(fg))

 new_df <- fg_data_pos30$meta
 new_df$id[1] <- "newID"

 fg <- fg_replace_meta(fg, new_df)
 head(fg_get_meta(fg))

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_meta}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{get_child}
\alias{get_child}
\title{Gets child populations of given cell populations}
\usage{
get_child(parens, pchild, pc_i, ac__, meta_cell__)
}
\arguments{
\item{parens}{Character vector of cell population phenotypes.}

\item{pchild}{Edge list where the name of the list is the cell population
and the vector in each element contains the child cell populations of the
named cell population.}

\item{pc_i}{A cell population x marker matrix where the values are 0/1/2/...
correspondng to marker conditions /-/+/... for possible PARENT populations.}

\item{ac__}{A list where the elements are marker index > "0"/"1"/"2"/... >
a logical vector the same length as the number of cell population phenotypes
indicating whether or not the marker condition exists in them; this is for
the possible CHILD cell populations}

\item{meta_cell__}{data frame with meta data for cell population phenotypes
from the flowGraph object; this is for the possible CHILD cell populations.}
}
\value{
A list containing child populations of \code{parens}; also globally
 updates \code{pchild}.
}
\description{
Gets the child populations of a vector of given cell populations
 \code{parens} and updates \code{pchild} the edge list if edge list doesn't contain
 the requested information.
}
\seealso{
\code{\link[purrr]{map}},\code{\link[purrr]{keep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_data.R
\docType{data}
\name{fg_data_fca}
\alias{fg_data_fca}
\title{fg_data_fca}
\format{
A list containing the following elements
 derived from the flowCAP-II AML data set for cell populations up to layer 3.
\itemize{
  \item{\code{count}: A numeric sample x cell population node
  matrix with cell count values.}
  \item{\code{meta}: A data frame containing meta information on
   samples in \code{count}; it contains columns:}
  \itemize{
      \item{\code{class}: a string indicating whether a sample is
       from a "control" or "aml" subject.}
      \item{\code{id}: a string containing sample id's.}
      \item{\code{train}: a logical variable indicating whether
       a sample is from the train or test set.}
      \item{\code{subject}: a numeric variable containing the
       id of the subject from whom the sample came from.}
      \item{\code{tube}: the tube or panel number;
       all samples in this data set is
       analyzed under the 6th panel.}
  }
}
}
\source{
{
  \insertRef{aghaeepour2013critical}{flowGraph}
}
}
\usage{
fg_data_fca
}
\description{
fg_data_fca
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_helpers.R
\name{loop_ind_f}
\alias{loop_ind_f}
\title{Prepares parallel loop indices.}
\usage{
loop_ind_f(x, n)
}
\arguments{
\item{x}{A vector of loop indices.}

\item{n}{An integer, or the number of vectors to split \code{x} into.}
}
\value{
list of \code{n} vectors with elements from \code{x}.
}
\description{
\code{loop_ind_f} is a helper function that splits
 a vector of loop indices into a list of multiple loop indices
 for use in parallel processes within the flowGraph package.
}
\examples{

 old_loop_inds <- 1:10
 no_cores <- 5

 new_loop_inds <- flowGraph:::loop_ind_f(old_loop_inds, no_cores)
 # future::plan(future::multisession)
 # example_indices <- furrr::future_map(new_loop_inds, function(ii) {
 #     purrr::map(ii, function(i) i )
 # s})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_meta}
\alias{fg_get_meta}
\title{Retrieves sample meta.}
\usage{
fg_get_meta(fg)
}
\arguments{
\item{fg}{flowGraph object.}
}
\value{
A data frame containing sample meta data.
}
\description{
Retrieves sample meta from a given flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 head(fg_get_meta(fg))

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_replace_meta}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_save}
\alias{fg_save}
\title{Saves flowGraph object to a specified path.}
\usage{
fg_save(fg, folder_path = NULL, save_plots = TRUE, paired = FALSE, ...)
}
\arguments{
\item{fg}{flowGraph object to save.}

\item{folder_path}{A string indicating the folder path to where the flowGraph
object should save its elements; if this is the first time the object is
being saved, this folder should be empty or if it is
not yet created, the function will create it. If the object has previously
been saved before and this parameter is set to \code{NULL}, the function
will save the object into the save folder it was previously saved in.}

\item{save_plots}{A logical indicating whether or not to save plots.}

\item{paired}{A logical indicating whether the summary is paired; used in
function \code{fg_plot_box}.}

\item{...}{Other parameters for the \code{fg_save_plots} function.}
}
\value{
TRUE if flowGraph object successfully saved.
}
\description{
Saves flowGraph object to a specified path.
}
\details{
See generated README.md file.
}
\examples{
 no_cores <- 1
 data(fg_data_pos2)
 fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
                 no_cores=no_cores)

 fg_save(fg, "tmp")

}
\seealso{
\code{length},\code{c("nrow", "nrow")},\code{NULL}
 \code{\link[purrr]{map}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_summary}
\alias{fg_get_summary}
\title{Retrieves a summary statistic.}
\usage{
fg_get_summary(
  fg,
  type = "node",
  index = NULL,
  summary_meta = NULL,
  adjust_custom = "byLayer",
  SpecEnr_filt = TRUE,
  summary_fun = colMeans,
  adjust0_lim = c(-0.1, 0.1),
  filter_adjust0 = 1,
  filter_es = 0,
  filter_btwn_tpthres = 0.05,
  filter_btwn_es = 0.5,
  default_p_thres = 1
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for
'node' or 'edge'.}

\item{index}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{index} is an integer indicating the row in
 \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
 summary the user would like to retrieve.}

\item{summary_meta}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{summary_meta} is a list containing
 \code{feat} (feature name), \code{test_name} (summary statistic name),
 \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.}

\item{adjust_custom}{A function or a string indicating the
test adjustment method to use.
 If a string is provided, it should be one of
 \code{c("holm", "hochberg", "hommel",
 "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
 If a function is provided, it should take as input
 a numeric vector and output the
 same vector adjusted.}

\item{SpecEnr_filt}{A logicle indicating whether or not to filter p-values
for SpecEnr.}

\item{summary_fun}{A function that takes in a matrix and outputs a
vector the same length as the number of columns this matrix has.
Set to \code{NULL} to not calculate this summary
(i.e. returned list will not contain \code{m1} and \code{m2}).
See \code{\link[flowGraph]{fg_get_feature_means}}.}

\item{adjust0_lim}{A vector of two numeric values indicating a range around
0, default set to -0.1 and 0.1.}

\item{filter_adjust0}{A numeric variable indicating what percentage of
SpecEnr values compared (minimum) should be not close to 0.
Set to 1 to not conduct filtering. Original p-values stored in
\code{values_original}.}

\item{filter_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant.}

\item{filter_btwn_tpthres}{A numeric variable between 0 and 1 indicating the
unadjusted T-test p-value threshold used to test whether the actual
and expected feature values used to calculate the specified SpecEnr
feature are significantly different for each sample class. Note this only
needs to be specified for SpecEnr features. Combined
with \code{filter_btwn_es}, we conduct three tests to understand if
there is an actual large difference between actual and expected features:
(1,2) T-test of significance between the actual and expected raw feature value
(e.g. proportion) for samples in each of the compared classes, (3) and the
T-test of significance between the differences of actual and
expected feature values of the two classes. If any two of the three tests
come out as insignificant, we set the p-value for the associated node/edge
to 1.}

\item{filter_btwn_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant -- see \code{filter_btwn_tpthres}.}

\item{default_p_thres}{A numeric variable indicating the p-value threshold
user is using. Currently, all nodes/edges not passing the \code{filter}
criterion will be defaulted to 1; if this parameter is set, then all
of these nodes/edges will be set to a minimum of \code{default_p_thres}.}
}
\value{
A list containing elements on feature summary retrieved by the user
 as in the \code{summary} slot of
 \code{\link[flowGraph]{flowGraph-class}}.
 If \code{summary_fun} is not \code{NULL}, this list also includes:
 \itemize{
  \item{\code{m1}: a numeric vector the same length as \code{values};
  this is a summary of the samples compared e.g. mean.}
  \item{\code{m2}: a numeric vector the same length as \code{values};
  this is a summary of the samples compared e.g. mean.}
  \item{\code{cohensd}: a numberic vector indicating cohen's d values
   considering effect size.}
  \item{\code{cohensd_size}: a factor vector interpreting cohen's d values.}
  \item{\code{adjust0}: a numeric vector indicating the percentage of
   samples that have a SpecEnr value in the range of \code{adjust0_lim}
   around 0; if there are two classes of samples being compared, we output
   the smaller percentage between the two classes.}
  \item{\code{btwn}: a data frame containing columns:
   \itemize{
    \item{\code{tpv1}: unadjusted p-value calculated
     between the actual and expected raw feature values of class 1.}
    \item{\code{tpv2}: unadjusted p-value calculated
     between the actual and expected raw feature values of class 2.}
    \item{\code{cd1}: Cohen's D between the actual and expected raw
     feature values of class 1.}
    \item{\code{cd2}: Cohen's D between the actual and expected raw
     feature values of class 2.}
    \item{\code{btp}: unadjusted p-value calculated between the
     difference between actual and expected raw feature of the two classes.}
    \item{\code{bcd}: Cohen's D calculated between the
     difference between actual and expected raw feature of the two classes.}
    \item{\code{btp_}: unadjusted p-value calculated between the
     log ratio between actual and expected raw feature of the two classes.}
    \item{\code{bcd_}: Cohen's D calculated between the
     log ratio between actual and expected raw feature of the two classes.}
   }
  }
 }
}
\description{
Retrieves a summary statistic from a given flowGraph object;
 while \code{fg} is required, the user can choose to input parameters
 \code{summary_meta}, \code{index}, or all of \code{type},
 \code{feat}, \code{test_name}, \code{class}, \code{label1}, and \code{label2}.
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 # set features to NULL to apply summary statistic to all features.
 fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                  overwrite=FALSE, test_name="t", diminish=FALSE,
                  node_features=NULL, edge_features=NULL)
 show(fg)

 feat_summ <- fg_get_summary(fg, type="node", summary_meta=list(
     feature="SpecEnr", test_name="t", class="class",
     label1="control", label2="exp"))

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_means}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_get_feature}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_graph.R
\name{get_phen_meta}
\alias{get_phen_meta}
\title{Genrates phenotype meta data.}
\usage{
get_phen_meta(phen, phenocode = NULL)
}
\arguments{
\item{phen}{A string vector of phenotype or cell population name labels.}

\item{phenocode}{A string vector of phenocodes corresponding
to the phenotypes in \code{phen}.}
}
\value{
A data frame with columns containing meta data on
   cell poulation nodes with columns:
   \itemize{
       \item{\code{phenotype}: cell population node label e.g. "A+B+".}
       \item{\code{phenocode}: a string penocode containing a numeric
        corresponding to the phenotype column e.g. "2200".}
       \item{\code{phenolayer}: a numeric layer on which a cell population
        resides in e.g. 2.}
   }
}
\description{
Generates phenotype meta data given a vector of
   phenotypes and optionally phenocodes.
}
\examples{

   phen <- c('A+B+C-D++', 'A+B-', '', 'B++D-E+')
   phenc <- c('22130','21000','00000','03012')
   get_phen_meta(phen, phenc)

}
\seealso{
\code{\link[flowGraph]{get_phen_list}}
   \code{\link[flowGraph]{cell_type_layers}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_rm_feature}
\alias{fg_rm_feature}
\title{Removes a feature.}
\usage{
fg_rm_feature(fg, type = "node", feature = NULL)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string specifying the type of the feature being
removed i.e. 'node' or 'edge'.}

\item{feature}{A string indicating the unique name of the feature removed;
note we cannot remove the 'node' 'count' feature type.}
}
\value{
flowGraph object with specified feature removed.
}
\description{
Removes a feature from a given flowGraph object.
}
\details{
\code{fg_rm_feature} removes a specified feature matrix from the
 given flowGraph object \code{fg} updating slots
 \code{feat} and \code{feat_desc}.
 See \code{\link[flowGraph]{flowGraph-class}}
 slot \code{feat} and \code{feat_desc} for what should be in these slots.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)
 fg_get_feature_desc(fg)

 fg <- fg_rm_feature(fg, type="node", feature="prop")
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_rm_summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_flowgraph_summary.R
\name{fg_summary}
\alias{fg_summary}
\title{Calculates feature summary statistics.}
\usage{
fg_summary(
  fg,
  no_cores = 1,
  class = "class",
  label1 = NULL,
  label2 = NULL,
  class_labels = NULL,
  node_features = "SpecEnr",
  edge_features = "NONE",
  test_name = "t_diminish",
  diminish = TRUE,
  p_thres = 0.05,
  p_rate = 2,
  test_custom = "t",
  effect_size = TRUE,
  adjust0 = TRUE,
  adjust0_lim = c(-0.1, 0.1),
  btwn = TRUE,
  btwn_test_custom = "t",
  save_functions = FALSE,
  overwrite = FALSE
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{no_cores}{An integer indicating how many cores to parallelize on.}

\item{class}{A string corresponding to the column name or index
of \code{fg_get_meta(fg)} whose values represent
the class label of each sample.}

\item{label1}{A string from the \code{class} column of the
\code{meta} slot indicating one of the labels compared to create
the summary statistic. If you would like to compare all other class
labels against one label, set \code{label2} to \code{NULL}.
If you would like to compare all labels against
all labels, set \code{label1} and \code{label2} to \code{NULL}.}

\item{label2}{A string from the \code{class} column of the
\code{meta} slot indicating one of the labels compared to create
the summary statistic.}

\item{class_labels}{A list of vectors, each containing two strings
represeting labels to compare; this parameter is an alternative to
parameters \code{label1} and \code{label2} that supports multiple label
pairings.}

\item{node_features}{A string vector indicating which node feature(s)
to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
and the function will perform summary statistics on all or no node features.}

\item{edge_features}{A string vector indicating which edge feature(s)
to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
and the function will perform summary statistics on all or no edge features.}

\item{test_name}{A string with the name of the test you are performing.}

\item{diminish}{A logical variable indicating whether to use
diminishing summary statistics;
if \code{TRUE}, a summary statistic for a node or edge will only be done if
at least one of its parent node or edge is significant. Otherwise, the test
will be performed on all nodes or edges.}

\item{p_thres}{A double indicating the summary statistic threshold;
if the result of a statistical test is greater than \code{p_thres}, then it
is insignificant.}

\item{p_rate}{A double; if \code{diminish=TRUE}, then \code{p_rate}
needs to be specified.
to determine whether or not a node or edge's parent is significant,
we use \code{p_thres}. However, the higher the layer on which a node
resides or to which an edge points to,
the less stringent this \code{p_thres} should be.
Therefore, we set \code{p_thres}
as the threshold for the parent node or edge of the last
layer and multiply \code{p_thres}
by \code{p_rate} for each increasing layer e.g. given
default values and 4 layers,
the thresholds for layers 1 through 4 would be .4, .2, .1, and .05.}

\item{test_custom}{A function or a string indicating the statistical test
to use. If a string is provided, it should be one of
\code{c("t","wilcox","ks","var","chisq")};
these correspond to statistical tests \code{stats::t.test},
\code{stats::wilcox.test}, and so on.
If a function is provided, it should take as input two numeric vectors and
output a numeric variable.}

\item{effect_size}{A logical variable indicating whether or not to calculate
effect size statistic (cohen's d) for this set of class labels;
later used for plotting.}

\item{adjust0}{A logical variable indicating whether or not to calculate the
minimum percentage of values from samples of each class label that falls
within the range of \code{adjust0_lim}. This is only done for SpecEnr values
as p-values become unstable when comparing near 0 values.}

\item{adjust0_lim}{A vector of two numeric values indicating a range around
0, default set to -0.1 and 0.1.}

\item{btwn}{A logical variable indicating whether or not to calculate the
\code{btwn} data frame given in the \code{fg_get_summary} function.}

\item{btwn_test_custom}{Same as \code{test_custom} but for \code{btwn}.}

\item{save_functions}{A logical variable indicating whether to save test and adjust functions.}

\item{overwrite}{A logical variable indicating whether to
overwrite the existing summary statistics if it exists.}
}
\value{
flowGraph object containing claculated summary statistics.
}
\description{
Calculates feature summary statistics for flowGraph features;
 users can choose from a list of statistical significance tests/adjustments
 or define custom summary functions.
 For special cases, see example in function
 \code{\link[flowGraph]{fg_add_summary}} on
 how to manually calculate summary statistics without using this function.
}
\details{
\code{fg_summary} calculates a summary statistic as specified
 by the user in parameters \code{test_name}, \code{diminish}
 (\code{p_thres}, \code{p_rate}), and \code{test_custom}.
 The test is done for a node or edge feature of interest within a given
 flowGraph object as specified by parameters \code{node_features},
 \code{edge_features}. It then returns information on the summary statistic
 inside the same flowGraph object and returns it to the user.
 See \code{\link[flowGraph]{flowGraph-class}} slot \code{summary}
 for details on the contents.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 fg_get_summary_desc(fg)

 fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                  overwrite=FALSE, test_name="t", diminish=FALSE,
                  node_features="count", edge_features="NONE")
 fg_get_summary_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_clear_summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_helpers.R
\name{tstr}
\alias{tstr}
\title{Formats time into string.}
\usage{
tstr(time)
}
\arguments{
\item{time}{A time variable of class \code{POSIXct}, \code{POSIXt}.}
}
\value{
Time formatted as a string; used in \code{time_output} function.
}
\description{
Formats time into a string HH:MM:SS given time zone.
}
\examples{
 # NOT EXPORTED
 flowGraph:::tstr(Sys.time())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_extract_phenotypes}
\alias{fg_extract_phenotypes}
\title{Extracts a set of phenotypes from a flowGraph object.}
\usage{
fg_extract_phenotypes(fg, phenotypes)
}
\arguments{
\item{fg}{flowGraph object.}

\item{phenotypes}{A string vector of phenotype or
cell population name labels.}
}
\value{
flowGraph object.
}
\description{
Extracts or removes a specified set of
 phenotypes from a flowGraph object.
}
\details{
The \code{summary} in \code{fg} will not be modified;
 we recommend users recalculate them.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 fg_get_feature_desc(fg0)

 fg <- fg_extract_phenotypes(fg0, fg_get_graph(fg0)$v$phenotype[1:10])
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_merge}}
 \code{\link[flowGraph]{fg_extract_samples}}
 \code{\link[flowGraph]{fg_merge_samples}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{ms_psig}
\alias{ms_psig}
\title{Determines which phenotypes are statistically significant}
\usage{
ms_psig(
  ms_,
  summary_pars,
  summary_adjust,
  test_cust,
  test_custom,
  lyrno,
  mp_,
  me_
)
}
\arguments{
\item{ms_}{sample x phenotype SpecEnr matrix}

\item{summary_pars}{See \code{flowGraphSubset}.}

\item{summary_adjust}{See \code{flowGraphSubset}.}

\item{test_cust}{Final significance test function.}

\item{test_custom}{Raw significance test function.}

\item{lyrno}{An integer indicating total number of layers in the cell
hierarchy including layer 0.}

\item{mp_}{sample x phenotype proportion matrix.}

\item{me_}{sample x phenotype expected proportion matrix.}
}
\value{
A logical vector the same length as the number of columns in
 \code{ms_} indicating whether or not each phenotype is significant;
 used only for the fast version of flowGraph to determine whether or not
 to keep testing the phenotypes' children.
}
\description{
Determines which phenotypes are statistically significant based
 on SpecEnr.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_flowgraph_plots.R
\name{fg_plot_pVSdiff}
\alias{fg_plot_pVSdiff}
\title{Creates a p value vs feature difference plot}
\usage{
fg_plot_pVSdiff(
  fg,
  type = "node",
  index = 1,
  summary_meta = NULL,
  adjust_custom = "byLayer",
  logged = TRUE,
  label_max = 5,
  p_thres = 0.05,
  filter_adjust0 = 1,
  filter_es = 0,
  filter_btwn_tpthres = 1,
  filter_btwn_es = 0,
  shiny_plot = FALSE,
  nodes_max = 30,
  main = NULL,
  interactive = FALSE,
  path = NULL
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for
'node' or 'edge'.}

\item{index}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{index} is an integer indicating the row in
 \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
 summary the user would like to retrieve.}

\item{summary_meta}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{summary_meta} is a list containing
 \code{feature} (feature name), \code{test_name} (summary statistic name),
 \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.}

\item{adjust_custom}{A function or a string indicating the
test adjustment method to use.
 If a string is provided, it should be one of
 \code{c("holm", "hochberg", "hommel",
 "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
 If a function is provided, it should take as input
 a numeric vector and output the
 same vector adjusted.}

\item{logged}{A logical indicating whether or not to log the summary
statistic p value.}

\item{label_max}{An integer indicating the maximum number of max difference
and/or min p value nodes/edges that should be labelled.}

\item{p_thres}{A numeric variable indicating a p-value threshold; a line will
be plotted at this threshold.}

\item{filter_adjust0}{A numeric variable indicating what percentage of
SpecEnr values compared (minimum) should be not close to 0.
Set to 1 to not conduct filtering.}

\item{filter_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant.}

\item{filter_btwn_tpthres}{A numeric variable between 0 and 1 indicating the
unadjusted T-test p-value threshold used to test whether the actual
and expected feature values used to calculate the specified SpecEnr
feature are significantly different for each sample class. Note this only
needs to be specified for SpecEnr features. Combined
with \code{filter_btwn_es}, we conduct three tests to understand if
there is an actual large difference between actual and expected features:
(1,2) T-test of significance between the actual and expected raw feature value
(e.g. proportion) for samples in each of the compared classes, (3) and the
T-test of significance between the differences of actual and
expected feature values of the two classes. If any two of the three tests
come out as insignificant, we set the p-value for the associated node/edge
to 1.}

\item{filter_btwn_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant -- see \code{filter_btwn_tpthres}.}

\item{shiny_plot}{A logical indicating whether this plot is made for shiny;
users don't need to change this.}

\item{nodes_max}{An integer indicating maximum number of nodes to plot;
this limit is set for interactive plots only.}

\item{main}{A string or the title of the plot; if left as \code{NULL},
a default title will be applied.}

\item{interactive}{A logical variable indicating whether the plot should be
an interactive plot; see package \code{ggiraph}.}

\item{path}{A string indicating the path to where the function should save
the plot; leave as \code{NULL} to not save the plot. Static plots are saved
as PNG.}
}
\value{
A static or interactive p value vs difference plot.
}
\description{
Creates a p value vs feature difference plot where the
 difference is that of the
 features of samples belonging to different classes corresponding
 to an existing summary statistic.
}
\details{
The interactive plot is made using the \code{ggiraph} package.
}
\examples{

 no_cores <- 1
 data(fg_data_pos2)
 fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
                 no_cores=no_cores)

 gp <- fg_plot_pVSdiff(fg, type="node", summary_meta=NULL,
                       adjust_custom="byLayer", index=1, label_max=10)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_plot}}
 \code{\link[flowGraph]{plot_gr}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_plot_qq}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_flowgraph_features.R
\name{fg_feat_edge_specenr}
\alias{fg_feat_edge_specenr}
\title{Generates the SpecEnr edge feature.}
\usage{
fg_feat_edge_specenr(fg, no_cores = 1, overwrite = FALSE)
}
\arguments{
\item{fg}{flowGraph object.}

\item{no_cores}{An integer indicating how many cores to parallelize on.}

\item{overwrite}{A logical variable indicating whether to
overwrite the existing proportion edge feature if it exists.}
}
\value{
flowGraph object containing the proportion edge feature.
}
\description{
Generates the SpecEnr edge feature and returns it
 inside the flowGraph object.
}
\details{
Given a flowGraph object, \code{fg_feat_edge_SpecEnr} returns the same
 flowGraph object with an additional SpecEnr and expected proportions
 \code{expect_prop} \code{edge}
 feature and its meta data. The expected proportions edge feature is
 calculated by taking the ratio of the child nodes' (e.g. A+B+)
 expected proportion value over
 its parent nodes' (e.g. A+) actual proportion value.
 The SpecEnr feature is the actual over expected proportion ratio, logged.
 The edge feature matrix has column names <from>_<to> e.g. A+_A+B+.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- fg_feat_edge_specenr(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_feat_node_prop}}
 \code{\link[flowGraph]{fg_feat_node_specenr}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_flowgraph_features.R
\name{fg_feat_mean_class}
\alias{fg_feat_mean_class}
\title{Normalizes all features for class.}
\usage{
fg_feat_mean_class(
  fg,
  class,
  no_cores = 1,
  node_features = NULL,
  edge_features = NULL
)
}
\arguments{
\item{fg}{PARAM_DESCRIPTION}

\item{class}{a column name in \code{fg_get_meta(fg)} indicating the
meta data that should be used as the class label of each sample
while conudcting normalization.}

\item{no_cores}{An integer indicating how many cores to parallelize on.}

\item{node_features}{A string vector indicating the node features to
perform normalization on; set as \code{NULL} to normalize all.}

\item{edge_features}{A string vector indicating the edge features to
perform normalization on; set as \code{NULL} to normalize all.}
}
\value{
A numeric matrix whose dimensions equate to that of the input
 and whose values are normalized per class.

flowGraph object with normalized features.
}
\description{
For each class label in column \code{class} of \code{meta},
 \code{fg_feat_mean_class}
 takes the column mean of the rows in the given feature matrices
 (as specified in \code{node_features} and \code{edge_features})
 associated with that class;
 it then takes the difference point by point between these means and
 the original rows for that class.

FUNCTION_DESCRIPTION
}
\details{
For all features in the given \code{flowGraph} object and for
 each class label in column \code{class} of \code{meta},
 \code{fg_feat_mean_class}. It takes the column mean of the rows in the given
 feature matrices
 (as specified in \code{node_features} and \code{edge_features})
 associated with that class;
 it then takes the difference point by point between these means and
 the original rows for that class.
 \code{fg_feat_mean_class}
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- fg_feat_mean_class(fg, class="class", node_features="count",
                        no_cores=no_cores)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_flowgraph_features.R
\name{fg_feat_cumsum}
\alias{fg_feat_cumsum}
\title{Converts cell counts into cumulated cell counts.}
\usage{
fg_feat_cumsum(fg, no_cores)
}
\arguments{
\item{fg}{flowGraph object.}

\item{no_cores}{An integer indicating how many cores to parallelize on.}
}
\value{
flowGraph object with cumulated counts.
}
\description{
Converts the cell counts in a flowGraph object into
 cumulated cell counts; this is optional and can be done only for there is
 more than one threshold for one or more markers.
 This should also only be ran when initializing a flowGraph object as
 converting back and forth is computationally expensive.
 If the user is interested in seeing non- and cumulated counts, we recommend
 keeping two flowGraph objects, one for each version.
 This function simply converts e.g. the count of A+ or A++ into
 the sum of count of A+, A++, and A+++ or A++, and A+++.
}
\details{
\code{fg_feat_cumsum} returns the given flowGraph object with an
 adjusted count feature. As in our example,
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- flowGraph:::fg_feat_cumsum(fg, no_cores=no_cores)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[Matrix]{Matrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_feature_desc}
\alias{fg_get_feature_desc}
\title{Retrieves and/or recalculates a feature description table.}
\usage{
fg_get_feature_desc(fg, re_calc = FALSE)
}
\arguments{
\item{fg}{flowGraph object.}

\item{re_calc}{A logical variable specifying whether or not a feature summary
should be re-calculated or directly retrieved from \code{fg}.}
}
\value{
A data frame where each row contains information on a feature
from the given flowGraph object; its columns is as in the \code{feat_desc}
slot of \code{\link[flowGraph]{flowGraph-class}}.
}
\description{
Retrieves and/or recalculates a feature
 description table for a given flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 fg_get_feature_desc(fg, re_calc=TRUE)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{ms_create}
\alias{ms_create}
\title{Calcuate SpecEnr from proportion and expected proportion}
\usage{
ms_create(mp_, me_)
}
\arguments{
\item{mp_}{Numerical sample x cell population matrix w/ proportions.}

\item{me_}{Numerical sample x cell population matrix w/ expected proportions.}
}
\value{
Numerical sample x cell population matrix w/ SpecEnr.
}
\description{
FUNCTION_DESCRIPTION
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_summary_index}
\alias{fg_get_summary_index}
\title{Retrieves the index of the requested summary.}
\usage{
fg_get_summary_index(fg, type = "node", index = NULL, summary_meta = NULL)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for
'node' or 'edge'.}

\item{index}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{index} is an integer indicating the row in
 \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
 summary the user would like to retrieve.}

\item{summary_meta}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{summary_meta} is a list containing
 \code{type} (feature type: \code{node} or \code{edge}),
 \code{feature} (feature name), \code{test_name} (summary statistic name),
 \code{class} (class), \code{lable1}, and \code{label2} (class labels compared).
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.}
}
\value{
An integer analagous to \code{index}.
 If both \code{index} and \code{summary_meta} are \code{NULL}, returns 1.
}
\description{
Retrieves the index of the requested summary
 from a given flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 # set features to NULL to apply summary statistic to all features.
 fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                  overwrite=FALSE, test_name="t", diminish=FALSE,
                  node_features=NULL, edge_features=NULL)
 show(fg)

 index <- flowGraph:::fg_get_summary_index(
  fg, type="node", summary_meta=list(
    feature="SpecEnr", test_name="t", class="class",
    label1="control", label2="exp"))

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_plot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_extract_samples}
\alias{fg_extract_samples}
\title{Extracts a set of samples from a flowGraph object.}
\usage{
fg_extract_samples(fg, sample_ids, rm_summary = TRUE)
}
\arguments{
\item{fg}{flowGraph object.}

\item{sample_ids}{A string vector of sample id's that the user wants to
keep in \code{fg}.}

\item{rm_summary}{A logical indicating whether or not to clear summary.}
}
\value{
flowGraph object.
}
\description{
Extracts or removes a specified set of samples from
 a flowGraph object.
}
\details{
The summaries in \code{fg} will not be modified;
 we recommend the user recalculates them.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 fg_get_feature_desc(fg0)

 fg <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[1:5])
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_merge}}
 \code{\link[flowGraph]{fg_extract_phenotypes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{test_c}
\alias{test_c}
\title{Converts input into a significance test function}
\usage{
test_c(test_custom)
}
\arguments{
\item{test_custom}{a string \code{c("t", "wilcox","ks","var","chisq")} or a
function.}
}
\value{
a statistical significance test function.
}
\description{
Converts input into a significance test function
}
\seealso{
\code{\link[stats]{t.test}},\code{\link[stats]{wilcox.test}},
 \code{\link[stats]{ks.test}},\code{\link[stats]{var.test}},
 \code{\link[stats]{chisq.test}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_data.R
\docType{data}
\name{fg_data_pos2}
\alias{fg_data_pos2}
\title{fg_data_pos2}
\format{
A list containing the following elements
 for a positive control data set with markers A, B, C, D. This is a positive
 control data set where node A+B+C+ increased by 50%.
\itemize{
  \item{\code{count}: A numeric sample x cell population node
  matrix with cell count values}
  \item{\code{meta}: A data frame containing meta information on
   samples in \code{count}; it contains columns:}
  \itemize{
      \item{\code{id}: a string containing sample id's.}
      \item{\code{class}: a string indicating whether a sample is
       from a "control" or "exp" (experiment) subject.}
  }
}
}
\usage{
fg_data_pos2
}
\description{
fg_data_pos2
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_rm_summary}
\alias{fg_rm_summary}
\title{Removes a feature summary.}
\usage{
fg_rm_summary(fg, type = "node", index = NULL, summary_meta = NULL)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for;
'node' or 'edge'.}

\item{index}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{index} is an integer indicating the row in
 \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
 summary the user would like to retrieve.}

\item{summary_meta}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{summary_meta} is a list containing
 \code{feat} (feature name), \code{test_name} (summary statistic name),
 \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.}
}
\value{
flowGraph object.
}
\description{
Removes a feature summary from a given flowGraph object;
 while \code{fg} is required, the user can choose to input parameters
 \code{summary_meta}, \code{index}, or all of \code{type},
 \code{feat}, \code{test_name}, \code{class}, \code{label1},
  and \code{label2}.
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                  overwrite=FALSE, test_name="wilcox_byLayer", diminish=FALSE,
                  node_features=NULL, edge_features=NULL)
 fg_get_summary_desc(fg)

 fg <- fg_rm_summary(fg, summary_meta=c(
     feature="count",test_name="wilcox_byLayer",
     class="class", label1="control", label2="exp"))
 fg_get_summary_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_rm_feature}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{extract_markers}
\alias{extract_markers}
\title{Extracts markers from cell population phenotypes}
\usage{
extract_markers(phen)
}
\arguments{
\item{phen}{A vector of cell population phenotypes.}
}
\value{
A vector of unique markers
}
\description{
Extracts all unique markers from cell population phenotypes
}
\seealso{
\code{\link[stringr]{str_split}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_merge}
\alias{fg_merge}
\title{Merges two flowGraph objects together.}
\usage{
fg_merge(
  fg1,
  fg2,
  method_sample = c("union", "intersect", "setdiff", "none"),
  method_phenotype = c("intersect", "setdiff", "none")
)
}
\arguments{
\item{fg1}{flowGraph object.}

\item{fg2}{flowGraph object.}

\item{method_sample}{A string indicating how samples from flowGraph objects
should be merged:
\itemize{
  \item{\code{union}: keep all samples from both flowGraph objects;
   in this case \code{method_phenotype} must be \code{intersect}.}
  \item{\code{intersect}: keep only samples that exist
   in both \code{fg1} and \code{fg2}.}
  \item{\code{setdiff}: keep only samples that exist
   in \code{fg1} and not in \code{fg2}.}
  \item{\code{none}: keep all samples in \code{fg1}.}
}}

\item{method_phenotype}{A string indicating how phenotypes from
flowGraph objects should be merged:
 \itemize{
  \item{\code{intersect}: keep only phenotypes that exist in both
   \code{fg1} and \code{fg2}.}
  \item{\code{setdiff}: keep only phenotypes that exist in
   \code{fg1} and not in \code{fg2}.}
  \item{\code{none}: keep all phenotypes in \code{fg1}.}
 }}
}
\value{
flowGraph object.
}
\description{
Merges two flowGraph objects together.
}
\details{
\code{fg_merge} is a generic function that merges the samples and
 phenotypes of two flowGraph objects.
 Note that if \code{method_sample="union"}
 then \code{method_phenotype} must be set to "intersect".
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg0 <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg1 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[1:5])
 fg2 <- fg_extract_samples(fg0, fg_get_meta(fg0)$id[4:7])
 fg <- fg_merge(fg1, fg2, method_sample="intersect",
                          method_phenotype="intersect")
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_extract_samples}}
 \code{\link[flowGraph]{fg_extract_phenotypes}}
 \code{\link[flowGraph]{fg_merge_samples}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_graph.R
\name{set_layout_graph}
\alias{set_layout_graph}
\title{Determines cell hierarchy layout.}
\usage{
set_layout_graph(gr, layout_fun = "layout.reingold.tilford")
}
\arguments{
\item{gr}{A list containing data frames \code{e} and \code{v}.}

\item{layout_fun}{A string of a function from the \code{igraph} package that
indicates what layout should be used if a cell hierarchy is to be ploted;
all such functions have prefix \code{layout_}
e.g. \code{layout_fun="layout.reingold.tilford"}.}
}
\value{
A list containing data frames \code{e} and \code{v}; each data frame
   contains an X, Y column or coordinate for each node and edge.
}
\description{
Determines cell hierarchy layout and returns the X, Y coordinate
   of each cell population.
}
\examples{

   no_cores <- 1
   data(fg_data_pos30)
   fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                                   prop=FALSE, specenr=FALSE,
                                   no_cores=no_cores)

   head(set_layout_graph(fg_get_graph(fg)))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_feature}
\alias{fg_get_feature}
\title{Retrieves a feature matrix.}
\usage{
fg_get_feature(fg, type = "node", feature = "count")
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type 'node' or 'edge'.}

\item{feature}{A string indicating feature name;}
}
\value{
A numeric matrix of the specified feature values.
}
\description{
Retrieves a feature matrix from a given flowGraph object,
 the feature type, and feature name.
}
\details{
Returns \code{NULL} if the requested feature does not exist.
}
\examples{

 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=1)

 feature_matrix <- fg_get_feature(fg, type='node', feature='count')

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_get_summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{fg_clean_phen}
\alias{fg_clean_phen}
\title{Reformats phenotype}
\usage{
fg_clean_phen(phen, markers = NULL)
}
\arguments{
\item{phen}{Vector of cell population phenotype names as character strings.}

\item{markers}{markers extracted from \code{phen}.}
}
\value{
Vector with the same length as \code{phen} containing reformatted
and not necessarily changed cell population phenotype names.
}
\description{
Reformats cell population phenotypes into flowGraph format
}
\examples{

# fg_clean_phen(c("A+_B+","B+_notC","A-_C"))

}
\seealso{
\code{\link[stringr]{str_extract}},\code{\link[stringr]{str_split}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_graph.R
\name{get_phen_list}
\alias{get_phen_list}
\title{Creates edge lists.}
\usage{
get_phen_list(meta_cell = NULL, phen = NULL, no_cores = 1)
}
\arguments{
\item{meta_cell}{A data frame containing meta data on cell populations
as produced by the \code{get_phen_meta} function.}

\item{phen}{A string vector of phenotype or cell population name labels.
Cannot be set to \code{NULL} if \code{meta_cell} is set to \code{NULL}.}

\item{no_cores}{An integer indicating how many cores to parallelize on.}
}
\value{
A list containing 'pchild', an edge list
   indicating where edges point to,
   'pparen', an edge list indicating where edges point from, and
   'edf', a data frame where each row contains
   the nodes an edge points 'from' and 'to'.
}
\description{
Creates edge lists indicating relationships
between cell populations
   given meta data on these cell populations
   produced by the \code{get_phen_meta} function.
}
\examples{

   phen <- c('A+B-C+', 'A+B-', 'A+')
   get_phen_list(phen=phen)

}
\seealso{
\code{\link[flowGraph]{get_phen_meta}}
   \code{\link[flowGraph]{cell_type_layers}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{flowGraphSubset_summary_adjust}
\alias{flowGraphSubset_summary_adjust}
\title{Default for flowGraphSubset's summary_adjust}
\usage{
flowGraphSubset_summary_adjust()
}
\value{
Default list parameter flowGraphSubset's \code{summary_adjust} parameter.
}
\description{
Default input for flowGraphSubset's \code{summary_adjust}
 parameter. ONLY USE THIS OVER flowGraph IF: 1) your data set has more than
 10,000 cell populations and you want to speed up your calculation time AND
 2) you only have one set of classes you want to test on the
 SAME SET OF SAMPLES (e.g. control vs experiment).
 As flowGraphSubset does not
 calculate the SpecEnr for all cell populations, so if you want to test other
 sets of classes on the same samples, you will not be able to test all
 possible cell populations on the new set of classes.
}
\examples{

 flowGraphSubset_summary_adjust()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_graph.R
\name{fg_set_layout}
\alias{fg_set_layout}
\title{Determines cell hierarchy layout.}
\usage{
fg_set_layout(fg, layout_fun = "layout.reingold.tilford")
}
\arguments{
\item{fg}{flowGraph object.}

\item{layout_fun}{A string version of a function name from
the \code{igraph} package that
indicates what layout should be used if a cell hierarchy is to be ploted;
all such functions have prefix \code{layout_}
e.g. \code{layout_fun="layout.reingold.tilford"}.}
}
\value{
flowGraph object with coordinate meta data on cell populations
   and edges for plotting use.
}
\description{
Determines cell hierarchy layout and returns the X, Y coordinate
   of each cell population. This function is a wrapper for
   \code{\link[flowGraph]{set_layout_graph}}.
}
\details{
Given a flowGraph object, modifies the \code{graph} slot such that
   it contains X, Y axes for each node in accordance to a user specified
   layout.
}
\examples{

   no_cores <- 1
   data(fg_data_pos30)
   fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                                   prop=FALSE, specenr=FALSE,
                                   no_cores=no_cores)

   fg <- fg_set_layout(fg)
   head(fg_get_graph(fg)$v)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_add_summary}
\alias{fg_add_summary}
\title{Adds a feature summary.}
\usage{
fg_add_summary(
  fg,
  type = "node",
  summary_meta = NULL,
  p = NULL,
  summ_fun = NULL,
  overwrite = FALSE,
  ...
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for;
'node' or 'edge'.}

\item{summary_meta}{The user must provide \code{type} and
 \code{summary_meta}.

 \code{summary_meta} is a list containing
 \code{feature} (feature name), \code{test_name} (summary statistic name),
 \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.}

\item{p}{A list containing summary values; this list contains elements:
\code{values} (a vector containing summary statistics e.g. p-values;
 this vector should be named by their associated phenotype or edge name),
\code{test_custom} (a function of the statistical test used), and
\code{adjust_custom} (a function of the p-value correction method used).
This list must contain the \code{values} element.}

\item{summ_fun}{A function that ouputs a feature summary matrix
as in \code{p} given \code{fg} and other optional parameters.}

\item{overwrite}{A logical variable indicating whether or not the function
should replace the existing feature summary with the
same name if one is already in \code{fg}.}

\item{...}{Other parameters that would be used as input into \code{summ_fun}.}
}
\value{
flowGraph object.
}
\description{
Adds a feature summary into a given flowGraph object.
 Only use this function if your summary statistic cannot be calcuated
 using the \code{\link[flowGraph]{fg_summary}} function.
}
\details{
\code{fg_add_summary} adds the given feature summary list \code{p}
 or the output of the given function \code{summ_fun} to the
 given flowGraph object \code{fg} updating slots
 \code{summary} and \code{summary_desc}.
 See \code{\link[flowGraph]{flowGraph-class}}
 slot \code{summary} and \code{summary_desc}
 for what should be in these slots. We do not recommend users directly use
 this function unless what is required is duly in the above slots is
 well understood --- note these slots are used in plotting functions
 e.g. \code{\link[flowGraph]{fg_plot}}. We instead recommend users to use
 the \code{\link[flowGraph]{fg_summary}} function.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 # get samples that we are going to compare
 m <- fg_get_feature(fg, type="node", feature="prop")
 m1_ <- m[fg_data_pos30$meta$class=="control",,drop=FALSE]
 m2_ <- m[fg_data_pos30$meta$class=="exp",,drop=FALSE]

 # define test or summary function to conduct comparison
 test_custom <- function(x,y)
     tryCatch(stats::t.test(x,y)$p.value, error=function(e) 1)
 values_p <- sapply(seq_len(ncol(m)), function(j)
     test_custom(m1_[,j], m2_[,j]) )
 values_p <- p.adjust(values_p , method="BY")

 # the user can choose to fill either parameter "p" or "summ_fun",
 # the latter of which must output a list with the same elements as "p".
 # see documentation for ?flowGraph-class, slot "summary" for
 # details on what should be in "p".
 p <- list(values=values_p, test_fun=test_custom, adjust_fun="BY")
 fg <- fg_add_summary(fg, type="node", summary_meta=list(
      feature="prop", test_name="wilcox_BY",
      class="class", label1="control", label2="exp"), p=p)

 fg_get_summary_desc(fg)


}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_summary}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_add_feature}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_summary_tables}
\alias{fg_get_summary_tables}
\title{Retrieves a table containing all node or edge summary statistics.}
\usage{
fg_get_summary_tables(fg, type = "node")
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summaries the user
wants to retrieve were created for, 'node' or 'edge'.}
}
\value{
A list; this output is the same as that of function
 \code{fg_get_graph} with additional columns.
 These columns contain summary statistics from the \code{summary}
 slot of the flowGraph object. These columns are named:
 <feature type: node/edge>.<feature>.<summary name>.<class>.<class labels>.
}
\description{
Retrieves a table containing all node or edge summary statistics
 given a flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 fg <- fg_summary(fg, no_cores=no_cores, class="class", label1="control",
                  overwrite=FALSE, test_name="t", diminish=FALSE)
 show(fg)

 feat_summ_table_node <- fg_get_summary_tables(fg, type="node")
 head(feat_summ_table_node)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_means}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_get_feature}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_flowgraph_plots.R
\name{fg_plot}
\alias{fg_plot}
\title{Creates a cell hierarchy plot.}
\usage{
fg_plot(
  fg,
  type = "node",
  index = 1,
  summary_meta = NULL,
  adjust_custom = "byLayer",
  show_nodes_edges = NULL,
  label_max = 30,
  p_thres = 0.05,
  filter_adjust0 = 1,
  filter_es = 0,
  filter_btwn_tpthres = 1,
  filter_btwn_es = 0,
  node_labels = c("prop", "expect_prop"),
  summary_fun = colMeans,
  layout_fun = NULL,
  show_bgedges = TRUE,
  main = NULL,
  interactive = FALSE,
  visNet_plot = TRUE,
  path = NULL,
  width = 9,
  height = 9
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{type}{A string indicating feature type the summary was created for
'node' or 'edge'.}

\item{index}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{index} is an integer indicating the row in
 \code{fg_get_summary_desc(<flowGraph>)} of the corresponding type and
 summary the user would like to retrieve.}

\item{summary_meta}{The user must provide \code{type} and
 additionally, one of \code{summary_meta} or \code{index}.

 \code{summary_meta} is a list containing
 \code{feature} (feature name), \code{test_name} (summary statistic name),
 \code{class} (class), \code{label1}, and \code{label2} (class labels compared).
 See \code{\link[flowGraph]{fg_get_summary_desc}} for details.}

\item{adjust_custom}{A function or a string indicating the
test adjustment method to use.
 If a string is provided, it should be one of
 \code{c("holm", "hochberg", "hommel",
 "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
 If a function is provided, it should take as input
 a numeric vector and output the
 same vector adjusted.}

\item{show_nodes_edges}{A logical vector indicating which nodes/edges (type)
to show in the plot; if this is not specified, only nodes/edges with
significant summary statistics will be shown.}

\item{label_max}{An integer specifying the maximum number of nodes to label.}

\item{p_thres}{A double indicating a summary statistic threshold
e.g. if we are plotting a T test summary statistic, we can set the threshold
to .05; nodes with a p-value greater than .05 will not be plotted.}

\item{filter_adjust0}{A numeric variable indicating what percentage of
SpecEnr values compared (minimum) should be not close to 0.
Set to 1 to not conduct filtering.}

\item{filter_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant.}

\item{filter_btwn_tpthres}{A numeric variable between 0 and 1 indicating the
unadjusted T-test p-value threshold used to test whether the actual
and expected feature values used to calculate the specified SpecEnr
feature are significantly different for each sample class. Note this only
needs to be specified for SpecEnr features. Combined
with \code{filter_btwn_es}, we conduct three tests to understand if
there is an actual large difference between actual and expected features:
(1,2) T-test of significance between the actual and expected raw feature value
(e.g. proportion) for samples in each of the compared classes, (3) and the
T-test of significance between the differences of actual and
expected feature values of the two classes. If any two of the three tests
come out as insignificant, we set the p-value for the associated node/edge
to 1.}

\item{filter_btwn_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant -- see \code{filter_btwn_tpthres}.}

\item{node_labels}{A string vector indicating which node feature(s)
should be used to label a node.
We recommend keeping the length of this vector to below 2.
Set to "NONE" if no p-value labels are needed.}

\item{summary_fun}{A function that takes in a matrix and outputs a
vector the same length as the number of columns this matrix has;
see \code{\link[flowGraph]{fg_summary}}.}

\item{layout_fun}{A string representing a function from the
\code{igraph} package that
indicates what layout should be used if a cell hierarchy is to be ploted;
all such functions have prefix \code{layout_}. Only specify if different
from the default one already calculated in the \code{fg} flowGraph object
given.}

\item{show_bgedges}{A logical variable indicating whether or not
edges not specified for plotting should be plotted as
light grey in the background.}

\item{main}{A string or the title of the plot; if left as \code{NULL},
a default title will be applied.}

\item{interactive}{A logical variable indicating whether the plot should be
an interactive plot; see package \code{ggiraph}.}

\item{visNet_plot}{A logical variable indicating if an interactive plot is
chosen, if function should output a visNetwork plot; if set to \code{FALSE},
ggplot's girafe will be used instead.}

\item{path}{A string indicating the path to where the function should save
the plot; leave as \code{NULL} to not save the plot. Static plots are saved
as PNG, interactive plots are saved as HTML.}

\item{width}{A numeric variable specifying, in inches,
what the plot width should be.}

\item{height}{A numeric variable specifying, in inches,
what the plot height should be.}
}
\value{
A list of nodes and edges for plotting with the \code{plot_gr}
 function. Other elements in this list include \code{show_bgedges},
 which has the same value as parameter \code{show_bgedges}, and \code{main},
 the title of the plot.
}
\description{
Creates a cell hierarchy plot given a flowGraph object. If a path is not provided for \code{fg_plot} to save the plot, please use \code{plot_gr} to view plot given the output of \code{fg_plot}.
}
\details{
\code{fg_plot} takes a flowGraph object as input and returns the
 \code{graph} slot of the given object with additional columns to serve as
 input into \code{\link[flowGraph]{plot_gr}} for plotting using functions
 in the \code{ggplot2} package. Users can choose to save a PNG version of
 the plot by filling out the \code{path} parameter with a full path to the
 PNG plot. In addition to specifying columns added from
 \code{\link[flowGraph]{ggdf}}, \code{fg_plot} also adds label column(s)
 whose values serve as labels in the interactive version of the plot.
}
\examples{

 no_cores <- 1
 data(fg_data_pos2)
 fg <- flowGraph(fg_data_pos2$count, class=fg_data_pos2$meta$class,
                 no_cores=no_cores)

 gr <- fg_plot(fg, type="node", index=1, label_max=30,
   show_nodes_edges=NULL, p_thres=.01, node_labels=c("prop", "expect_prop"),
   path=NULL) # set path to a full path to save plot as a PNG
 # plot_gr(gr)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{get_phen_meta}}
 \code{\link[flowGraph]{ggdf}}
 \code{\link[flowGraph]{plot_gr}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_get_summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_gsub_markers}
\alias{fg_gsub_markers}
\title{Replace marker names.}
\usage{
fg_gsub_markers(fg, markers_new, markers_old = NULL)
}
\arguments{
\item{fg}{flowGraph object.}

\item{markers_new}{A string vector of new marker names;
if \code{markers_old} is set to \code{NULL},
each marker in \code{markers_new} should correspond to
each marker in the \code{markers} slot of the \code{flowGraph} object.}

\item{markers_old}{A string vector of old marker names user wants to replace;
these marker names corresponding to those
in \code{fg_get_markers(fg)} with the same length as \code{markers_new}.
If \code{markers_old=NULL}, \code{markers_new} should be the same length as
\code{fg_get_markers(fg)}.}
}
\value{
flowGraph object with marker names replaced.
}
\description{
Replace marker names in a flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- fg_gsub_markers(fg, c("Anew", "Bnew", "Cnew", "Dnew"))
 fg_get_feature_desc(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_gsub_ids}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{flowGraphSubset_summary_pars}
\alias{flowGraphSubset_summary_pars}
\title{Default for flowGraphSubset's summary_pars}
\usage{
flowGraphSubset_summary_pars()
}
\value{
Default list parameter flowGraphSubset's \code{summary_pars} parameter.
}
\description{
Default input for flowGraphSubset's \code{summary_pars} parameter.
}
\examples{

 flowGraphSubset_summary_pars()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_helpers.R
\name{time_output}
\alias{time_output}
\title{Outputs elapsed time.}
\usage{
time_output(start, msg = "")
}
\arguments{
\item{start}{A time variable of class \code{POSIXct}, \code{POSIXt}.}

\item{msg}{A string with a message to print out after the elapsed time.}
}
\value{
Prints to console, the time from which process
 started \code{start} - ended, and > time elapsed from
 \code{start} until now.
}
\description{
Given a time, prints the time elapsed from that time until now.
}
\examples{

 start <- Sys.time()
 flowGraph:::time_output(start,'start - now > time elapsed')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_flowgraph_plots.R
\name{ggdf}
\alias{ggdf}
\title{Prepares a given node and edge graph list for plotting.}
\usage{
ggdf(gr0)
}
\arguments{
\item{gr0}{A list containing data frames \code{e} and \code{v}.}
}
\value{
A list containing data frames \code{e} and \code{v}, each
 with additional meta data column.
}
\description{
Prepares a given node and edge graph list
 for plotting by function plot_gr;
 do not use this function on its own.
}
\details{
code{ggdf} adds to the data frames \code{v} and \code{e} in slot
 \code{graph} from a \code{flowGraph} object specifying plotting options as
 required by \code{\link[flowGraph]{plot_gr}}:
 \itemize{
   \item{\code{v}}
   \itemize{
     \item{\code{size}: a numeric indicating node size.}
     \item{\code{colour}: a numeric or string indicating node colour.}
     \item{\code{label}: a string indicating the label of a node.}
     \item{\code{label_long}: a string indicating teh long label of a node;
      used in interactive plots in \code{\link[flowGraph]{plot_gr}}}.
     \item{\code{label_ind}: a vector of logical variables indicating which
      nodes to add a label to in a static plot.}
     \item{\code{v_ind}: a vector of logical variables indicating which
      nodes to plot.}
   }
   \item{\code{e}}
   \itemize{
     \item{\code{colour}: a numeric or string indicating edge colour.}
     \item{\code{e_ind}: a vector of logical variables indicating which
      edges to plot.}
   }
 }
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 gr_ <- ggdf(fg_get_graph(fg))
 head(gr_$v)
 head(gr_$e)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{get_phen_meta}}
 \code{\link[flowGraph]{plot_gr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowgraph_constructor.R
\name{flowGraph}
\alias{flowGraph}
\title{flowGraph object constructor.}
\usage{
flowGraph(
  input_,
  meta = NULL,
  class = "class",
  no_cores = 1,
  markers = NULL,
  layout_fun = "layout.reingold.tilford",
  max_layer = NULL,
  cumsumpos = FALSE,
  prop = TRUE,
  specenr = TRUE,
  path = NULL,
  calculate_summary = TRUE,
  node_features = "SpecEnr",
  edge_features = "NONE",
  test_name = "t_diminish",
  test_custom = "t",
  diminish = TRUE,
  label1 = NULL,
  label2 = NULL,
  save_plots = FALSE
)
}
\arguments{
\item{input_}{Any of the following:
\itemize{
  \item{a numeric matrix or vector of the cell counts;
  its column/names must be the phenotype names and its rownames
  must be sample ID's.}
}
All input samples should have the same \code{markers} and
\code{partitionsPerMarker}.}

\item{meta}{A data frame with meta data for each \code{Phenotypes} or sample;
One of its column names should be "id" whose values correspond to
the name of each \code{Phenotypes} object. We also recommend for it to have
a column named "class" where one of its unique values is "control".}

\item{class}{A string corresponding to the column name or index
of \code{meta} whose values represent
the class label of each sample; OR a vector the same length
as the the number of samples in \code{input_} specifiying the class
of each given sample --- this vector will be appended to \code{meta} under
column name \code{class}.}

\item{no_cores}{An integer indicating how many cores to parallelize on.}

\item{markers}{A string vector of marker names used in \code{input_}.}

\item{layout_fun}{A string of a function from the \code{igraph} package that
indicates what layout should be used if a cell hierarchy is to be ploted;
all such functions have prefix \code{layout_}. This is defaulted to
e.g. \code{layout_fun="layout.reingold.tilford"}.}

\item{max_layer}{And integer indicating the maximum layer in the cell
hierarchy to analyze; set to `NULL` to analyze all layers.}

\item{cumsumpos}{A logical variable indicating whether
or not to cumulate cell counts;
this applies only when \code{partitionsPerMarker > 3} and will convert
e.g. the count of A+ or A++ into the sum of the counts of
A+, A++, A+++, ..., or A++, A+++, ... .}

\item{prop}{A logical variable indicating whether or not to
calculate the proportion feature;
this can be done later on with \code{flowGraph_prop}.}

\item{specenr}{logical variable: whether or not to calculate
the SpecEnr feature, Default: T}

\item{path}{A string indicating the folder path to where the flowGraph
object should save its elements, Default = NULL (don't save).}

\item{calculate_summary}{A logical variable indicating whether or not to
calculate the summary statistics for SpecEnr based on default parameters
using the \code{fg_summary} summary function on class
specified in parameter \code{class}.}

\item{node_features}{A string vector indicating which node feature(s)
to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
and the function will perform summary statistics on all or no node features.}

\item{edge_features}{A string vector indicating which edge feature(s)
to perform summary statistics on; set to \code{NULL} or \code{"NONE"}
and the function will perform summary statistics on all or no edge features.}

\item{test_name}{A string with the name of the test you are performing.}

\item{test_custom}{See \code{\link[flowGraph]{fg_summary}}.}

\item{diminish}{A logical variable; applicable if \code{calculate_summary} is
\code{TRUE}; see \code{\link[flowGraph]{fg_summary}}.}

\item{label1}{A string indicating a class label in
\code{fg_get_meta(fg)[,class]}; set to \code{NULL} if you would
like to compare all classes aganst all classes;
applicable if \code{calculate_summary} is \code{TRUE}.}

\item{label2}{A string indicating a class label in
\code{fg_get_meta(fg)[,class]};
applicable if \code{calculate_summary} is \code{TRUE}.}

\item{save_plots}{A logical indicating whether or not to save plots.}
}
\value{
flowGraph object
}
\description{
Initializes a \code{flowGraph} object given the cell counts
 for one or more flow cytometry sample(s).
 The flowGraph object returned holds meta data
 for each sample, each cell population node,
 edges representing how each cell population node relate to one another,
 and features for these nodes and edges.
}
\details{
\code{flowGraph} is the constructor for the \code{flowGraph} object.
 The user can choose to input as \code{input_} a vector, a \code{Phenotypes}
 object (meaning there is only one sample), a matrix, or a \code{Phenotypes}
 object list. If the user is also inputting a sample meta data frame, it
 must contain a \code{id} column corresponding to sample names.
}
\examples{

no_cores <- 1
samplen <- 10
meta_file <- data.frame(
    id=1:samplen,
    class=append(rep("control", samplen/2), rep("exp", samplen/2)),
    stringsAsFactors=FALSE
)


## using the constructor -----------------------

data(fg_data_pos30)

# input: vector of load-able Phenotypes paths
fg <- flowGraph(fg_data_pos30$count[1,], no_cores=no_cores)

# input: matrix + vector of class corresponding to samples
fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                no_cores=no_cores)
# - save to file directly
# fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
#                no_cores=no_cores, path="path_to_folder)

# input: matrix + meta data frame
# fg <- flowGraph(fg_data_pos30$count, meta=fg_data_pos30$meta,
#                 no_cores=no_cores)


}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_get_summary_desc}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_add_summary}}
 \code{\link[flowGraph]{fg_rm_summary}}
 \code{\link[flowGraph]{fg_gsub_markers}}
 \code{\link[flowGraph]{fg_gsub_ids}}
 \code{\link[flowGraph]{fg_merge_samples}}
 \code{\link[flowGraph]{fg_extract_samples}}
 \code{\link[flowGraph]{fg_extract_phenotypes}}
 \code{\link[flowGraph]{fg_merge}}
 \code{\link[doParallel]{registerDoParallel}}
 \code{\link[Matrix]{Matrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_extract_raw}
\alias{fg_extract_raw}
\title{Clears all features and feature summaries in a flowGraph object.}
\usage{
fg_extract_raw(fg)
}
\arguments{
\item{fg}{flowGraph object.}
}
\value{
flowGraph object with all summary statistics and feature values
 removed except for the node count feature.
}
\description{
Returns a flowGraph object with only the \code{count} feature
 and meta data. This function clears all other features and
 feature summaries to save space.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 no_cores=no_cores)

 fg <- fg_extract_raw(fg)
 show(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_accessors.R
\name{fg_get_graph}
\alias{fg_get_graph}
\title{Retrieves a graph list from a given flowGraph object.}
\usage{
fg_get_graph(fg)
}
\arguments{
\item{fg}{flowGraph object.}
}
\value{
A list containing two data frames (\code{v} and ]code{e})
 from the \code{graph} slot of the given \code{flowGraph} object containing
 information on the cell populations phenotype nodes and edges representing
 relation between cell populations.
}
\description{
Retrieves a graph list from a given flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)
 gr <- fg_get_graph(fg)
 head(gr$v)
 head(gr$e)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_plot}}
 \code{\link[flowGraph]{ggdf}}
 \code{\link[flowGraph]{plot_gr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_flowgraph_features.R
\name{fg_feat_node_prop}
\alias{fg_feat_node_prop}
\title{Generates the proportion node feature.}
\usage{
fg_feat_node_prop(fg, overwrite = FALSE)
}
\arguments{
\item{fg}{flowGraph object.}

\item{overwrite}{A logical variable indicating whether to
overwrite the existing proportion node feature if it exists.}
}
\value{
flowGraph object containing the proportion node feature.
}
\description{
Generates the proportion node feature and returns it
 inside the returned flowGraph object.
}
\details{
Given a flowGraph object, \code{fg_feat_node_prop} returns the same
 flowGraph object, inside of which is an additional proportions \code{prop}
 \code{node} feature
 and its meta data. The proportions feature is made using the node count
 feature and is the cell count of each cell population over the total
 cell count.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- fg_feat_node_prop(fg)

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_feat_node_specenr}}
 \code{\link[flowGraph]{fg_add_feature}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_rm_feature}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_flowgraph_plots.R
\name{fg_save_plots}
\alias{fg_save_plots}
\title{Saves numerous plots for all summary statistics to a folder.}
\usage{
fg_save_plots(
  fg,
  plot_path,
  plot_types = "node",
  interactive = FALSE,
  adjust_custom = "byLayer",
  label_max = 10,
  box_no = 20,
  paired = FALSE,
  logged = FALSE,
  filter_adjust0 = 1,
  filter_es = 0,
  filter_btwn_tpthres = 1,
  filter_btwn_es = 0,
  overwrite = TRUE,
  node_labels = "NONE",
  ...
)
}
\arguments{
\item{fg}{flowGraph object.}

\item{plot_path}{A string indicating the folder path to where the function
should save the plots.}

\item{plot_types}{A string or a vector of strings indicating what feature
types and their summaries the function should plot for: 'node' or 'edge'.}

\item{interactive}{A logical indicating whether the QQ plot, p-value vs
difference plot, and the
cell hierarchy plots should be interactive; see functions
\code{fg_plot} and \code{fg_plot_qq}.}

\item{adjust_custom}{A function or a string indicating the
test adjustment method to use.
 If a string is provided, it should be one of
 \code{c("holm", "hochberg", "hommel",
 "bonferroni", "BH", "BY", "fdr", "none")} (see \code{p.adjust.methods}).
 If a function is provided, it should take as input
 a numeric vector and output the
 same vector adjusted.}

\item{label_max}{An integer indicating how many labels should be shown
in the functions \code{fg_plot_pVSdiff} and \code{fg_plot}.}

\item{box_no}{An integer indicating the maximum number of boxplots to save;
used in function \code{fg_plot_box}.}

\item{paired}{A logical indicating whether the summary is paired; used in
function \code{fg_plot_box}.}

\item{logged}{A logical indicating whether or not to log the summary
statistic p value in the qq plots.}

\item{filter_adjust0}{A numeric variable indicating what percentage of
SpecEnr values compared (minimum) should be not close to 0.
Set to 1 to not conduct filtering. This parameter is used for the QQ and the
pVSdifference plots.}

\item{filter_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant.}

\item{filter_btwn_tpthres}{A numeric variable between 0 and 1 indicating the
unadjusted T-test p-value threshold used to test whether the actual
and expected feature values used to calculate the specified SpecEnr
feature are significantly different for each sample class. Note this only
needs to be specified for SpecEnr features. Combined
with \code{filter_btwn_es}, we conduct three tests to understand if
there is an actual large difference between actual and expected features:
(1,2) T-test of significance between the actual and expected raw feature value
(e.g. proportion) for samples in each of the compared classes, (3) and the
T-test of significance between the differences of actual and
expected feature values of the two classes. If any two of the three tests
come out as insignificant, we set the p-value for the associated node/edge
to 1.}

\item{filter_btwn_es}{A numeric variable between 0 and 1 indicating what the
Cohen's D value of the nodes/edges in question must be greater or
equal to, to be significant -- see \code{filter_btwn_tpthres}.}

\item{overwrite}{A logical variable indicating whether or not to replace
old plots if they exist under the same folder name.}

\item{node_labels}{Parameter for the \code{fg_plot} function.}

\item{...}{Other parameters for the \code{fg_plot} function.}
}
\value{
No return; plots are saved to file.
}
\description{
Saves numerous plots for all summary statistics in a given
 flowGraph object to a user specified folder.
}
\details{
The interactive plots are made using the \code{ggiraph} package.
}
\examples{

 no_cores <- 1
 data(fg_data_pos2)
 fg <- flowGraph(fg_data_pos2$count,
                 class=fg_data_pos2$meta$class,
                 no_cores=no_cores)

 fg_save_plots(fg, "temp")

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_plot}}
 \code{\link[flowGraph]{plot_gr}}
 \code{\link[flowGraph]{fg_get_feature}}
 \code{\link[flowGraph]{fg_get_summary}}
 \code{\link[flowGraph]{fg_plot_qq}}
 \code{\link[flowGraph]{fg_plot_pVSdiff}}
 \code{\link[flowGraph]{fg_plot_box}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowGraphSubset_constructor.R
\name{get_paren}
\alias{get_paren}
\title{Gets parent populations of given cell populations}
\usage{
get_paren(childs, pparen, pc__i, ac_, meta_cell_)
}
\arguments{
\item{childs}{Character vector of cell population phenotypes.}

\item{pparen}{Edge list where the name of the list is the cell population
and the vector in each element contains the parent cell populations of the
named cell population.}

\item{pc__i}{A cell population x marker matrix where the values are 0/1/2/...
correspondng to marker conditions /-/+/...;
this is for the possible CHILD cell populations.}

\item{ac_}{A list where the elements are marker index > "0"/"1"/"2"/... >
a logical vector the same length as the number of cell population phenotypes
indicating whether or not the marker condition exists in them;
this is for the possible PARENT cell populations.}

\item{meta_cell_}{data frame with meta data for cell population phenotypes
from the flowGraph object; this is for the possible PARENT cell populations.}
}
\value{
A list containing parent populations of \code{childs}; also globally
 updates \code{pparen}.
}
\description{
Gets the parent populations of a vector of given cell
 \code{childs} and updates \code{pparen} the edge list if edge list doesn't
 contain the requested information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_data.R
\docType{data}
\name{fg_data_pos30}
\alias{fg_data_pos30}
\title{fg_data_pos30}
\format{
A list containing the following elements
 for a positive control data set with
 markers A, B, C, D; note it was made with two and three thresholds
 for markers A and B to test functions with multiple thresholds
 (this is a positive
 control data set where nodes A+..B+..C+ increased by 50%).
\itemize{
  \item{\code{count}: A numeric sample x cell population node
  matrix with cell count values}
  \item{\code{meta}: A data frame containing meta information on
  samples in \code{count}; it contains columns:}
  \itemize{
      \item{\code{id}: a string containing sample id's.}
      \item{\code{class}: a string indicating whether a sample is
       from a "control" or "exp" (experiment) subject.}
  }
}
}
\usage{
fg_data_pos30
}
\description{
fg_data_pos30
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_flowgraph_modifiers.R
\name{fg_gsub_ids}
\alias{fg_gsub_ids}
\title{Replace sample id's.}
\usage{
fg_gsub_ids(fg, ids_new, ids_old = NULL)
}
\arguments{
\item{fg}{flowGraph object.}

\item{ids_new}{A string vector of new sample id's; if \code{ids_old} is
set to \code{NULL}, each id in \code{ids_new} should correspond to
each id in \code{fg_get_meta(fg)$id}.}

\item{ids_old}{A string vector of old sample id's the user wants to replace;
these marker names corresponding to those
in \code{fg_get_meta(fg)$id} with the same length as \code{ids_new}.
If \code{ids_old=NULL}, \code{ids_new} should be the same length as
\code{fg_get_meta(fg)$id}.}
}
\value{
flowGraph object with sample id's replaced.
}
\description{
Replace sample id's in a flowGraph object.
}
\examples{

 no_cores <- 1
 data(fg_data_pos30)
 fg <- flowGraph(fg_data_pos30$count, class=fg_data_pos30$meta$class,
                 prop=FALSE, specenr=FALSE,
                 no_cores=no_cores)

 fg <- fg_gsub_ids(fg, ids_new=paste0(fg_get_meta(fg)$id, "_new"))

}
\seealso{
\code{\link[flowGraph]{flowGraph-class}}
 \code{\link[flowGraph]{fg_get_feature_desc}}
 \code{\link[flowGraph]{fg_gsub_markers}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_flowgraph_class.R
\docType{class}
\name{flowGraph-class}
\alias{flowGraph-class}
\alias{show,flowGraph-method}
\title{'flowGraph': A class for storing cell count feature values
 for the Phenotype class.}
\usage{
\S4method{show}{flowGraph}(object)
}
\arguments{
\item{object}{A flowGraph object.}
}
\value{
a flowGraph object.
}
\description{
'flowGraph': A class for storing cell count feature values
 for the Phenotype class.
}
\section{Methods (by generic)}{
\itemize{
\item \code{show}: show method
}}

\section{Slots}{

\describe{
\item{\code{feat}}{A list containing elements \code{node} and \code{edge},
each containing a list with feature values;
  each element in this list is named by the feature name and contains a
  numeric matrix with the sample id's as row names and cell populations
  phenotype labels or edge labels as column names. Column names
  for edge features are labelled as <from>_<to> e.g. A+_A+B+.}

\item{\code{feat_desc}}{A list containing elements \code{node} and \code{edge},
each containing a data frame describing the features in the \code{feat} slot
with columns:
\itemize{
  \item{\code{feat}: feature name.}
  \item{\code{nrow}: number of samples.}
  \item{\code{ncol}: number of nodes or edges.}
  \item{\code{inf}: number of infinite values in the matrix.}
  \item{\code{neginf}: number of negative infinite values in the matrix.}
  \item{\code{na}: number of NA values in the matrix.}
  \item{\code{nan}: number of NaN values in the matrix.}
  \item{\code{neg}: number of negative values in the matrix.}
  \item{\code{pos}: number of positive values in the matrix.}
  \item{\code{zero}: number of 0's in the matrix.}
  \item{\code{max}: The maximum value in the matrix.}
  \item{\code{min}: The minimum value in the matrix.}
}}

\item{\code{summary}}{A list containing elements \code{node} and \code{edge},
 each containing a list with a feature summary list;
 each feature summary in this list contains elements:
\itemize{
  \item{\code{values}: a numeric vector the same length
   as the number of nodes or edges.}
  \item{test_custom: a function or a string name
   of the summary test method used.}
}}

\item{\code{summary_desc}}{A list containing elements \code{node} and \code{edge},
 each containing a data frame describing the features in \code{feat} with
 columns:
\itemize{
  \item{\code{feat}: A string indicating feature name the
   summary was created for.}
  \item{\code{test_name}: A string containing the name of the summary.}
  \item{\code{class}: A string corresponding to the column name
   of the \code{meta} slot whose values represent the class label of
   each sample on which the summary was created to compare or analyze.}
   \item{\code{label1}: A string from the \code{class} column of the
    \code{meta} slot indicating one of the labels compared to create
    the summary statistic.}
   \item{\code{label2}: A string from the \code{class} column of the
    \code{meta} slot indicating one of the labels compared to create
    the summary statistic.}
}}

\item{\code{meta}}{A data frame containing the column(s) \code{id} (sample id's
corresponding to row names of features in the \code{feat} slot) and any
other meta data pertaining to samples being analyzed.}

\item{\code{markers}}{A character vector containing markers used.}

\item{\code{edge_list}}{A list containing elements \code{child} and \code{parent}.
These elements contain an edge list from child to parent and vice versa.}

\item{\code{graph}}{A list containing data frames \code{v} and \code{e} with
information on cell population nodes and edges. \code{v} contains columns:
\itemize{
  \item{\code{phenotype}: The cell population node label names e.g. A+B+C+.}
  \item{\code{phenocode}: A string of "0", "1", "2", ... indicating the
   whether each marker is expressed on a cell population.}
  \item{\code{phenolayer}: The layer on which a cell population resides i.e.
   the numer of markers in its phenotype label.}
  \item{\code{phenogroup}: The markers used the make up the phenotype.}
}}

\item{\code{plot_layout}}{A string indicating the name of the \code{igraph} layout
function used to layout the cell population nodes for plotting.}

\item{\code{etc}}{A list containing other information
(see \code{\link[flowGraph]{fg_get_summary}} for other things stored
in this slot):
\itemize{
  \item{\code{cumsumpos}: A logical indicating whether cell counts
   in \code{flowGraph} object contains cumulated cell counts;
   this is optional and can be done only for there is
   more than one threshold for one or more markers.
   This should also only be ran when initializing a \code{flowGraph} object
   as converting back and forth is computationally expensive.
   If the user is interested in seeing non- and cumulated counts,
   we recommend keeping two \code{flowGraph} objects, one for each version.
   This function simply converts e.g. the count of A+ or A++ into
   the sum of count of A+, A++, and A+++ or A++, and A+++.}
  \item{\code{class_mean_normalized}: A logical indicating whether the
   features in the flowGraph object has been normalized according to
   some sample meta e.g. subject.}
   \item{\code{save}: A list containing a string indicating the save ID
    of the object
and a string indicating path where the object is saved -- used in function
\code{save_fg} to identify whether or not to save to the same folder.}
}}
}}

\section{Creating Objects}{
 Objects can be created using
\code{new("flowFrame")} or the constructor \code{flowGraph},
with mandatory argument \code{input_}.
Creating objects using \code{new} is discouraged.
}

\section{Methods}{
 'object' represents a \code{flowGraph} object.
\itemize{
  \item{\code{show(fg)}: Shows a description of the \code{flowGraph} object.}
  \item{\code{fg_get_meta}: Retrieves the sample meta data
   from a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_get_meta}}.}
  \item{\code{fg_get_graph}: Retrieves the cell population (v) and edge (e)
   meta data from a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_get_graph}}.}
  \item{\code{fg_get_feature}: Retrieves the numeric feature matrix requested
   by the user from a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_get_feature}}.}
  \item{\code{fg_get_summary}: Retrieves the feature summary list requested
   by the user from a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_get_summary}}.}
  \item{\code{fg_get_feature_desc}: Retrieves the data frame from the
   \code{feat_desc} slot of a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_get_feature_desc}}.}
  \item{\code{fg_get_summary_desc}: Retrieves the data frame
   from the \code{summary_desc}
   slot of a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_get_summary_desc}}.}
  \item{\code{fg_add_feature}: Adds a feature to a given
   \code{flowGraph} object;
   we do not recommend users directly use this method, instead please use
   wrapper functions e.g. \code{\link[flowGraph]{fg_feat_node_prop}},
   \code{\link[flowGraph]{fg_feat_node_specenr}},
   See \code{\link[flowGraph]{fg_add_feature}}.}
  \item{\code{fg_rm_feature}: Removes a user specified feature
   from a given flowGraph object.
   See \code{\link[flowGraph]{fg_rm_feature}}.}
  \item{\code{fg_add_summary}: Adds a feature to a given \code{flowGraph}
   object;
   we do not recommend users directly use this method, instead please use
   wrapper function \code{\link[flowGraph]{fg_summary}}.}
  \item{\code{fg_clear_summary}: Removes all feature summaries
   from a given flowGraph object.
   See \code{\link[flowGraph]{fg_clear_summary}}.}
  \item{\code{fg_rm_summary}: Removes a user specified feature summaries
   from a given flowGraph object.
   See \code{\link[flowGraph]{fg_rm_summary}}.}
  \item{\code{fg_gsub_markers}:
   Substitutes marker names in a given \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_gsub_markers}}.}
  \item{\code{fg_gsub_ids}: substitutes sample id's in a \code{flowGraph}
   object
   See \code{\link[flowGraph]{fg_gsub_ids}}.}
  \item{\code{fg_merge_samples}: Merges the samples of two \code{flowGraph}
   objects;
   we recomment users use the wrapper function
    \code{\link[flowGraph]{fg_merge}} instead.
   See \code{\link[flowGraph]{fg_merge_samples}}.}
  \item{\code{fg_extract_samples}: Extract data for specific samples
   from a flowGraph object.
   See \code{\link[flowGraph]{fg_extract_samples}}.}
  \item{\code{fg_extract_phenotypes}: Extract data for specific
   cell population nodes from a \code{flowGraph} object.
   See \code{\link[flowGraph]{fg_extract_phenotypes}}.}
  \item{\code{fg_merge}: Merges two given \code{flowGraph} objects.
   See \code{\link[flowGraph]{fg_merge}}.}
  \item{\code{fg_set_layout}: Sets layout for cell population nodes
   for the purpose of plotting.
   See \code{\link[flowGraph]{fg_set_layout}}.}
   \item{\code{fg_plot}: Plots cell hierarchies in the flowGraph object.
   See \code{\link[flowGraph]{fg_plot}}.}
}
}

\examples{

 showClass("flowGraph")
}
